package br.usp.talkagent.ide;

/**
 * Ontology Tree GUI
 * @author Percival Lucena 
 */

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.util.*;
import br.usp.talkagent.kb.*;
import br.usp.talkagent.kb.objects.*;

public class OntologyTree 
{
	private JTree ontologyTree;
	public DefaultTreeModel treeModel;
	private XktClient xktc;
	
	public OntologyTree(XktClient xktc)
	{
		ontologyTree = new JTree();
		this.xktc = xktc;
	}
	
	public DefaultMutableTreeNode nodeAddChildren(DefaultMutableTreeNode parent, DefaultMutableTreeNode child)
	{
		parent.add(child);
		return child;		
	}
	
	
	public DefaultMutableTreeNode createNewNode(String name) 
	{
		return new DynamicTreeNode(new NodeData(null, Color.black, name));
	}
	
	
	public DefaultMutableTreeNode createConceptNode(String name)
	{
		NodeData ndata = new NodeData(null, Color.BLACK, name);
		Icon ic = new ImageIcon("../ide/images/concept.gif");		
		ndata.setIcon(ic);
		DynamicTreeNode nd = new DynamicTreeNode(ndata);
		return nd;
	}

	public DefaultMutableTreeNode createComponentNode(String name)
	{
		
		NodeData ndata = new NodeData(null, Color.GRAY, name);
		Icon ic = new ImageIcon("../ide/images/component.gif");		
		ndata.setIcon(ic);
		DynamicTreeNode nd = new DynamicTreeNode(ndata);
		return nd;
	}
		
	public DefaultMutableTreeNode newAssertionNode(String name) 
	{
		return new DynamicTreeNode(new NodeData(null, Color.red, name));
	}
	
	public DefaultMutableTreeNode newComponentNode(String name)
	{
		return new DynamicTreeNode(new NodeData(null, Color.green, name));
	}
	
	
   /**
	* Returns the TreeNode instance that is selected in the tree.
	* If nothing is selected, null is returned.
	*/
	public DefaultMutableTreeNode getSelectedNode() 
	{
	TreePath  selPath = this.ontologyTree.getSelectionPath();
	if(selPath != null)
	    return (DefaultMutableTreeNode)selPath.getLastPathComponent();
	return null;
	}
	

	public DefaultMutableTreeNode lastItem()
	{
		TreePath  selPath = this.ontologyTree.getSelectionPath();	
		if(selPath!=null)
		{   
			DefaultMutableTreeNode LastItem = (DefaultMutableTreeNode)selPath.getLastPathComponent();
			return LastItem;
		}
		return null;
	}
	
	/**
	 * Returns the ontologyTree.
	 * @return JTree
	 */
	public JTree getTree() {
		return ontologyTree;
	}

	/**
	 * Sets the ontologyTree.
	 * @param ontologyTree The ontologyTree to set
	 */
	public void setTree(JTree ontologyTree) {
		this.ontologyTree = ontologyTree;
	}


	public void buildTree()
	{
		/* Create the JTreeModel. */
		Vector index = xktc.getIndex();
		int i;
		String nd;
		NodeData data;
		DynamicTreeNode newNode;
		String item;
		int cdn;
		ozConceptImpl idx;
		
		DefaultMutableTreeNode root = createNewNode("ontologies");
		System.err.println("IDE - Loading Main Ontology");
		System.err.println(index.size() + " categories");
		for(i=0;i<index.size();i++)
		{
		 	idx = ((ozConceptImpl)index.elementAt(i));
		 	item = idx.getID();
		 	cdn = Integer.parseInt(xktc.getNumberOfChildren(item));		 		 
		 	nd =  cdn + " items";
 		 	data = new NodeData(Color.black, item, nd);					    
		 	newNode = new DynamicTreeNode(data);	   
		 	root.add(newNode);
		}	
		treeModel = new SampleTreeModel(root);
		/* Create the tree. */
		this.ontologyTree = new JTree(treeModel);
		/* Enable tool tips for the tree, without this tool tips will not
	   	be picked up. */
		//ToolTipManager.sharedInstance().registerComponent(tree);
		/* Make the tree use an instance of SampleTreeCellRenderer for
	   	drawing. */
		this.ontologyTree.setCellRenderer(new SampleTreeCellRenderer());
		/* Make tree ask for the height of each row. */
		this.ontologyTree.setRowHeight(-1);
}




}
