// Copyright 1997-1999 by softwarebuero m&b (SMB). All rights reserved.
//
// You can redistribute this software and/or modify it under the terms of
// the Ozone Library License version 1 published by softwarebuero m&b (SMB).
//
// $Id: Cattell.java,v 1.1 2002/11/27 16:30:45 cvs Exp $

import java.io.*;
import java.util.Random;
import java.util.Date;
import org.ozoneDB.DxLib.*;

import com.objy.db.*;
import com.objy.db.app.*;


public class Cattell extends ooObj implements Runnable {
    static String bootFileName = "bootfile";
    static String dbName = "cattellDB";
    static String contName = "cattellCont";
    static String rootName = "Cattell";
    static int runningThreads = 0;
    
    protected transient Session session = null;
    protected transient ooFDObj daFD = null;
    protected transient ooDBObj cattellDB = null;
    protected transient String[] args = new String[0];
    protected transient String id = "";
    
    
    public static void main( String[] args ) throws Exception {
        if (args.length == 0) {
            System.out.println( "usage: java CattellImpl [-u<count>] <test> <size> [<name>] [<host>|local]" );
            System.out.println( "   test: 0: deletes the database" );
            System.out.println( "         1: create database; size: 100..(10000)" );
            System.out.println( "         2: lookup; size 1.." );
            System.out.println( "         3: traversal; size: 1..7" );
            System.out.println( "         4: insert; size: 1.." );
            System.out.println( "         5: insert; size: 1.. (without delete)" );
            System.out.println( "         6: insert size*1000 objects" );
            System.out.println( "         7: traversal on the first size cattell objects" );
            System.out.println( "         8: insert on the first size cattell objects" );
            System.out.println( "   size: the size of the test" );
            System.out.println( "   name: optional name of the benchmark object" );
            System.out.println( " server: optional name of the server host" );
            System.exit( 1 );
        } 
        
        int users = 1;
        if (args[0].startsWith( "-u" )) {
            users = new Integer( args[0].substring( 2 ) ).intValue();
            String[] help = new String[args.length - 1];
            System.arraycopy( args, 1, help, 0, help.length );
            args = help;
        } 
        
        com.objy.db.app.Connection connection = com.objy.db.app.Connection.open( bootFileName, oo.openReadWrite );
        connection.setThreadPolicy( oo.THREAD_POLICY_RESTRICTED );
        
        new Cattell( args, "" ).run();
        /*
      Thread.currentThread().setPriority (Thread.MAX_PRIORITY);
      for (int i=0; i<users; i++) {
         Thread thread = new Thread (new Cattell (args, "" + i));
         thread.setPriority (Thread.MIN_PRIORITY);
         thread.start();
         }

      runningThreads = users;
      while (runningThreads > 0) 
         Thread.currentThread().sleep (1000);
         */
        connection.close();
    } 
    
    
    /**
     */
    public Cattell( String[] a, String i ) {
        args = a;
        id = i;
    }
    
    
    /**
     */
    public void run() {
        try {
            String name = args.length > 2 ? args[2] : "cattell";
            debugMess( "benchmark object name: " + name );
            initSession();
            
            //datenbank loeschen
            if (args[0].equals( "0" )) {
                session = new Session();
                session.begin();
                
                daFD = session.getFD();
                if (daFD.hasDB( dbName )) {
                    cattellDB = daFD.lookupDB( dbName );
                    cattellDB.delete();
                } 
                
                session.commit();
                debugMess( "Deleted database: " + dbName );
            } else if (args[0].equals( "1" )) {
                
                //datenbank bauen
                int n = Integer.parseInt( args[1] );
                debugMess( "creating..." );
                
                session = new Session();
                session.begin();
                
                daFD = session.getFD();
                if (daFD.hasDB( dbName )) {
                    cattellDB = daFD.lookupDB( dbName );
                } else {
                    try {
                        cattellDB = daFD.newDB( dbName );
                    } catch (LockNotGrantedException e) {
                        session.abort();
                        return;
                    } 
                    debugMess( "Created database: " + dbName );
                } 
                
                long start = System.currentTimeMillis();
                
                Cattell cattell = new Cattell();
                cattell.addParts( new Integer( n ) );
                cattellDB.bind( cattell, rootName );
                
                session.commit();
                debugMess( "build time: " + (System.currentTimeMillis() - start) + "msec" );
            } else if (args[0].equals( "2" )) {
                
                //lookup benchmark
                for (int i = 0; i < 10; i++) {
                    long start = System.currentTimeMillis();
                    //session = new Session();
                    
                    session.begin();
                    Cattell cattell = (Cattell)cattellDB.lookup( rootName );
                    cattell.lookup( new Integer( Integer.parseInt( args[1] ) ) );
                    session.commit();
                    
                    debugMess( "lookup time " + (System.currentTimeMillis() - start) + "msec" );
                } 
            } else if (args[0].equals( "3" )) {
                
                //traversal benchmark
                for (int i = 0; i < 10; i++) {
                    long start = System.currentTimeMillis();
                    session.begin();
                    
                    Cattell cattell = (Cattell)cattellDB.lookup( rootName );
                    cattell.traversal( new Integer( Integer.parseInt( args[1] ) ) );
                    session.commit();
                    
                    debugMess( "traversal time " + (System.currentTimeMillis() - start) + "msec" );
                } 
            } else if (args[0].equals( "4" )) {
                
                //insert benchmark
                for (int i = 0; i < 10; i++) {
                    long start = System.currentTimeMillis();
                    session.begin();
                    
                    Cattell cattell = (Cattell)cattellDB.lookup( rootName );
                    cattell.insert( new Integer( Integer.parseInt( args[1] ) ) );
                    session.commit();
                    debugMess( "insert time " + (System.currentTimeMillis() - start) + "msec" );
                    
                    session.begin();
                    cattell = (Cattell)cattellDB.lookup( rootName );
                    cattell.delete( new Integer( Integer.parseInt( args[1] ) ) );
                    session.commit();
                } 
            } else if (args[0].equals( "5" )) {
                
                //insert (ohne delete)
                long start = System.currentTimeMillis();
                session.begin();
                Cattell cattell = (Cattell)cattellDB.lookup( rootName );
                cattell.insert( new Integer( args[1] ) );
                session.commit();
                debugMess( "insert time " + (System.currentTimeMillis() - start) + "msec" );
            } else if (args[0].equals( "6" )) {
                
                //multi insert
                int n = Integer.parseInt( args[1] );
                for (int i = 0; i < n; i++) {
                    debugMess( "name: " + name + String.valueOf( i ) );
                    long start = System.currentTimeMillis();
                    session.begin();
                    
                    String contName = name + "Cont" + String.valueOf( i );
                    ooGCContObj cont = null;
                    if (cattellDB.hasContainer( contName )) {
                        cont = (ooGCContObj)cattellDB.lookupContainer( contName );
                    } else {
                        try {
                            cont = new ooGCContObj();
                            cattellDB.addContainer( cont, contName, 0, 5, 30 );
                        } catch (LockNotGrantedException e) {
                            session.abort();
                            return;
                        } 
                    } 
                    
                    Cattell cattell = new Cattell();
                    cont.cluster( cattell );
                    cattellDB.bind( cattell, name + String.valueOf( i ) );
                    cattell.addParts( new Integer( 1000 ) );
                    session.commit();
                    debugMess( "build time: " + (System.currentTimeMillis() - start) + " msec" );
                } 
            } else if (args[0].equals( "7" )) {
                
                //multi benchmark
                int n = Integer.parseInt( args[1] );
                Random rand = new Random();
                for (;;) {
                    int i = (int)(rand.nextDouble() * n);
                    debugMess( "name: " + name + String.valueOf( i ) );
                    
                    try {
                        session.setWaitOption( oo.WAIT );
                        session.begin();
                        Cattell cattell = (Cattell)cattellDB.lookup( name + String.valueOf( i ) );
                        if (cattell != null) {
                            long start = System.currentTimeMillis();
                            //cattell.traversal (new Integer(6));
                            cattell.lookup( new Integer( 100 ) );
                            debugMess( "traversal time " + (System.currentTimeMillis() - start) + "msec" );
                        } 
                        session.commit();
                    } catch (Exception e) {
                        session.abort();
                        debugMess( "Transaction aborted: " + e );
                        e.printStackTrace();
                    } 
                } 
            } else {
                
                //multi benchmark
                if (args[0].equals( "8" )) {
                    int n = Integer.parseInt( args[1] );
                    Random rand = new Random();
                    for (;;) {
                        int i = (int)(rand.nextDouble() * n);
                        debugMess( "name: " + name + String.valueOf( i ) );
                        
                        try {
                            session.setWaitOption( oo.WAIT );
                            session.begin();
                            Cattell cattell = (Cattell)cattellDB.lookup( name + String.valueOf( i ) );
                            if (cattell != null) {
                                long start = System.currentTimeMillis();
                                cattell.insert( new Integer( 100 ) );
                                cattell.delete( new Integer( 100 ) );
                                debugMess( "time " + (System.currentTimeMillis() - start) + "msec" );
                            } 
                            session.commit();
                        } catch (Exception e) {
                            session.abort();
                            debugMess( "Transaction aborted: " + e );
                            e.printStackTrace();
                        } 
                    } 
                } 
            } 
            Cattell.runningThreads--;
        } catch (Exception e) {
            Cattell.runningThreads--;
            e.printStackTrace();
        } 
    } 
    
    
    /** */
    public void debugMess( String msg ) {
        System.out.println( (id.equals( "" ) ? "" : id + ": ") + msg );
    } 
    
    
    /** */
    public synchronized boolean initSession() throws Exception {
        try {
            session = new Session();
            session.begin();
            daFD = session.getFD();
            if (daFD.hasDB( dbName )) {
                cattellDB = daFD.lookupDB( dbName );
            } 
            session.commit();
        } catch (Exception e) {
            e.printStackTrace();
        } 
        
        return cattellDB != null;
    } 
    
    
    /** */
    public synchronized void createDatabase( String name ) throws Exception {
        session.begin();
        try {
            cattellDB = daFD.newDB( dbName );
        } catch (LockNotGrantedException e) {
            session.abort();
            return;
        } 
        session.commit();
        System.out.println( "Created database: " + dbName );
    } 
    
    protected Part[] parts = new Part[0];
    protected int size = 0;
    
    
    /**
     */
    public Cattell() {
        fetch();
    }
    
    
    /**
     */
    public void addParts( Integer num ) throws Exception {
        markModified();
        int n = num.intValue();
        int oldSize = size;
        size += n;
        
        System.out.println( "adding " + n + " to " + oldSize + " old parts..." );
        
        System.out.println( "copying the array..." );
        Part[] newArray = new Part[size + n];
        System.arraycopy( parts, 0, newArray, 0, oldSize );
        parts = newArray;
        
        System.out.println( "creating parts..." );
        for (int i = oldSize; i < size; i++) {
            parts[i] = new Part();
        } 
        
        System.out.println( "connecting parts..." );
        Random rand = new Random();
        for (int i = oldSize; i < size; i++) {
            if (i % 200 == 0) {
                System.out.println( "" + i );
            } 
            for (int j = 0; j < 3; j++) {
                int cpart;
                if (rand.nextDouble() > 0.1) {
                    cpart = (i + (int)(rand.nextDouble() * size / 100) - size / 200);
                    if (cpart < size / 200) {
                        cpart += size / 200;
                    } 
                    if (cpart > (size - size / 200)) {
                        cpart -= size / 200;
                    } 
                } else {
                    cpart = (int)(rand.nextDouble() * size);
                } 
                // System.out.println ("connecting " + i + ":" + cpart);
                parts[i].connect( new Integer( j ), parts[cpart] );
            } 
        } 
    } 
    
    
    /**
     */
    public void lookup( Integer num ) throws Exception {
        fetch();
        
        int n = num.intValue();
        System.out.println( "" + n + " lookups..." );
        Random rand = new Random();
        for (int i = 0; i < n; i++) {
            int j = (int)(rand.nextDouble() * size);
            //parts[j].set ("part", new Integer(1), new Integer(1));
            parts[j].get();
        } 
    } 
    
    
    /**
     */
    public void traversal( Integer num ) throws Exception {
        fetch();
        
        System.out.println( "traversal (" + num.toString() + " hops)..." );
        Random rand = new Random();
        int j = (int)(rand.nextDouble() * size);
        parts[j].traversal( new Integer( 2 ), new Integer( 2 ), num );
    } 
    
    
    /**
     */
    public void insert( Integer num ) throws Exception {
        markModified();
        
        int n = num.intValue();
        int oldSize = size;
        System.out.println( "inserting " + n + " parts..." );
        addParts( num );
        System.out.println( "invoke..." );
        for (int i = oldSize; i < size; i++) {
            parts[i].set( "part", new Integer( 1 ), new Integer( 1 ) );
        } 
    } 
    
    
    /**
     */
    public void delete( Integer num ) throws Exception {
        markModified();
        
        int n = num.intValue() != -1 ? num.intValue() : size;
        System.out.println( "deleting " + n + " of " + size + " parts..." );
        for (int i = size - n; i < size; i++) {
            parts[i].deconnect();
            parts[i] = null;
        } 
        
        size -= n;
        Part[] newArray = new Part[size];
        System.arraycopy( parts, 0, newArray, 0, size );
        parts = newArray;
    } 
}
