// Copyright 1997-1999 by softwarebuero m&b (SMB). All rights reserved.
//
// You can redistribute this software and/or modify it under the terms of
// the Ozone Library License version 1 published by softwarebuero m&b (SMB).
//
// $Id: Cattell.java,v 1.1 2002/11/27 16:30:45 cvs Exp $

import java.io.*;
import java.util.Random;
import java.util.Date;
import org.ozoneDB.DxLib.*;
import COM.odi.*;


public class Cattell {
    static String DbName = "db.odb";
    
    
    public static void main( String[] args ) throws Exception {
        ObjectStore.initialize( null, null );
        
        //datenbank bauen
        if (args[0].equals( "1" )) {
            int n = Integer.parseInt( args[1] );
            System.out.println( "creating..." );
            Database db = Database.create( DbName, Database.allRead | Database.allWrite );
            
            Transaction ta = Transaction.begin( Transaction.update );
            long start = System.currentTimeMillis();
            Cattell cattell = new Cattell();
            cattell.addParts( new Integer( n ) );
            db.createRoot( "Cattell", cattell );
            ta.commit();
            System.out.println( "build time: " + (System.currentTimeMillis() - start) + "msec" );
            db.close();
        } 
        
        //lookup benchmark
        if (args[0].equals( "2" )) {
            Database db = Database.open( DbName, Database.openUpdate );
            for (int i = 0; i < 10; i++) {
                long start = System.currentTimeMillis();
                Transaction ta = Transaction.begin( Transaction.readOnly );
                Cattell cattell = (Cattell)db.getRoot( "Cattell" );
                cattell.lookup( new Integer( Integer.parseInt( args[1] ) ) );
                ta.commit();
                System.out.println( "lookup time " + (System.currentTimeMillis() - start) + "msec" );
            } 
            db.close();
        } 
        
        //traversal benchmark
        if (args[0].equals( "3" )) {
            Database db = Database.open( DbName, Database.openUpdate );
            for (int i = 0; i < 10; i++) {
                long start = System.currentTimeMillis();
                Transaction ta = Transaction.begin( Transaction.readOnly );
                Cattell cattell = (Cattell)db.getRoot( "Cattell" );
                cattell.traversal( new Integer( Integer.parseInt( args[1] ) ) );
                ta.commit();
                System.out.println( "traversal time " + (System.currentTimeMillis() - start) + "msec" );
            } 
            db.close();
        } 
        
        //insert benchmark
        if (args[0].equals( "4" )) {
            Database db = Database.open( DbName, Database.openUpdate );
            for (int i = 0; i < 10; i++) {
                long start = System.currentTimeMillis();
                Transaction ta = Transaction.begin( Transaction.update );
                Cattell cattell = (Cattell)db.getRoot( "Cattell" );
                cattell.insert( new Integer( Integer.parseInt( args[1] ) ) );
                ta.commit();
                System.out.println( "insert time " + (System.currentTimeMillis() - start) + "msec" );
                
                ta = Transaction.begin( Transaction.update );
                cattell = (Cattell)db.getRoot( "Cattell" );
                cattell.delete( new Integer( Integer.parseInt( args[1] ) ) );
                ta.commit();
            } 
            db.close();
        } 
    } 
    
    Part[] parts = new Part[0];
    int size = 0;
    
    
    public Cattell() {
    }
    
    
    /**
     */
    public void addParts( Integer num ) throws Exception {
        int n = num.intValue();
        int oldSize = size;
        size += n;
        
        System.out.println( "adding " + n + " to " + oldSize + " old parts..." );
        
        System.out.println( "copying the array..." );
        Part[] newArray = new Part[size + n];
        System.arraycopy( parts, 0, newArray, 0, oldSize );
        parts = newArray;
        
        System.out.println( "creating parts..." );
        for (int i = oldSize; i < size; i++) {
            parts[i] = new Part();
        } 
        
        System.out.println( "connecting parts..." );
        Random rand = new Random();
        for (int i = oldSize; i < size; i++) {
            if (i % 200 == 0) {
                System.out.println( "" + i );
            } 
            for (int j = 0; j < 3; j++) {
                int cpart;
                if (rand.nextDouble() > 0.1) {
                    cpart = (i + (int)(rand.nextDouble() * size / 100) - size / 200);
                    if (cpart < size / 200) {
                        cpart += size / 200;
                    } 
                    if (cpart > (size - size / 200)) {
                        cpart -= size / 200;
                    } 
                } else {
                    cpart = (int)(rand.nextDouble() * size);
                } 
                // System.out.println ("connecting " + i + ":" + cpart);
                parts[i].connect( new Integer( j ), parts[cpart] );
            } 
        } 
    } 
    
    
    /**
     */
    public void lookup( Integer num ) throws Exception {
        int n = num.intValue();
        System.out.println( "" + n + " lookups..." );
        Random rand = new Random();
        for (int i = 0; i < n; i++) {
            int j = (int)(rand.nextDouble() * size);
            parts[j].set( "part", new Integer( 1 ), new Integer( 1 ) );
        } 
    } 
    
    
    /**
     */
    public void traversal( Integer num ) throws Exception {
        System.out.println( "traversal (" + num.toString() + " hops)..." );
        Random rand = new Random();
        int j = (int)(rand.nextDouble() * size);
        parts[j].traversal( new Integer( 2 ), new Integer( 2 ), num );
    } 
    
    
    /**
     */
    public void insert( Integer num ) throws Exception {
        int n = num.intValue();
        int oldSize = size;
        System.out.println( "inserting " + n + " parts..." );
        addParts( num );
        System.out.println( "invoke..." );
        for (int i = oldSize; i < size; i++) {
            parts[i].set( "part", new Integer( 1 ), new Integer( 1 ) );
        } 
    } 
    
    
    /**
     */
    public void delete( Integer num ) throws Exception {
        int n = num.intValue() != -1 ? num.intValue() : size;
        
        System.out.println( "deleting " + n + " of " + size + " parts..." );
        for (int i = size - n; i < size; i++) {
            parts[i].deconnect();
            parts[i] = null;
        } 
        
        size -= n;
        Part[] newArray = new Part[size];
        System.arraycopy( parts, 0, newArray, 0, size );
        parts = newArray;
    } 
    
    /**
     */
    /*
   public Cattell copy() {
      Cattell obj = new Cattell();
      obj.size = size;
      obj.parts = new Part[size];
      for (int i=0; i<size; i++)
         obj.parts[i] = (Part)parts[i].copy();
      return obj;
      }
     */
}
