// $Id: Client.java,v 1.1 2002/11/27 16:30:48 cvs Exp $

import org.ozoneDB.*;
import java.net.*;


/**
 * Simple client program that accesses an object in a remote or local database
 * server.<p>
 * 
 * <i> Note: Do not use this sample to do any benchmarking if you are new to
 * ozone! Use the OO1 sample for that. The code of OO1 is more complex and shows
 * how to use ozone for 'real-world' applications. </i>
 * 
 * 
 * @author <a href="http://www.softwarebuero.de/">SMB</a>
 * @version $Revision: 1.1 $Date: 2002/11/27 16:30:48 $
 */
class Client {
    
    public static void main( String[] args ) throws Exception {
        
        ExternalDatabase db = ExternalDatabase.openDatabase( "ozonedb:remote://localhost:3333" );
        
        // this is just another way to open the same connection
        // db.open ("localhost", 3333);
        
        // this would initialize a local database instead of accessing a
        // server remote
        // LocalDatabase db = new LocalDatabase();
        // db.open ("/tmp/db");
        
        db.reloadClasses();
        System.out.println( "connected..." );
        
        Bus bus = (Bus)db.createObject( BusImpl.class.getName() );
        System.out.println( "bus:" + bus );
        bus.print();
        db.deleteObject( bus );
        
        Auto auto = (Auto)db.objectForName( "auto" );
        if (auto == null) {
            auto = (Auto)db.createObject( AutoImpl.class.getName(), OzoneInterface.Public, "auto" );
        } 
        
        // doSomething() returns the database object itself. However, in the
        // client we get a proper proxy object.
        Auto auto2 = (Auto)auto.doSomthing( auto );
        
        System.out.println( "auto:" + auto );
        auto.setAge( new Integer( 3 ) );
        System.out.println( "auto.setAge (3)" );
        // auto and auto2 are proxies of the same object.
        System.out.println( "auto2:" + auto2 );
        
        Auto copy = (Auto)db.copyObject( auto );
        copy.setAge( new Integer( 100 ) );
        System.out.println( "\nauto:" + auto );
        System.out.println( "copy:" + copy );
        
        db.deleteObject( copy );
        
        db.close();
        System.out.println( "deconnected..." );
    } 
    
}
