package de.cinek.rssview;

import java.io.*;
import java.util.*;

public class IniFile {

    private static final boolean VERBOSE=false;
    Hashtable section=new Hashtable();
    String inifilename=null;

    public IniFile(String filename) {
	inifilename=filename;
    }

    public boolean readSettings() {
	if (inifilename==null) return false;
	try {
	    if (VERBOSE) {
		System.err.println("IniFile: opening '"+inifilename+"'.");
	    }
	    BufferedReader iniread=new BufferedReader
		(new FileReader(inifilename));
	    String currentsection=null;
	    while (true) {
		String line=iniread.readLine();
		if (line==null) break;
		line=line.trim();
		if (line.startsWith("#") || line.startsWith(";") ||
		    line.length()<1) continue;
		
		if (line.startsWith("[")) {
		    if (line.endsWith("]")) {
			line=line.substring
			    (1,line.length()-1).trim();
			if (!section.containsKey(line)) {
			    if (VERBOSE) {
				System.err.println
				    ("IniFile: section ["+line+"] added.");
			    }
			    section.put(line,new Hashtable());
		    }
			currentsection=line;
		    }
		} else {
		    if (currentsection!=null) {
			int i=line.indexOf('=');
			if (i>=0) {
			    String key=line.substring(0,i).trim();
			    String value=line.substring(i+1).trim();
			    if (VERBOSE) {
				System.err.println
				    ("IniFile: ["+currentsection+"] "+
				     key+"='"+value+"'");
			    }
			    setString(currentsection,key,value);
			}
		    }
		}
	    }
	    iniread.close();
	    if (VERBOSE) {
		System.err.println("IniFile: closed.");
	    }
	} catch (IOException e) {
	    return false;
	}
	return true;
    }

    public void setString(String sec,String key,String value) {
	if (sec==null) return;

	Hashtable keys=(Hashtable)section.get(sec);
	if (keys==null) {
	    keys=new Hashtable();
	    section.put(sec,keys);
	}

	keys.put(key,value);
    }
    
    public void setInt(String sec,String key,int value) {
	setString(sec,key,String.valueOf(value));
    }
    
    public String getString(String sec,String key,String def) {
	Hashtable keys=(Hashtable)section.get(sec);
	if (keys==null) return def;
	String str=(String)keys.get(key);
	return str!=null ? str : def;
    }

    public int getInt(String sec,String key,int def) {
	String str=getString(sec,key,null);
	if (str==null) return def;
	return Integer.parseInt(str);
    }

    public boolean getBool(String sec,String key,boolean def) {
	String str=getString(sec,key,null);
	if (str==null) return def;
	str=str.toLowerCase();
	return str.equals("y") || str.equals("yes") || str.equals("1")
	    || str.equals("on") || str.equals("enabled");
    }

    public void setBool(String sec,String key,boolean b) {
	setString(sec,key,b ? "yes" : "no");
    }

    /* save procedure */
    public boolean saveSettings() {
	if (inifilename==null) return false;

	try {
	    PrintWriter ini=new PrintWriter(new FileWriter(inifilename));
	    
	    ini.println("\n# This is a initialisation file for rssview.");
	    ini.println("# Be careful! Everything (including keys and\n"+
			"# section names) is CASE SENSITIVE!");
	    
	    if (section!=null) {
		Enumeration sections=section.keys();
		while (sections.hasMoreElements()) {
		    String seckey=(String)sections.nextElement();
		    ini.println("\n["+seckey+"]");
		    Hashtable sec=(Hashtable)section.get(seckey);
		    if (sec!=null) {
			Enumeration e=sec.keys();
			while (e.hasMoreElements()) {
			    String key=(String)e.nextElement();
			    ini.println(key+"="+sec.get(key));
			}
		    }
		}
	    }
	    
	    ini.close();
	} catch (IOException e) {
	    return false;
	}
	return true;
    }
}
