package de.cinek.rssview;

import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.net.*;

public class RssChannel {

    private static long BEEP_MININTERVAL=30*1000;
    private static boolean shownerror=false;

    private String name;
    private String url;
    private Vector channelarticles;
    private boolean hasnews;
    private ImageIcon icon;
    private String title;
    private String description;
    private String link;
    private ImageIcon image;
    private int articlecount;
    private int pollinterval;
    private RssFetchThread fetchthread;
    private String searchtitle;
    private String searchdescription;
    private String searchquery;
    private boolean showtext;
    private int currentcount;
    private long lastbeeptime;
    private boolean beep;

    public RssChannel(String name,String url,int articlecount,
		      int pollinterval,boolean showtext,boolean beep) {
	this.name= (name==null) ? null : name.trim();
	this.url= (url==null) ? null : url.trim();
	this.articlecount=articlecount;
	this.pollinterval=pollinterval;
	this.showtext=showtext;
	this.beep=beep;
	channelarticles=new Vector();
	icon=null;
	hasnews=false;
	title=null;
	description=null;
	link=null;
	image=null;
	searchtitle=null;
	searchdescription=null;
	searchquery=null;
	lastbeeptime=System.currentTimeMillis();
	currentcount=0;

	if (url!=null) {
	    fetchthread=new RssFetchThread(this);
	    fetchthread.start();
	}
    }

    public void update() {
	fetchNews();
    }
    public void cleanUp() {
	fetchthread.stopFetch();
    }

    public String getName() {
	return name;
    }
    public void setName(String name) {
	this.name=name;
    }
    public String getURL() {
	return url;
    }
    public void setURL(String url) {
	this.url=url;
    }
    public boolean getShowText() {
	return showtext;
    }
    public void setShowText(boolean showtext) {
	this.showtext=showtext;
    }
    public boolean getBeep() {
	return beep;
    }
    public void setBeep(boolean beep) {
	this.beep=beep;
    }
    public ImageIcon getIcon() {
	return icon;
    }
    public String getTitle() {
	return title;
    }
    public String getLink() {
	return link;
    }
    public int getArticlesCount() {
	return articlecount;
    }
    public void setArticlesCount(int a) {
	articlecount=a;
    }
    public int getPollInterval() {
	return pollinterval;
    }
    public void setPollInterval(int i) {
	pollinterval=i;
    }

    public int getCurrentCount() {
	return currentcount;
    }
    public void setCurrentCount(int a) {
	currentcount=a;
    }

    public void setTextInput(String searchtitle,String searchdescription,
			     String searchname,String searchurl) {
	this.searchtitle=searchtitle==null ? "" : searchtitle.trim();
	this.searchdescription=searchdescription==null ? "" : 
	    searchdescription.trim();
	if (searchurl==null) searchquery=null;
	else {
	    if (searchname==null) searchname="";
	    searchquery=searchurl.trim()+"?"+searchname.trim()+"=";
	}
    }
    public String getTextInputTitle() {
	return searchtitle;
    }
    public String getTextInputDescription() {
	return searchdescription;
    }
    public String getTextInputQuery() {
	return searchquery;
    }

    public String getDescription() {
	return description;
    }
    public ImageIcon getImage() {
	return image;
    }
    public boolean isGlobal() {
	return url==null;
    }
    public boolean hasNews() {
	return isGlobal() ? RssArticleContainer.hasNews() : hasnews;
    }
    public void flushNews() {
	if (isGlobal()) RssArticleContainer.flushNews();
	else hasnews=false;
    }
    public void fetchNews() {
	
	if (url!=null) {

	    try {
		RssParser parser=new RssParser(this);
		parser.parse();
	    } 
	    catch (NoClassDefFoundError ex) {

		if (!shownerror) {
		    shownerror=true;
		    JOptionPane.showMessageDialog
			(null,
			 "Couldn't find/initialize my XML parser.\n"+
			 "Are You sure, You've read INSTALL-file?",
			 "Error",
			 JOptionPane.ERROR_MESSAGE);
		}
	    }
	}
    }
    public synchronized boolean addArticle(RssArticle a) {
	if (RssArticleContainer.addArticle(a)) {
	    //System.err.println(this.toString()+": Add article no. "+
	    //channelarticles.size());
	    channelarticles.add(0,a);
	    expireOldArticles();
	    hasnews=true;
	    return true;
	}
	return false;
    }
    public void addArticles(Vector v) {

	boolean beepallowed=false;

	if (v==null) return;
	for (int i=0;i<v.size();i++)
	    beepallowed|=addArticle((RssArticle)v.elementAt(i));

	if (beepallowed) doBeep();
    }

    public void doBeep() {
	if (!beep) return;

	long timenow=System.currentTimeMillis();
	if ((timenow-lastbeeptime)<BEEP_MININTERVAL) return;

	lastbeeptime=timenow;
	Toolkit.getDefaultToolkit().beep();
    }

    public void expireOldArticles() {

	/* should be safe, if someone is
	   not playing around with the
	   ticker-site */

	if (articlecount<=0) return;

	int safesize=
	    currentcount > articlecount ?
	    currentcount * 3 :
	    articlecount * 3;

	if (channelarticles.size()<=safesize) return;

	do {
	    int last=channelarticles.size()-1;
	    //System.err.println("*** "+this.toString()+": article no. "+
	    //last+" ready to expire");
	    RssArticle article=(RssArticle)channelarticles.elementAt(last);
	    channelarticles.removeElementAt(last);
	    RssArticleContainer.removeArticle(article);
	} while (channelarticles.size()>safesize);
    }

    public void setIcon(String url) {
	//System.err.println("SETTING Icon: '"+url+"'");
	try {
	    setIconFromImageIcon(new ImageIcon(new URL(url.trim())));
	}
	catch (Exception ex) {
	}
    }
    public void setIconFromImageIcon(ImageIcon image) {
	//System.err.println("SETTING ImageIcon: '"+image+"'");
	this.image=image;
	Image img=image.getImage().
	    getScaledInstance(16,16,Image.SCALE_DEFAULT);
	icon=new ImageIcon(img);
    }
    public void setTitle(String t) {
	title= (t==null) ? null : t.trim();
    }
    public void setLink(String l) {
	link= (l==null) ? null : l.trim();
    }
    public void setDescription(String d) {
	description= (d==null) ? null : d.trim();
    }

    public synchronized RssArticle[] getLatestArticles(int count) {
	if (isGlobal()) return RssArticleContainer.getLatestArticles(count);
	if (count<0 || count>channelarticles.size()) 
	    count=channelarticles.size();

	if (count<=0) return new RssArticle[0];

	return (RssArticle[])
	    channelarticles.subList(0,count).toArray(new RssArticle[0]);
    }

    public String toString() {
	return getName();
    }
    public boolean equals(RssChannel c) {
	return getName().equals(c.getName());
    }
}
