package de.cinek.rssview;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import de.cinek.rssview.images.IconContainer;

public class RssChannelDialog extends JDialog {

    private JTextField tf_name;
    private JTextField tf_url;
    private JTextField tf_articles;
    private JTextField tf_pollinterval;
    private JCheckBox cb_showtext;
    private JCheckBox cb_beep;

    private RssChannelList list;
    private RssChannel channel;

    public RssChannelDialog(RssView parent,RssChannelList list) {
	super(parent,"Add a new channel",true);
	this.list=list;
	this.channel=null;
	initComponents(null);
	pack();
	Dimension d=Toolkit.getDefaultToolkit().getScreenSize();
	setLocation(d.width/2,d.height/2);
	show();
    }

    public RssChannelDialog(RssView parent,RssChannelList list,String url) {
	super(parent,"Add a new channel",true);
	this.list=list;
	this.channel=null;
	initComponents(url);
	pack();
	Dimension d=Toolkit.getDefaultToolkit().getScreenSize();
	setLocation(d.width/2,d.height/2);
	show();
    }

    public RssChannelDialog(RssView parent,RssChannelList list,
			    RssChannel channel) {
	super(parent,"Modify a channel",true);
	this.list=list;
	this.channel=channel;
	initComponents(null);
	pack();
	Dimension d=Toolkit.getDefaultToolkit().getScreenSize();
	setLocation(d.width/2,d.height/2);
	show();
    }

    private void initComponents(String url) {

	InsetsPanel ip=new InsetsPanel();
	getContentPane().add(ip);

	GridBagLayout gbl=new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();

	JPanel widgetpanel=new JPanel(gbl);
	ip.add(widgetpanel,BorderLayout.CENTER);

	gbc.fill = GridBagConstraints.BOTH;
	gbc.weightx = 1.0;
	gbc.gridheight = 5;
	gbc.weighty = 1.0;
	gbc.ipadx=10;
	JLabel imglabel=new JLabel(channel==null ? 
				   IconContainer.ICON_STAR :
				   IconContainer.ICON_STARTOOL);
	gbl.setConstraints(imglabel,gbc);
	widgetpanel.add(imglabel);

	gbc.gridheight = 1;
	JLabel lab=new JLabel("Channel name:");
	gbl.setConstraints(lab,gbc);
	widgetpanel.add(lab);
	
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	tf_name=new JTextField(20);
	gbl.setConstraints(tf_name,gbc);
	widgetpanel.add(tf_name);

	gbc.gridwidth = 1;
	lab=new JLabel("Channel URL:");
	gbl.setConstraints(lab,gbc);
	widgetpanel.add(lab);
	
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	tf_url=(url==null) ? new JTextField(20) : new JTextField(url,20);
	gbl.setConstraints(tf_url,gbc);
	widgetpanel.add(tf_url);

	gbc.gridwidth = 1;
	lab=new JLabel("View articles:");
	gbl.setConstraints(lab,gbc);
	widgetpanel.add(lab);
	
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	tf_articles=new JTextField
	    (String.valueOf(RssSettings.DEFAULT_ARTICLESCOUNT),20);
	gbl.setConstraints(tf_articles,gbc);
	widgetpanel.add(tf_articles);

	gbc.gridwidth = 1;
	lab=new JLabel("Poll interval:");
	gbl.setConstraints(lab,gbc);
	widgetpanel.add(lab);
	
	tf_pollinterval=new JTextField
	    (String.valueOf(RssSettings.DEFAULT_POLLINTERVAL/60),20);
	gbl.setConstraints(tf_pollinterval,gbc);
	widgetpanel.add(tf_pollinterval);

	gbc.gridwidth = GridBagConstraints.REMAINDER;
	JLabel lab_1=new JLabel("minutes");
	gbl.setConstraints(lab_1,gbc);
	widgetpanel.add(lab_1);

	gbc.gridwidth = 1;
	cb_showtext=new JCheckBox
	    ("view article text",RssSettings.DEFAULT_SHOWTEXT);
	gbl.setConstraints(cb_showtext,gbc);
	widgetpanel.add(cb_showtext);

	gbc.gridwidth = GridBagConstraints.REMAINDER;
	cb_beep=new JCheckBox
	    ("beep on new article",RssSettings.DEFAULT_BEEP);
	gbl.setConstraints(cb_beep,gbc);
	widgetpanel.add(cb_beep);

	if (channel!=null) {
	    tf_name.setText(channel.getName());
	    tf_url.setText(channel.getURL());
	    tf_articles.setText
		(String.valueOf(channel.getArticlesCount()));
	    tf_pollinterval.setText
		(String.valueOf(channel.getPollInterval()/60));
	    cb_showtext.setSelected(channel.getShowText());
	    cb_beep.setSelected(channel.getBeep());

	    if (channel.isGlobal()) {
		tf_name.setEnabled(false);
		tf_url.setEnabled(false);
		tf_pollinterval.setEnabled(false);
		cb_beep.setEnabled(false);
	    }
	}

	JPanel buttonpanel=new JPanel(new FlowLayout(FlowLayout.RIGHT));
	ip.add(buttonpanel,BorderLayout.SOUTH);

	JButton add=new JButton(channel==null ? "Add" : "Modify");
	JButton cancel=new JButton("Cancel");
	buttonpanel.add(add);
	buttonpanel.add(cancel);

	cancel.addActionListener(new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
		    dispose();
		}
	    });

	add.addActionListener(new AbstractAction() {
		public void actionPerformed(ActionEvent e) {

		    String name=tf_name.getText();
		    String url=tf_url.getText();

		    int articlecount=0;
		    int pollinterval=0;
		    boolean showtext=cb_showtext.isSelected();
		    boolean beep=cb_beep.isSelected();

		    try {
			articlecount=
			    Integer.parseInt(tf_articles.getText());
			if (channel==null || !channel.isGlobal()) {
			    pollinterval=
				Integer.parseInt(tf_pollinterval.getText())*60;
			}
		    }
		    catch (Exception ex) {
			return;
		    }
			
		    if (channel==null || !channel.isGlobal()) {
			if (name==null || name.length()<1 ||
			    url==null || url.length()<1) return;
		    }

		    if (channel==null)
			list.addChannel(name,url,articlecount,
					pollinterval,showtext,beep);
		    else {
			if (!channel.isGlobal()) {
			    channel.setName(name);
			    channel.setURL(url);
			    channel.setPollInterval(pollinterval);
			    channel.setBeep(beep);
			}
			channel.setArticlesCount(articlecount);
			channel.setShowText(showtext);
			list.updateList(channel);
		    }

		    dispose();
		}
	    });
    }

    private class InsetsPanel extends JPanel {

	Insets insets=new Insets(10,10,10,10);

	public InsetsPanel() {
	    super(new BorderLayout());
	}

	public Insets getInsets() {
	    return insets;
	}
    }
}
