package de.cinek.rssview;

import javax.swing.*;
import java.awt.*;
import java.awt.dnd.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import de.cinek.rssview.images.IconContainer;
import java.io.*;

public class RssChannelList extends JScrollPane
    implements ListSelectionListener {

    private RssView parent;
    private RssChannelView channelview;
    private RssChannelTitle channeltitle;
    private RssDropDownMenu dropdownmenu;
    private JList channellist;

    public RssChannelList(RssView parent,
			  RssChannelTitle channeltitle,
			  RssChannelView channelview) {
	super();
	this.parent=parent;
	this.channelview=channelview;
	this.channeltitle=channeltitle;
	initComponents();
    }

    public void initComponents() {

	channellist=new JList(RssArticleContainer.getChannelModel());
	setViewportView(channellist);

	channellist.setCellRenderer(new MyCellRenderer());
	channellist.addListSelectionListener(this);
	dropdownmenu=new RssDropDownMenu(parent,this);

	final RssChannelList thislist=this;

	channellist.addMouseListener(new MouseAdapter() {
		
		public void mousePressed(MouseEvent e) {

		    Point p=e.getPoint();
		    int index = channellist.locationToIndex(p);
		    Rectangle r=channellist.getCellBounds(index,index);
			
		    if ((e.getModifiers()&InputEvent.BUTTON3_MASK)!=0) {
			if (index>=0 && r.contains(p) &&
			    !channellist.isSelectedIndex(index)) {
			    channellist.setSelectedIndex(index);
			}
			if (!RssArticleContainer.getChannelModel().
			    isExclusiveMode()) {
			    dropdownmenu.setUpdateEnabled(index!=0);
			} else {
			    /* TODO: gray-out items */
			}
			dropdownmenu.show(e.getComponent(),p.x,p.y);
		    }
		}

		public void mouseClicked(MouseEvent e) {

		    Point p=e.getPoint();
		    int index = channellist.locationToIndex(p);
		    Rectangle r=channellist.getCellBounds(index,index);
			
		    if (index>=0 && r.contains(p)) {

			if ((e.getModifiers()&InputEvent.BUTTON1_MASK)!=0 &&
			    (e.getClickCount()>1)) {
			    if (!channellist.isSelectedIndex(index)) {
				channellist.setSelectedIndex(index);
			    }
				
			    RssChannel channel=
				(RssChannel)channellist.getModel().
				getElementAt(index);
			    new RssChannelDialog
				(parent,thislist,channel);
			}
		    }
		}
	    });

	channellist.setDropTarget(new DropTarget(channellist,
						 new RssDropListener(this)));

	channellist.setSelectedIndex(0);
    }

    public void valueChanged(ListSelectionEvent e) {
	RssChannel channel=(RssChannel)channellist.getSelectedValue();
	if (channel!=null) channel.flushNews();
	channeltitle.updateContent(channel);
	channelview.updateContent(channel);
    }

    public void updateList() {
	channeltitle.updateContent();
	channelview.updateContent();
	revalidate();
	repaint();
    }

    public void updateList(RssChannel channel) {
	channeltitle.updateContent(channel);
	channelview.updateContent(channel);
	revalidate();
	repaint();
    }

    public void updateMenu() {
	dropdownmenu.updateMenu();
    }

    public void moveUp() {
		    
	int i=channellist.getSelectedIndex();
	
	if (i>1) {
	    ((RssChannelModel)channellist.getModel()).moveUp(i);
	    channellist.setSelectedIndex(i-1);
	}
	
	updateList();
    }

    public void moveDown() {

	int i=channellist.getSelectedIndex();
	
	if (i>0 && (i<channellist.getModel().getSize()-1)) {
	    ((RssChannelModel)channellist.getModel()).moveDown(i);
	    channellist.setSelectedIndex(i+1);
	}
	
	updateList();
    }

    public void addChannel(String name,String url,
			   int articlecount,int pollinterval,
			   boolean showtext,boolean beep) {
	((RssChannelModel)channellist.getModel()).
	    addChannel(name,url,articlecount,pollinterval,showtext,beep);
	updateList();
    }

    public RssChannel getSelectedChannel() {
	return (RssChannel)channellist.getSelectedValue();
    }
    
    public RssChannel[] getSelectedChannels() {
	Vector v=new Vector(Arrays.asList(channellist.getSelectedValues()));
	return (RssChannel[])v.toArray(new RssChannel[0]);
    }

    class MyCellRenderer extends JLabel implements ListCellRenderer {

	public Component getListCellRendererComponent
	    (
	     JList list,
	     Object value,
	     int index,
	     boolean isSelected,
	     boolean cellHasFocus)
	{
	    RssChannel channel = (RssChannel)value;
	    ImageIcon icon=channel.getIcon();
	    if (icon==null) icon=IconContainer.ICON_CHANNEL;

	    setText(channel.toString());
	    setIcon(channel.hasNews() ? 
		    IconContainer.ICON_NEWS : 
		    icon);

	    String desc=channel.getDescription();
	    if (desc!=null) {
		if (desc.length()>40) desc=desc.substring(0,40)+"...";
		setToolTipText(desc);
	    } else setToolTipText(null);
	    if (isSelected) {
		setBackground(list.getSelectionBackground());
		setForeground(list.getSelectionForeground());
	    }
	    else {
		setBackground(list.getBackground());
		setForeground(list.getForeground());
	    }
	    setEnabled(list.isEnabled());
	    setFont(list.getFont());
	    setOpaque(true);
	    return this;
	}
    }

    private class RssDropListener implements DropTargetListener {

	RssChannelList list=null;
	
	public RssDropListener(RssChannelList l) {
	    list=l;
	}

	public void dragEnter(DropTargetDragEvent e) {
	    e.acceptDrag(DnDConstants.ACTION_COPY);
	}

	public void dragOver(DropTargetDragEvent e) {
	    e.acceptDrag(DnDConstants.ACTION_COPY);
	}
	
	public void dragExit(DropTargetEvent e) {
	}
	
	public void drop (DropTargetDropEvent e) {
	    
	    try {
		Transferable transferable = e.getTransferable();
		
		if (transferable.isDataFlavorSupported
		    (DataFlavor.stringFlavor)){
		    
		    e.acceptDrop(DnDConstants.ACTION_MOVE);
		    String s = (String)transferable.getTransferData
			(DataFlavor.stringFlavor);
		    new RssChannelDialog(parent,list,s);
		    e.getDropTargetContext().dropComplete(true);
		} 
		else{
		    e.rejectDrop();
		}
	    }
	    catch (IOException ex) {
		e.rejectDrop();
	    } 
	    catch (UnsupportedFlavorException ex) {
		e.rejectDrop();
	    }
	}

	public void dropActionChanged(DropTargetDragEvent dtde) {
	}
    }
}
