package de.cinek.rssview;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import de.cinek.rssview.images.IconContainer;

public class RssChannelTitle extends JScrollPane {

    private JTextPane content;
    private RssChannel channel;
    private String link;
    private String searchquery;
    private JButton button;
    private JTextField textfield;
    private RssDropDownMenu dropdownmenu;

    public RssChannelTitle(RssView parent) {
	super();
	initComponents(parent);
    }

    private void initComponents(RssView parent) {

	content=new JTextPane();
	content.setEditable(false);
	setViewportView(content);
	
	textfield=new MyTextField();
	button=new MyButton((ActionListener)textfield);

	dropdownmenu=new RssDropDownMenu(parent,this);

	content.addMouseListener(new MouseAdapter() {
		
		public void mousePressed(MouseEvent e) {
		    if ((e.getModifiers()&InputEvent.BUTTON3_MASK)!=0) {
			dropdownmenu.show(e.getComponent(),e.getX(),e.getY());
			return;
		    }
		    if ((e.getModifiers()&InputEvent.BUTTON1_MASK)!=0) {
			try {
			    if (channel==null || channel.isGlobal()) {
				new RssBrowserStart
				    ("http://sf.net/");
			    } else {
				new RssBrowserStart
				    (channel.getURL());
			    }
			}
			catch (Exception ex) {
			    JOptionPane.showMessageDialog
				(null,
				 "Channel URL could not be retrieved.",
				 "Browse error",
				 JOptionPane.WARNING_MESSAGE);
			}
			return;
		    }
		}
	    });
    }

    public RssChannel getShownChannel() {
	return channel;
    }

    public void updateContent(RssChannel channel) {

	this.channel=channel;
	if (channel==null) {
	    channel=RssArticleContainer.getGlobalChannel();
	}
	 
	String title=channel.getTitle();
	if (title==null) title="";
	link=channel.getLink();
	String description=channel.getDescription();
	if (description==null) description="";
	searchquery=channel.getTextInputQuery();

	ImageIcon img=channel.getImage();

	String[] styles=
	    {img==null ? null : "image","bold","regular",
	    searchquery==null ? null : "sbold",
	    searchquery==null ? null : "regular",
	    searchquery==null ? null : "textentry",
	    searchquery==null ? null : "button"
	    };

	String[] contents={
	    " "," "+title+"\n",description+"\n\n",
	    searchquery==null ? "" : channel.getTextInputTitle()+"\n",
	    searchquery==null ? "" : channel.getTextInputDescription()+"\n",
	    " "," "
	};

	initStyles(img);

	Document sd=content.getDocument();
	try {
	    sd.remove(0,sd.getLength());
	}
	catch (BadLocationException ex) {}
	for (int i=0;i<contents.length;i++) {
	    try {
		sd.insertString(sd.getLength(),contents[i],
				styles[i]==null ? null :
				content.getStyle(styles[i]));
	    }
	    catch (BadLocationException ex) {}
	}
	
	content.setCaretPosition(0);
    }

    public void updateContent() {
	updateContent(channel);
    }

    public void updateMenu() {
	dropdownmenu.updateMenu();
    }

    public void initStyles(ImageIcon i) {

	if (i==null) i=IconContainer.APP_LOGO;

	Style def = StyleContext.getDefaultStyleContext().
	    getStyle(StyleContext.DEFAULT_STYLE);

        Style regular = content.addStyle("regular", def);
        StyleConstants.setFontFamily(def, "SansSerif");
	
        Style s = content.addStyle("sbold", regular);
        StyleConstants.setFontSize(s,16);
        StyleConstants.setBold(s, true);

        s = content.addStyle("bold", s);
        StyleConstants.setFontSize(s,22);

	s = content.addStyle("image", regular);
        StyleConstants.setIcon(s,i);

	s = content.addStyle("button", regular);
        StyleConstants.setComponent(s,button);

	s = content.addStyle("textentry", regular);
        StyleConstants.setComponent(s,textfield);
    }

    private class MyTextField extends JTextField implements ActionListener {
	public MyTextField() {
	    super();
	    addActionListener(this);
	}

	public void actionPerformed(ActionEvent e) {
	    String query=getText();
	    if (query==null || query.length()<1) return;

	    try {
		new RssBrowserStart(searchquery+query);
	    }
	    catch (Exception ex) {
		JOptionPane.showMessageDialog
		    (null,
		     "URL could not be resolved.",
		     "Query error",
		     JOptionPane.WARNING_MESSAGE);
	    }
	}
    }

    private class MyButton extends JButton {
	public MyButton(ActionListener a) {
	    super("Submit");
	    addActionListener(a);
	}
    }
}
