package de.cinek.rssview;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.net.URL;
import de.cinek.rssview.images.IconContainer;

public class RssChannelView extends JScrollPane implements ChangeListener {

    private JTextPane content;
    private RssChannel channel;
    private RssNotifyThread notifythread=null;
    private RssDropDownMenu dropdownmenu;
    private RssView parent;
    private String selectedlink=null;

    private static final Point TOP=new Point(0,0);
    private static final String HREF="HREF";

    public RssChannelView(RssView parent) {
	super();
	this.parent=parent;
	initComponents();
    }

    private void initComponents() {
	content=new JTextPane();
	content.setEditable(false);
	LinkController lc=new LinkController(content);
	content.addMouseListener(lc);
	content.addMouseMotionListener(lc);
	content.addHyperlinkListener(new RssHyperlinkListener());
	setViewportView(content);
	getViewport().addChangeListener(this);

	dropdownmenu=new RssDropDownMenu(parent,this);

    }

    public synchronized void updateContent(RssChannel channel) {

	if (channel==null) channel=RssArticleContainer.getGlobalChannel();
	this.channel=channel;
	RssArticle[] article=channel.
	    getLatestArticles(channel.getArticlesCount());

	boolean global=channel.isGlobal();

	Document sd=content.getDocument();
	try {
	    sd.remove(0,sd.getLength());
	}
	catch (BadLocationException ex) {}
	for (int i=0;i<article.length;i++) {
	    insertArticle(sd,article[i],channel,global);
	}

	content.setCaretPosition(0);
	//	home();
    }

    public void updateContent() {

	
	if (channel!=null && !channel.hasNews()) return;

	JViewport vp=getViewport();
	if (vp==null || vp.getViewPosition().equals(TOP))
	    updateContent(channel);
	else {
	    if (notifythread!=null)
		notifythread.kill();

	    notifythread=new RssNotifyThread(this);
	    notifythread.start();
	}
    }

    private synchronized void insertArticle(Document sd,RssArticle article,
			       RssChannel channel,boolean global) {

	String title=article.getTitle();
	String desc=channel.getShowText() ?
	    article.getDescription() : null;
	
	if (title!=null && title.length()>0) {
	    if (global) {
		insertIcon(sd,article.getIcon());
		insertText(sd," ");
	    }
	    
	    insertTitle(sd,title+"\n",article.getLink());
	    
	    if (desc!=null && desc.length()>0)
		insertText(sd,desc+"\n");
	    else
		insertText(sd,"\n");
	}
    }

    public void insertIcon(Document doc,ImageIcon i) {
	if (i==null) i=IconContainer.ICON_CHANNEL;
	try {
	    SimpleAttributeSet attrs=new SimpleAttributeSet();
	    StyleConstants.setIcon(attrs,i);
	    doc.insertString(doc.getLength()," ",attrs);
	}
	catch (BadLocationException ex) {}
    }

    public void insertTitle(Document doc,String title,String url) {
	try {
	    SimpleAttributeSet attrs = new SimpleAttributeSet();
	    StyleConstants.setUnderline(attrs, true);
	    StyleConstants.setForeground(attrs, Color.blue);
	    StyleConstants.setFontFamily(attrs, "SansSerif");
	    StyleConstants.setBold(attrs,true);
	    attrs.addAttribute(HREF,url);
	    doc.insertString(doc.getLength(),title, attrs);
	}
	catch (BadLocationException ex) {}
    }

    public void insertText(Document doc,String text) {
	if (text==null || text.length()<1) return;
	try {
	    doc.insertString(doc.getLength(),text,
			     content.getStyle("regular"));
	}
	catch (BadLocationException ex) {}
    }

    public void initStyles() {
	Style def = StyleContext.getDefaultStyleContext().
	    getStyle(StyleContext.DEFAULT_STYLE);

        Style regular = content.addStyle("regular", def);
        StyleConstants.setFontFamily(regular, "SansSerif");
    }

    public void home() {
	JViewport vp=getViewport();
	if (vp!=null)
	    vp.setViewPosition(TOP);
    }

    public void updateMenu() {
	dropdownmenu.updateMenu();
    }

    public void stateChanged(ChangeEvent e) {

	JViewport vp=getViewport();
	if (vp==null) return;
	if (vp.getViewPosition().equals(TOP)) {
	    if (channel!=null) {
		if (channel.hasNews()) {
		    updateContent(channel);
		    channel.flushNews();
		}
	    }
	}
    }

    public String getSelectedLink() {
	return selectedlink;
    }

    public class LinkController extends MouseAdapter 
	implements MouseMotionListener {

	JTextPane parent;

	public LinkController(JTextPane parent) {
	    this.parent=parent;
	}

	public void mouseMoved(MouseEvent e) {

	    JTextPane editor = (JTextPane) e.getSource();

	    Point pt = new Point(e.getX(),e.getY());
	    int pos = editor.viewToModel(pt);

	    if (pos>=0) {                    
		DefaultStyledDocument hdoc = 
		    (DefaultStyledDocument)editor.getDocument();

		Element el = hdoc.getCharacterElement(pos);
		AttributeSet a = el.getAttributes();
		String href = (String) a.getAttribute(HREF);

		if (href != null) {
		    parent.setCursor(new Cursor(Cursor.HAND_CURSOR));
		} else {
		    parent.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
	    } else {
		parent.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	    }
	}

	public void mousePressed(MouseEvent e) {
	    if ((e.getModifiers()&InputEvent.BUTTON3_MASK)!=0) {
		Element el=getSelectedElement(e);

		if (el!=null) {
		    AttributeSet a = el.getAttributes();
		    selectedlink = (String) a.getAttribute(HREF);
		    
		    dropdownmenu.grayURLCopy(selectedlink==null);
		} else
		    dropdownmenu.grayURLCopy(true);

		dropdownmenu.show(e.getComponent(),e.getX(),e.getY());
	    }
	}

	public void mouseClicked(MouseEvent e) {

	    if ((e.getModifiers()&InputEvent.BUTTON1_MASK)==0) return;

	    JTextPane editor = (JTextPane) e.getSource();
            Element el=getSelectedElement(e);

	    if (el!=null) {
		AttributeSet a = el.getAttributes();
		String href = (String) a.getAttribute(HREF);

		if (href != null) {
		    try {
			parent.fireHyperlinkUpdate
			    (new RssHyperlinkEvent
			     (editor,HyperlinkEvent.EventType.ACTIVATED,
			      href));
		    }
		    catch (Exception ex) {
		    }
		}
	    }
	}

	public void mouseDragged(MouseEvent e) {
	}

	private Element getSelectedElement(MouseEvent e) {

	    JTextPane editor = (JTextPane) e.getSource();
            
	    Point pt = new Point(e.getX(), e.getY());
	    int pos = editor.viewToModel(pt);
	    if (pos >= 0) {
		DefaultStyledDocument hdoc = 
		    (DefaultStyledDocument)editor.getDocument();

		Element el = hdoc.getCharacterElement(pos);
		return el;
	    }
	    return null;
	}
    }
}
