package de.cinek.rssview;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;

public class RssDropDownMenu extends JPopupMenu {

    public static final int LIST_MENU=1;
    public static final int TITLE_MENU=2;
    public static final int CONTENTS_MENU=3;

    private JMenuItem displaytype;
    private JMenuItem channelreload=null;
    private JMenuItem copylink=null;
    private static final String[] displaytype_string={"shrink","grow","none"};
    private RssView parent;
    private RssChannelList list=null;
    private RssChannelTitle title=null;
    private RssChannelView view=null;

    public RssDropDownMenu(RssView parent,RssChannelView view) {
	super("Menu");
	this.parent=parent;
	this.view=view;
	initMenuItems(CONTENTS_MENU);
    }

    public RssDropDownMenu(RssView parent,RssChannelList list) {
	super("Menu");
	this.parent=parent;
	this.list=list;
	initMenuItems(LIST_MENU);
    }

    public RssDropDownMenu(RssView parent,RssChannelTitle title) {
	super("Menu");
	this.parent=parent;
	this.title=title;
	initMenuItems(TITLE_MENU);
    }

    private void initMenuItems(int menutype) {
	displaytype=new JMenuItem("setup error");
	displaytype.addActionListener(new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
		    switch (parent.getViewSize()) {
		    case RssView.VIEWSIZE_NORMAL:
			parent.setViewSize(RssView.VIEWSIZE_SMALL);
			break;
		    case RssView.VIEWSIZE_SMALL:
			parent.setViewSize(RssView.VIEWSIZE_NORMAL);
			break;
		    }
		    parent.updateMenus();
		}
	    });

	add(displaytype);
	addSeparator();

	switch (menutype) {
	case LIST_MENU:
	    channelreload=new JMenuItem("Update");
	    channelreload.addActionListener(new AbstractAction() {
		    public void actionPerformed(ActionEvent e) {
			RssChannel channel=
			    list.getSelectedChannel();
			if (channel==null) {
			    JOptionPane.showMessageDialog
				(null,
				 "Please select to update.",
				 "Error",JOptionPane.ERROR_MESSAGE);
			    return;
			}
			channel.update();
			parent.updateViews();
		    }
		});
	    add(channelreload);
	    addSeparator();
	    JMenuItem channelnew=new JMenuItem("New channel...");
	    channelnew.addActionListener(new AbstractAction() {
		    public void actionPerformed(ActionEvent e) {
			new RssChannelDialog(parent,list);
		    }
		});
	    add(channelnew);
	    JMenuItem channelmodify=new JMenuItem("Modify channel...");
	    channelmodify.addActionListener(new AbstractAction() {
		    public void actionPerformed(ActionEvent e) {
			RssChannel channel=
			    list.getSelectedChannel();
			if (channel==null) {
			    JOptionPane.showMessageDialog
				(null,
				 "Please select a channel to modify.",
				 "Error",JOptionPane.ERROR_MESSAGE);
			    return;
			}
			new RssChannelDialog(parent,list,channel);
		    }
		});
	    add(channelmodify);
	    JMenuItem channeldelete=new JMenuItem("Delete channel");
	    channeldelete.addActionListener(new AbstractAction() {
		    public void actionPerformed(ActionEvent e) {
			RssChannel[] channel=
			    list.getSelectedChannels();
			if (channel==null || channel.length<1) return;

			String clist="";
			for (int i=0;i<channel.length;i++) {
			    if (channel[i].isGlobal()) {
				JOptionPane.showMessageDialog
				    (null,
				     "<"+RssChannelModel.GLOBAL_CHANNEL
				     +"> channel cannot be removed.",
				     "Error",
				     JOptionPane.ERROR_MESSAGE);
				return;
			    }
			    clist+=channel[i].getName();
			    if (i<channel.length-1)
				clist+=", ";
			}

			if (JOptionPane.showConfirmDialog
			    (null,
			     "Are You sure to delete the channel"+
			     (channel.length==1 ? ":" : "s:")+
			     "\n"+clist,
			     "Confirmation",
			     JOptionPane.YES_NO_OPTION,
			     JOptionPane.QUESTION_MESSAGE)==
			    JOptionPane.YES_OPTION) {
			    RssArticleContainer.getChannelModel().
				removeChannels(channel);
			}
		    }
		});
	    add(channeldelete);
	    addSeparator();

	    JMenuItem copylistlink=new JMenuItem("Copy channel URL");
	    copylistlink.addActionListener(new AbstractAction() {
		    public void actionPerformed(ActionEvent e) {
			RssChannel channel=
			    list.getSelectedChannel();
			if (channel==null) {
			    JOptionPane.showMessageDialog
				(null,
				 "Please select a channel.",
				 "Error",JOptionPane.ERROR_MESSAGE);
			    return;
			}
			Clipboard clipboard=Toolkit.getDefaultToolkit().
			    getSystemClipboard();
			clipboard.setContents
			    (new StringSelection(channel.getURL()),null);
		    }
		});
	    add(copylistlink);
	    JMenuItem pastelink=new JMenuItem("Paste channel URL...");
	    pastelink.addActionListener(new AbstractAction() {
		    public void actionPerformed(ActionEvent e) {

			try {
			    Transferable transferable=Toolkit.
				getDefaultToolkit().
				getSystemClipboard().getContents(this);
			    if (transferable.isDataFlavorSupported
				(DataFlavor.stringFlavor)){
				String s = (String)transferable.getTransferData
				    (DataFlavor.stringFlavor);
				new RssChannelDialog(parent,list,s);
			    }
			}
			catch (Exception ex) {}
		    }
		});
	    add(pastelink);
	    addSeparator();

	    if (RssArticleContainer.getChannelModel().isExclusiveMode()) {
		channelnew.setEnabled(false);
		channelmodify.setEnabled(false);
		channeldelete.setEnabled(false);
		pastelink.setEnabled(false);
	    }
	    break;

	case TITLE_MENU:
	    JMenuItem copytitlelink=new JMenuItem("Copy channel URL");
	    copytitlelink.addActionListener(new AbstractAction() {
		    public void actionPerformed(ActionEvent e) {
			RssChannel channel=
			    title.getShownChannel();
			if (channel==null) {
			    JOptionPane.showMessageDialog
				(null,
				 "Please select a channel.",
				 "Error",JOptionPane.ERROR_MESSAGE);
			    return;
			}
			Clipboard clipboard=Toolkit.getDefaultToolkit().
			    getSystemClipboard();
			clipboard.setContents
			    (new StringSelection(channel.getURL()),null);
		    }
		});
	    add(copytitlelink);
	    addSeparator();
	    break;

	case CONTENTS_MENU:
	    copylink=new JMenuItem("Copy URL");
	    copylink.addActionListener(new AbstractAction() {
		    public void actionPerformed(ActionEvent e) {
			String link=
			    view.getSelectedLink();
			if (link==null) return;

			Clipboard clipboard=Toolkit.getDefaultToolkit().
			    getSystemClipboard();
			clipboard.setContents
			    (new StringSelection(link),null);
		    }
		});
	    add(copylink);
	    addSeparator();
	    break;
	}

	JMenuItem browser=new JMenuItem("Browser setup...");
	browser.addActionListener(new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
		    new RssBrowserSetup(parent);
		}
	    });
	add(browser);
	addSeparator();

	JMenuItem info=new JMenuItem("Info...");
	info.addActionListener(new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
		    RssInfoDialog.showInfo(parent);
		}
	    });
	add(info);
	JMenuItem exit=new JMenuItem("Exit");
	exit.addActionListener(new AbstractAction() {
		public void actionPerformed(ActionEvent e) {
		    parent.updateSettings();
		    RssSettings.saveSettings();
		    System.exit(0);
		}
	    });
	add(exit);
    }

    public void setUpdateEnabled(boolean b) {
	if (channelreload!=null) channelreload.setEnabled(b);
    }

    public void updateMenu() {
	int viewsize=parent.getViewSize();

	displaytype.setText(displaytype_string[parent.getViewSize()]);
	displaytype.setEnabled(viewsize!=RssView.VIEWSIZE_EXCLUSIVE);
    }

    public void grayURLCopy(boolean b) {
	if (copylink!=null) copylink.setEnabled(!b);
    }
}
