package de.cinek.rssview;

import java.io.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import java.util.*;

public class RssParser extends DefaultHandler {

    private SAXParser parser;
    private Vector channelarticles;
    private String currentElement;
    private String title;
    private String link;
    private String desc;
    private String name;
    private String iconurl;
    private RssChannel channel;
    private boolean insideImage=false;
    private boolean insideChannel=false;
    private boolean insideItem=false;
    private boolean insideTextInput=false;
    private String channelTitle;
    private String channelLink;
    private String channelDescription;

    /*
      Doesn't work correctly
    private class RssEntityResolver implements EntityResolver {
	public InputSource resolveEntity(String publicId,
					 String systemId)
	{
	    
	    return null;
	}
    }
    */

    public RssParser(RssChannel channel) {
	try {
	    parser=SAXParserFactory.newInstance().newSAXParser();
	    //parser.getXMLReader().setEntityResolver
	    //(new RssEntityResolver());
	}
	catch (Exception ex) {
	    System.err.println("Error initializing SAXParser: "+
			       channel.getName());
	    ex.printStackTrace();
	}

	this.channel=channel;
    }

    public void parse() {

	iconurl=null;
	channelarticles=new Vector();
	channelTitle="";
	channelLink="";
	channelDescription="";

	try {
	    parser.parse(new InputSource(channel.getURL()),this);
	} catch (Exception e) {
	    System.err.println("Error while fetching channel: "+
			       channel.getName());
	    e.printStackTrace();
	    return;
	}
	channel.addArticles(channelarticles);
	channel.setCurrentCount(channelarticles.size());
    }

    public void startElement(String uri,String localname, String qname,
			     Attributes attr) {
	currentElement=qname;
	if (qname.equals("item")) {
	    title="";
	    link="";
	    desc="";
	    insideItem=true;
	    return;
	}
	if (qname.equals("image")) {
	    insideImage=true;
	    return;
	}
	if (qname.equals("channel")) {
	    insideChannel=true;
	    return;
	}
	if (qname.equals("textinput")) {
	    title="";
	    link="";
	    desc="";
	    name="";
	    insideTextInput=true;
	    return;
	}
    }

    public void endElement(String uri,String localname, String qname) {
	if (qname.equals("textinput")) {
	    insideTextInput=false;
	    channel.setTextInput(title,desc,name,link);
	    return;
	}
	if (qname.equals("item")) {
	    channelarticles.add(0,new RssArticle(channel,title,link,desc));
	    insideItem=false;
	    return;
	}
	if (qname.equals("image")) {
	    if (channel.getIcon()==null && iconurl!=null) {
		channel.setIcon(iconurl);
	    }
	    insideImage=false;
	    return;
	}
	if (qname.equals("channel")) {
	    channel.setTitle(channelTitle);
	    channel.setLink(channelLink);
	    channel.setDescription(channelDescription);
	    insideChannel=false;
	    return;
	}
    }

    public void characters(char[] ch,int start,int length) {

	if (insideTextInput) {
	    if (currentElement.equals("title")) {
		title+=new String(ch,start,length);
		return;
	    }
	    if (currentElement.equals("link")) {
		link+=new String(ch,start,length);
		return;
	    }
	    if (currentElement.equals("description")) {
		desc+=new String(ch,start,length);
		return;
	    }
	    if (currentElement.equals("name")) {
		name+=new String(ch,start,length);
		return;
	    }
	}
	if (insideItem) {
	    if (currentElement.equals("title")) {
		title+=new String(ch,start,length);
		return;
	    }
	    if (currentElement.equals("link")) {
		link+=new String(ch,start,length);
		return;
	    }
	    if (currentElement.equals("description")) {
		desc+=new String(ch,start,length);
		return;
	    }
	}
	if (insideImage) {
	    if (currentElement.equals("url")) {
		if (iconurl==null) iconurl=new String(ch,start,length);
	    }
	    return;
	}
	if (insideChannel) {
	    if (currentElement.equals("title")) {
		channelTitle+=new String(ch,start,length);
		return;
	    }
	    if (currentElement.equals("link")) {
		channelLink+=new String(ch,start,length);
		return;
	    }
	    if (currentElement.equals("description")) {
		channelDescription+=new String(ch,start,length);
		return;
	    }
	    return;
	}
    }
}
