package de.cinek.rssview;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import de.cinek.rssview.images.IconContainer;

public class RssView extends JFrame {

    public static final String APP_NAME="RssView";
    public static final String APP_VER="1.1.5";

    private RssChannelList channelList;
    private RssChannelView channelContent;
    private RssChannelTitle channelTitle;

    public static final int VIEWSIZE_NORMAL=0;
    public static final int VIEWSIZE_SMALL=1;
    public static final int VIEWSIZE_EXCLUSIVE=2;
    public static final int VIEWMODE_INTERACTIVE=0;
    public static final int VIEWMODE_EXCLUSIVE=1;

    private int viewsize=-1;
    private int viewmode=VIEWMODE_INTERACTIVE;

    private JSplitPane rightPart;
    private JSplitPane jsp;

    public RssView() {
	super(APP_NAME+" - "+APP_VER);
	RssFetchThread.setParent(this);
	new de.cinek.rssview.images.IconContainer();
	RssSettings.loadSettings();
	viewmode=VIEWMODE_INTERACTIVE;
	initComponents();
	initActions();
	pack();
	show();
    }

    public RssView(String channel) {
	super(APP_NAME+" - "+APP_VER);
	RssFetchThread.setParent(this);
	new de.cinek.rssview.images.IconContainer();
	RssSettings.loadSettings();
	RssArticleContainer.getChannelModel().setExclusiveMode(channel);
	viewmode=VIEWMODE_EXCLUSIVE;
	initComponents();
	initActions();
	pack();
	show();
    }

    private void initComponents() {

	getContentPane().setLayout(new BorderLayout());

	channelTitle=new RssChannelTitle(this);
	channelContent=new RssChannelView(this);
	channelList=new RssChannelList(this,channelTitle,channelContent);

	setViewSize(viewmode==VIEWMODE_EXCLUSIVE ? VIEWSIZE_EXCLUSIVE :
		    RssSettings.getViewSize());

	setIconImage(IconContainer.ICON_APP.getImage());
    }

    private void initActions() {

	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    updateSettings();
		    RssSettings.saveSettings();
		    System.exit(0);
		}
	    });

	addComponentListener(new ComponentAdapter() {
		public void componentResized(ComponentEvent e) {
		    updateSettings();
		}
	    });

	/*(new RssThread(this)).start();*/
    }

    public void updateSettings() {
	if (viewmode!=VIEWMODE_EXCLUSIVE) {
	    if (rightPart!=null && rightPart.isVisible())
		RssSettings.setDivider1(rightPart.getDividerLocation());
	    if (jsp!=null && jsp.isVisible())
		RssSettings.setDivider2(jsp.getDividerLocation());
	    RssSettings.setViewSize(viewsize);
	    switch (viewsize) {
	    case VIEWSIZE_NORMAL:
		RssSettings.setNormalViewSize(getSize());
		RssSettings.setNormalViewPosition(getLocation());
		break;
	    case VIEWSIZE_SMALL:
		RssSettings.setSmallViewSize(getSize());
		RssSettings.setSmallViewPosition(getLocation());
		break;
	    }
	} else {
	    RssSettings.setExclusiveViewSize(getSize());
	    RssSettings.setExclusiveViewPosition(getLocation());
	    if (rightPart!=null && rightPart.isVisible())
		RssSettings.setExclusiveDivider1
		    (rightPart.getDividerLocation());
	    if (jsp!=null && jsp.isVisible())
		RssSettings.setExclusiveDivider2
		    (jsp.getDividerLocation());
	}
    }

    public void updateViews() {
	if (channelList!=null) channelList.updateList();
    }

    public void updateMenus() {
	channelList.updateMenu();
	channelContent.updateMenu();
	channelTitle.updateMenu();
    }

    public int getViewSize() {
	return viewsize;
    }

    public void setViewSize(int viewsize) {


	if (this.viewsize>=0) {
	    updateSettings();
	    getContentPane().removeAll();
	}
	setVisible(true);

	JPanel leftPart;

	switch (viewsize) {
	case VIEWSIZE_NORMAL:
	    if (this.viewsize>=0) setSize(RssSettings.getNormalViewSize());
	    setLocation(RssSettings.getNormalViewPosition());
	    rightPart=new JSplitPane
		(JSplitPane.VERTICAL_SPLIT,channelTitle,channelContent);
	    rightPart.setDividerSize(3);
	    rightPart.setDividerLocation(RssSettings.getDivider1());


	    leftPart=new JPanel(new BorderLayout());
	    leftPart.add(channelList,BorderLayout.CENTER);
	    leftPart.add(new RssToolBar(this,channelList),BorderLayout.SOUTH);
	    jsp=new JSplitPane
		(JSplitPane.HORIZONTAL_SPLIT,leftPart,rightPart);
	    jsp.setDividerSize(3);
	    jsp.setDividerLocation(RssSettings.getDivider2());
	    getContentPane().add
		(jsp,BorderLayout.CENTER);
	    break;
	case VIEWSIZE_EXCLUSIVE:
	    if (this.viewsize>=0) setSize(RssSettings.getExclusiveViewSize());
	    setLocation(RssSettings.getExclusiveViewPosition());
	    rightPart=new JSplitPane
		(JSplitPane.VERTICAL_SPLIT,channelTitle,channelContent);
	    rightPart.setDividerSize(3);
	    rightPart.setDividerLocation(RssSettings.getExclusiveDivider1());


	    leftPart=new JPanel(new BorderLayout());
	    leftPart.add(channelList,BorderLayout.CENTER);
	    leftPart.add(new RssToolBar(this,channelList),BorderLayout.SOUTH);
	    jsp=new JSplitPane
		(JSplitPane.HORIZONTAL_SPLIT,leftPart,rightPart);
	    jsp.setDividerSize(3);
	    jsp.setDividerLocation(RssSettings.getExclusiveDivider2());
	    getContentPane().add
		(jsp,BorderLayout.CENTER);
	    break;
	case VIEWSIZE_SMALL:
	    if (this.viewsize>=0) setSize(RssSettings.getSmallViewSize());
	    setLocation(RssSettings.getSmallViewPosition());
	    getContentPane().add(channelContent,BorderLayout.CENTER);
	    break;
	}


	if (this.viewsize>=0) {
	    validate();
	    repaint();
	}

	this.viewsize=viewsize;
	updateMenus();
    }

    public Dimension getPreferredSize() {
	if (viewmode==VIEWMODE_EXCLUSIVE)
	    return RssSettings.getExclusiveViewSize();
	switch (viewsize) {
	case VIEWSIZE_SMALL:
	    return RssSettings.getSmallViewSize();
	case VIEWSIZE_EXCLUSIVE:
	    return RssSettings.getExclusiveViewSize();
	}
	return RssSettings.getNormalViewSize();
    }

    public static void main(String arg[]) {

	switch (arg.length) {
	case 0:
	    new RssView();
	    break;
	case 1:
	    new RssView(arg[0]);
	    break;
	case 2:
	    JOptionPane.showMessageDialog
		(null,
		 "Please specify not more than 1 channel on command-line.",
		 "Error",JOptionPane.ERROR_MESSAGE);
	    break;
	}
    }
}
