package br.usp.talkagent.configutil;

/**
 * @author Percival Lucena 
 */

import org.w3c.dom.*;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.Result;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import java.io.*;
import java.net.*; 

public class PrefReader 
{

private String amsclient;
private String tt_host;
private String filename;
private String ozoneServer;
private String install;
private String java; 
private boolean load_tt_ontology;
private boolean restore_tt_ontology;
private boolean findvalue=false;
private boolean buildST=false;
private boolean restoreComponentsDB=false;
private Document xmldoc;
private Node root;
private NodeList sels;

// Constructor
public PrefReader()
{
	super();
	this.filename = "../db/config.xml";
	init();
}


public PrefReader(String cfgfile)
{
	super();
	this.filename = cfgfile;
	init();
	
}


private void init()
{
	try
	{	
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
  		DocumentBuilder db = dbf.newDocumentBuilder();
  		File prefs = new File(filename);
  		xmldoc = db.parse(prefs);  
  		xmldoc.normalize();
  		this.ReadXMLConfig();
	}
	catch(Exception e)
	{
		System.out.println(e);
	}
}


private void ReadXMLConfig()
{	    
  	Node aux;
	int i, nels;  
	String saux;	
  	  
  // start XML Parsing
  try
  {
  	
	root = xmldoc.getDocumentElement();
	NodeList els = root.getChildNodes();
	nels = els.getLength();

	for (i = 0; i < nels; i++) 
	{
		aux = els.item(i);
		if ((aux.getNodeName().equalsIgnoreCase("INSTALL_DIRECTORY"))  && (aux!=null) )
		{
			NodeList child = aux.getChildNodes();
			install = child.item(0).getNodeValue();
		}
		if ((aux.getNodeName().equalsIgnoreCase("JAVA"))  && (aux!=null) )
		{
			NodeList child = aux.getChildNodes();
			java = child.item(0).getNodeValue();
		}		
		if ((aux.getNodeName().equalsIgnoreCase("TT_HOST"))  && (aux!=null) )
		{
			NodeList child = aux.getChildNodes();
			tt_host = child.item(0).getNodeValue();
		}		
		if ((aux.getNodeName().equalsIgnoreCase("AMS"))  && (aux!=null) )
		{
			NodeList child = aux.getChildNodes();
			amsclient = child.item(0).getNodeValue();
		}
		if ((aux.getNodeName().equalsIgnoreCase("OZONE"))  && (aux!=null) )
		{
			NodeList child = aux.getChildNodes();
			ozoneServer = child.item(0).getNodeValue();
		}
		if ((aux.getNodeName().equalsIgnoreCase("LOAD_TT_ONTOLOGY"))  && (aux!=null) )
		{		
			NodeList child = aux.getChildNodes();
			saux = child.item(0).getNodeValue();			
			this.load_tt_ontology = true;
			if(saux.equalsIgnoreCase("NO"))			
			this.load_tt_ontology = false;
			this.findvalue=true;			
		}				
		if ((aux.getNodeName().equalsIgnoreCase("RESTORE_TT_ONTOLOGY"))  && (aux!=null) )
		{		
			NodeList child = aux.getChildNodes();
			saux = child.item(0).getNodeValue();
			if(saux.equalsIgnoreCase("YES"))			
			this.restore_tt_ontology=true;
			if(saux.equalsIgnoreCase("NO"))			
			this.restore_tt_ontology = false;
			this.findvalue=true;						
		}
		if ((aux.getNodeName().equalsIgnoreCase("BUILD_STATISTICS_TREE"))  && (aux!=null) )
		{		
			NodeList child = aux.getChildNodes();
			saux = child.item(0).getNodeValue();
			if(saux.equalsIgnoreCase("YES"))			
			this.buildST = true;
			if(saux.equalsIgnoreCase("NO"))			
			this.buildST = false;
			this.findvalue = true;						
		}
		if ((aux.getNodeName().equalsIgnoreCase("RESTORE_COMPONENTS_DB"))  && (aux!=null) )
		{		
			NodeList child = aux.getChildNodes();
			saux = child.item(0).getNodeValue();
			if(saux.equalsIgnoreCase("YES"))			
			this.restoreComponentsDB = true;
			if(saux.equalsIgnoreCase("NO"))			
			this.restoreComponentsDB = false;
			this.findvalue = true;						
		}	
	}	
//	this.xml = true; 	  	
  }
  catch(Exception e)
  {
	try
	  {  	
  		  this.amsclient = InetAddress.getLocalHost().getHostAddress();
  	  	  this.tt_host="127.0.0.1";  	  	
	  	  System.out.println("exception: " + e.getMessage());	
	  }
	catch(Exception e1){}   	 
  }
}


public void setElement(String element, String value)
{	    
  	Node aux;
	int i, nels;  
	String saux;	
  	  
  // start XML Parsing
  try
  { 	
	sels = root.getChildNodes();
	nels = sels.getLength();

	for (i = 0; i < nels; i++) 
	{
		aux = sels.item(i);
		if ((aux.getNodeName().equalsIgnoreCase(element))  && (aux!=null) )
		{
			NodeList child = aux.getChildNodes();
			child.item(0).setNodeValue(value);
		}
	}
  }  
  catch(Exception e)
  {
  	System.out.println(e);
  }	
}


public void saveFile()
{
	this.writeXmlToFile(this.filename, this.xmldoc);	
}


public String getAmsClient()
{	
	return this.amsclient;
}


public String getTTHost()
{
	return this.tt_host;
}


/**
 * Returns the ozondeServer.
 * @return String
 */
public String getOzoneServer()
{
	return ozoneServer;
}


/**
 * Returns the load_tt_ontology.
 * @return boolean
 */
public boolean isLoad_tt_ontology() 
{
	if(this.findvalue==false)
		System.err.println("WARNING ERROR ON READING LOADTTONTOLOGY PROPERTY");
	return load_tt_ontology;
}

/**
 * Returns the restore_tt_ontology.
 * @return boolean
 */
public boolean isRestore_tt_ontology() 
{
	if(this.findvalue==false)
		System.err.println("WARNING ERROR ON READING RESTORETTONTOLOGY PROPERTY");
	return restore_tt_ontology;
}

/**
 * Sets the restore_tt_ontology.
 * @param restore_tt_ontology The restore_tt_ontology to set
 */
public void setRestore_tt_ontology(boolean restore_tt_ontology) {
	this.restore_tt_ontology = restore_tt_ontology;
}

/**
 * Returns the buildST.
 * @return boolean
 */
public boolean isBuildST() {
	if(this.findvalue==false)
		System.err.println("WARNING ERROR ON READING RESTORETTONTOLOGY PROPERTY");
	return buildST;
}

/**
 * Sets the buildST.
 * @param buildST The buildST to set
 */
public void setBuildST(boolean buildST) {
	this.buildST = buildST;
}

/**
 * Returns the restoreComponentsDB.
 * @return boolean
 */
public boolean isRestoreComponentsDB() 
{
	if(this.findvalue==false)
		System.err.println("WARNING ERROR ON READING RESTORETTONTOLOGY PROPERTY");
	return restoreComponentsDB;
}

/**
 * Returns the install.
 * @return String
 */
public String getInstall() {
	return install;
}

/**
 * Returns the java.
 * @return String
 */
public String getJava() {
	return java;
}



/**
   * This method writes a DOM document to a file
   * @param filename
   * @param document
   */
  public static void writeXmlToFile(String filename, Document document) {
    try {
      // Prepare the DOM document for writing
      Source source = new DOMSource(document);

      // Prepare the output file
      File file = new File(filename);
      Result result = new StreamResult(file);

      // Write the DOM document to the file
      // Get Transformer
      Transformer xformer = TransformerFactory.newInstance().newTransformer();
      // Write to a file
      xformer.transform(source, result);
    } catch (TransformerConfigurationException e) {
      System.out.println("TransformerConfigurationException: " + e);
    } catch (TransformerException e) {
      System.out.println("TransformerException: " + e);
    }
  }


}
