package br.usp.talkagent.ide;

/**
 * @author Percival Lucena
 */

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;

import br.usp.talkagent.component.*; 
import br.usp.talkagent.component.Component;
import br.usp.talkagent.component.CompLink;
import br.usp.talkagent.component.CElement;

import java.util.*;


public class ComponentTree 
{

private JTree componentTree;
private CRclient crmc;
private Vector components;

	public	ComponentTree(CRclient crmc)
	{
		super();
		this.crmc = crmc;
		components = crmc.getComponentList();
		componentTree = new JTree(SetupTreeView());
		componentTree.setFont(new Font("Monospaced", 0, 12));
		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) componentTree.getCellRenderer();    
    	renderer.setClosedIcon(null);
    	renderer.setOpenIcon(null);
    	renderer.setLeafIcon(null);    	

	}

	public DefaultMutableTreeNode SetupTreeView() 
	{
		String root="Components (" + components.size() + ")";
       DefaultMutableTreeNode top = new DefaultMutableTreeNode(root);              
       return top;
	}


	public DefaultMutableTreeNode lastItem()
	{
		TreePath  selPath = this.componentTree.getSelectionPath();	   
		if(selPath!=null)
		{
			DefaultMutableTreeNode LastItem = (DefaultMutableTreeNode)selPath.getLastPathComponent();
			return LastItem;			
		}
		else
			return null;
	}


	public DefaultMutableTreeNode nodeAddChildren(DefaultMutableTreeNode parent, DefaultMutableTreeNode child)
	{
		parent.add(child);
		return child;		
	}

	public void setTreeModel(DefaultTreeModel tm)
	{
		this.componentTree.setModel(tm);
	}

	/**
	 * Returns the ontologyTree.
	 * @return JTree
	 */
	public JTree getTree() {
		return componentTree;
	}

	/**
	 * Sets the ontologyTree.
	 * @param ontologyTree The ontologyTree to set
	 */
	public void setTree(JTree Tree) {
		this.componentTree = Tree;
	}
	
	public void buildTree()
	{
		int i,j,k,l,m;
		CompLink cp;
		Component cpnt;
		
		String root = "Components (" + components.size() + ")";		
		DefaultMutableTreeNode tnRoot = new DefaultMutableTreeNode(new NodeData(root, "Root"));
		for(i=0;i<components.size();i++)
		{
			cpnt = (Component) components.elementAt(i);
			cp = (CompLink)  cpnt.getComponentInfo();
			NodeData nd = new NodeData(cp.getName(),"component");
			DefaultMutableTreeNode cpn = new DefaultMutableTreeNode(nd);
			NodeData anc = new NodeData("Ancestor: " + cp.getAncestor(), "ancestor");
			DefaultMutableTreeNode ancestor = new DefaultMutableTreeNode(anc);
			cpn.add(ancestor);
			NodeData ndm = new NodeData("Methods:","label");
			DefaultMutableTreeNode label = new DefaultMutableTreeNode(ndm);
			cpn.add(label);
			// for each method ...												
			for(j=0;j<cp.getNumeberofMethods();j++)
			{
				String value = "public " + cp.getMethod(j).getReturntype() + " " +  cp.getMethod(j).getName();				 								
				DefaultMutableTreeNode node = nodeAddChildren(label,new DefaultMutableTreeNode(new NodeData(value,"method")));
				Vector mconcepts = cp.getMethod(j).getAssociatedConcepts();				
				if(mconcepts!=null)				
				{
					String label0 ="Associated Concepts (" + mconcepts.size() + ")";
					NodeData ndata0 = new NodeData(label0, "other");
					ndata0.setColor(Color.BLUE);
					DefaultMutableTreeNode mc = new DefaultMutableTreeNode(ndata0); 					
					nodeAddChildren(node,mc);								
					for(l=0;l<mconcepts.size();l++)
					{
						NodeData ndata00 = new NodeData((String)mconcepts.elementAt(l), "link");
						ndata00.setColor(Color.GREEN);
						nodeAddChildren(mc,new DefaultMutableTreeNode(ndata00));
					}
				}
				// for each parameter in each method...				
				for(k=0;k<cp.getMethod(j).getParametersNumber();k++)
				{					
					String data = cp.getMethod(j).getParameter(k).toString();
					NodeData ndata = new NodeData(data, "param");					
					DefaultMutableTreeNode nparam = nodeAddChildren(node,new DefaultMutableTreeNode(ndata));
					Vector pconcepts = cp.getMethod(j).getParameter(k).getAssociatedConcepts();
					if(pconcepts!=null)
					{
						String label00 = "Associated Concepts (" + pconcepts.size() + ")";
						NodeData ndata2 = new NodeData(label00, "other");
						ndata2.setColor(Color.BLUE);						
						DefaultMutableTreeNode pc = new DefaultMutableTreeNode(ndata2);				
						DefaultMutableTreeNode ca = nodeAddChildren(nparam,pc);
						for(m=0;m<pconcepts.size();m++)
						{
							NodeData ndata000 = new NodeData((String)pconcepts.elementAt(m), "link");
							ndata000.setColor(Color.GREEN);
							nodeAddChildren(ca,new DefaultMutableTreeNode(ndata000));
						}
					}
						
				}				
			}					
			tnRoot.add(cpn);
		}		      			
		DefaultTreeModel treeModel = new DefaultTreeModel(tnRoot);
       	setTreeModel(treeModel);
	}

	/* Returns a CElement if node is a method or a paramter */
	public CElement getElement(DefaultMutableTreeNode node)
	{
		
		NodeData currel = (NodeData) node.getUserObject();
		CElement el = new CElement();
		if(currel.getType() == "method")
		{
			el.setType("method");
			el.setMethod(currel.getValue());
			DefaultMutableTreeNode parent = (DefaultMutableTreeNode) node.getParent();			
			DefaultMutableTreeNode ncomponent = (DefaultMutableTreeNode) parent.getParent();
			NodeData component = (NodeData) ncomponent.getUserObject();
			el.setComponent(component.getValue());
			return el;			 		
		}
		if(currel.getType() == "param")
		{
			el.setType("param");
			el.setParam(currel.getValue());
			DefaultMutableTreeNode parent = (DefaultMutableTreeNode) node.getParent();			
			NodeData methodvalue = (NodeData) parent.getUserObject();
			el.setMethod(methodvalue.getValue());
			DefaultMutableTreeNode nmethod = (DefaultMutableTreeNode) parent.getParent();				
			DefaultMutableTreeNode grandparent = (DefaultMutableTreeNode) nmethod.getParent();
			NodeData gparentn = (NodeData) grandparent.getUserObject();
			el.setComponent(gparentn.getValue());
			return el;
		} 
		return null;
	}
}
