package br.usp.talkagent.ide;

import javax.swing.tree.DefaultMutableTreeNode;
import java.awt.Color;
import java.awt.Font;
import java.awt.Toolkit;
import java.util.Random;
import br.usp.talkagent.kb.*;
import br.usp.talkagent.kb.objects.*;
import br.usp.talkagent.component.*;
import java.util.*;
import javax.swing.*;

/**
  * DynamicTreeNode illustrates one of the possible ways in which dynamic
  * loading can be used in tree.  The basic premise behind this is that
  * getChildCount() will be messaged from JTreeModel before any children
  * are asked for.  So, the first time getChildCount() is issued the
  * children are loaded.<p>
  * It should be noted that isLeaf will also be messaged from the model.
  * The default behavior of TreeNode is to message getChildCount to
  * determine this. As such, isLeaf is subclassed to always return false.<p>
  * There are others ways this could be accomplished as well.  Instead of
  * subclassing TreeNode you could subclass JTreeModel and do the same
  * thing in getChildCount().  Or, if you aren't using TreeNode you could
  * write your own TreeModel implementation.
  * Another solution would be to listen for TreeNodeExpansion events and
  * the first time a node has been expanded post the appropriate insertion
  * events.  I would not recommend this approach though, the other two
  * are much simpler and cleaner (and are faster from the perspective of
  * how tree deals with it).
  *
  * NOTE: getAllowsChildren() can be messaged before getChildCount().
  *       For this example the nodes always allow children, so it isn't
  *       a problem, but if you do support true leaf nodes you may want
  *       to check for loading in getAllowsChildren too.
  *
  * @version 1.3 09/23/97
  * @author Scott Violet
  */

public class DynamicTreeNode extends DefaultMutableTreeNode
{
	// Class stuff.
	/** Number of names. */
	static protected float                    nameCount;

	/** Names to use for children. */
	static protected String[]                 names;

	/** Potential fonts used to draw with. */
	static protected Font[]                   fonts;

	/** Used to generate the names. */
	static protected Random                   nameGen;

	/** Number of children to create for each node. */
	//static protected final int                DefaultChildrenCount = 5;
	static int                DefaultChildrenCount = 0;

	//Add - Carlos
	static String s_nodefather = "";
	static Vector v_childs;


	static {
	String[]            fontNames;

	try {
	    fontNames = Toolkit.getDefaultToolkit().getFontList();
	} catch (Exception e) {
	    fontNames = null;
	}
	if(fontNames == null || fontNames.length == 0) {
	    names = new String[] {"Mark Andrews", "Tom Ball", "Alan Chung",
				      "Rob Davis", "Jeff Dinkins",
				      "Amy Fowler", "James Gosling",
				      "David Karlton", "Dave Kloba", 
				      "Dave Moore", "Hans Muller",
				      "Rick Levenson", "Tim Prinzing",
				      "Chester Rose", "Ray Ryan",
				      "Georges Saab", "Scott Violet",
				      "Kathy Walrath", "Arnaud Weber" };
	}
	else {
	    /* Create the Fonts, creating fonts is slow, much better to
	       do it once. */
	    int              fontSize = 12;

	    names = fontNames;
	    fonts = new Font[names.length];
	    for(int counter = 0, maxCounter = names.length;
		counter < maxCounter; counter++) {
		try {
		    fonts[counter] = new Font(fontNames[counter], 0, fontSize);
		}
		catch (Exception e) {
		    fonts[counter] = null;
		}
		fontSize = ((fontSize + 2 - 12) % 12) + 12;
	    }
	}
	nameCount = (float)names.length;
	nameGen = new Random(System.currentTimeMillis());
	}


	/** Have the children of this node been loaded yet? */
	protected boolean           hasLoaded;

	/**
	  * If hasLoaded is false, meaning the children have not yet been
	  * loaded, loadChildren is messaged and super is messaged for
	  * the return value.
	  */
	public int getChildCount() 
	{
	if(!hasLoaded) {
	    loadChildren();
	}
	return super.getChildCount();
	}
	
	
	public boolean isLeaf() {
	return false;
	}

	/**
	  * Constructs a new DynamicTreeNode instance with o as the user
	  * object.
	  */
public DynamicTreeNode(Object o) 
{
	super(o);
}

/*
public DynamicTreeNode(Object o, String toolTipText) 
{
	super(o);
}
*/
	/**
	  * Messaged the first time getChildCount is messaged.  Creates
	  * children with random names from names.
	  */
	protected void loadChildren() 
	{
	DynamicTreeNode             newNode;
	Font                        font;
	int                         randomIndex;
	NodeData                  	data;
	ozConcept 					cpt;
	int							ct = 0;
	boolean 					haschildren	= false;

	System.out.println( ">>" + this.toString() );
	
	s_nodefather = this.toString();
	try 
	{		
		v_childs = IDEform.xktc.getChildrenVector(s_nodefather);				
		if(v_childs!=null)		
		DynamicTreeNode.DefaultChildrenCount = v_childs.size();
	} 
	catch (Exception e) 
	{
		e.printStackTrace();
	}

	font = fonts[0];
	
	if( v_childs  != null)
	{		
		int size = DynamicTreeNode.DefaultChildrenCount;

		int idx=0;	
				
		//for(int counter = 0; counter < size; counter++)
		 Iterator iterator = v_childs.iterator();
		 int counter =0;			
		 while ( iterator.hasNext() )  
		 {
		    randomIndex = (int)(nameGen.nextFloat() * nameCount);		
			
			// String item = ((ozConcept) v_childs.elementAt(counter)).getID();
			String item = ((ozConcept) iterator.next()).getID();			
			cpt = IDEform.xktc.getConcept(item);	
			if(cpt!=null)
			{
				idx++;
				String nd = IDEform.xktc.getNumberOfChildren(item) + " items \n";
				int na  = cpt.getNumberOfAssertions();
				int nc = cpt.getComponetLinks().size();
				nd += na + " assertions";			
				data = new NodeData(font, Color.black, item, nd);
				Icon ic = new ImageIcon("../ide/images/concept.gif");
				data.setIcon(ic);					    
			    newNode = new DynamicTreeNode(data);		   		
				ct = counter;								
			    /* Don't use add() here, add calls insert(newNode, getChildCount())
		        so if you want to use add, just be sure to set hasLoaded = true
		        first. */
			    insert(newNode, counter);
    			haschildren=true;
    			counter++;
			}
			else
			{
				String nd = "!" + item + "!";
				data = new NodeData(font, Color.RED, nd, nd);									    
			    newNode = new DynamicTreeNode(data);		   		
			    insert(newNode, counter);
			    haschildren=false;
				System.err.println("WARNING@IDE - Trying to get invalid concept " + item);
			}
		 }
	}
	

	 int i;
	 if(!s_nodefather.equalsIgnoreCase("ontologies"))
	 {
	 cpt = IDEform.xktc.getConcept(s_nodefather);
  	 String assertion;
  	 int na;
  	 int nc=0;
  	 Vector Assertions=null;
  	 Vector Components=null; 
  	 if(cpt!=null)
  	 {
	  	  na = cpt.getNumberOfAssertions();
	  	  Assertions = cpt.getAssertions();
	  	  Components = cpt.getComponetLinks();
	  	  nc = Components.size();	  	   
  	 }
	 else
	     na = 0;
  	 
  	 
	 for(i=0;i<na;i++)
	 {
	 		
			ozAssertion ozA = (ozAssertion) Assertions.elementAt(i);
			if(ozA != null)
			{ 							
				assertion = ozA.getType();
				Relation rl =ozA.getvalue(); 
				assertion += ":" + rl.getType();
				assertion += ":" + rl.getValue().toString();
				NodeData dataAst = new NodeData(font, Color.BLUE, assertion, "-");
				Icon ic = new ImageIcon("../ide/images/assertion.gif");
				dataAst.setIcon(ic);				    										
			    newNode = new DynamicTreeNode(dataAst); 			    
			    if(haschildren)			      				
					insert(newNode, ct+1);
				else
				
					insert(newNode, ct);
			}								
	 }
	 
	 for(i=0;i<nc;i++)
	 {
	 	CElement cel = (CElement)Components.elementAt(i);
	 	if(cel!=null)
	 	{
	 		String ndata = cel.getComponent() + ":" + cel.getMethod();
	 		if(cel.getParam()!=null)
	 		{
	 			ndata+= ":" + cel.getParam();
	 		}
	 		NodeData dataComp = new NodeData(font, Color.GRAY, ndata, "-", "Component");	 		
			Icon ic = new ImageIcon("../ide/images/component.gif");
			dataComp.setIcon(ic);
	 		newNode = new DynamicTreeNode(dataComp); 			    
			 if(haschildren)			      				
				insert(newNode, ct+1);
			else
			
				insert(newNode, ct);
	 	}
	 	  
	 }
	 			
	 }	
	
	/* This node has now been loaded, mark it so. */
	hasLoaded = true;
	}
}
