/**
 *
 *  Java FTP client library.
 *
 *  Copyright (C) 2000  Enterprise Distributed Technologies Ltd
 *
 *  www.enterprisedt.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Bug fixes, suggestions and comments should be sent to:
 *
 *  bruceb@cryptsoft.com
 *
 *  or by snail mail to:
 *
 *  Bruce P. Blackshaw
 *  53 Wakehurst Road
 *  London SW11 6DB
 *  United Kingdom
 *
 *  Change Log:
 *
 *      $Log: TAFTP.java,v $
 *      Revision 1.1  2002/11/27 16:24:48  cvs
 *      no message
 *
 *      Revision 1.1  2002/11/14 21:20:19  cvs
 *      no message
 *
 *      Revision 1.2  2001/10/09 20:54:36  bruceb
 *      Active mode testing
 *
 */

package com.enterprisedt.net.ftp;

import java.io.IOException;
import java.util.*;


/**
 *  Crude test harness.
 *
 *    TO DO: expand this!
 *
 *  @author             Bruce Blackshaw
 *  @version        $Revision: 1.1 $
 *
 */
public class TAFTP {

    /**
     *  Revision control id
     */
    private static String cvsId = "$Id: TAFTP.java,v 1.1 2002/11/27 16:24:48 cvs Exp $";
    private String host;
    private int port;
    

    public TAFTP(String host, int port)
    {
    	this.host = host;
    	this.port = port;
    }
    
    
    public void transfer(String file) 
    {       
        try {

            // assign args to make it clear                        
            String user = "talkagent";
            String password = "talkagent";
            String mode = "BINARY";
            String connMode = "PASV";            
            String directory="";
            String filename="";
            int i;

			
			Vector path = new Vector();
					
			StringTokenizer st = new StringTokenizer(file, "/", true);
			while(st.hasMoreElements())		
				path.add(st.nextToken());
			
			int size = path.size();
			if(size > 1)		
			{
            	filename = (String) path.elementAt(size-1);
            	for(i=0;i<size-1;i++)	
            		directory += (String) path.elementAt(i); 
			}
			
			
			
            // connect and test supplying port no.
            FTPClient ftp = new FTPClient(this.host, this.port);
            ftp.login(user, password);

            // switch on debug of responses
            ftp.debugResponses(true);
            

            // binary transfer
            if (mode.equalsIgnoreCase("BINARY")) {
                ftp.setType(FTPTransferType.BINARY);
            }
            else if (mode.equalsIgnoreCase("ASCII")) {
                ftp.setType(FTPTransferType.ASCII);
            }
            else {
                System.out.println("Unknown transfer type: " +mode);
                System.exit(-1);
            }

            // PASV or active?
            if (connMode.equalsIgnoreCase("PASV")) {
                ftp.setConnectMode(FTPConnectMode.PASV);
            }
            else if (connMode.equalsIgnoreCase("ACTIVE")) {
                ftp.setConnectMode(FTPConnectMode.ACTIVE);
            }
            else {
                System.out.println("Unknown connect mode: " + connMode);
                System.exit(-1);
            }

            // change dir
            //ftp.chdir(directory);
		
            // put a local file to remote host
            ftp.put(file, filename);
            

			/*
            // get bytes
            byte[] buf = ftp.get(filename);
            System.out.println("Got " + buf.length + " bytes");

            // append local file
            try {
                ftp.put(filename, filename, true);
            }
            catch (FTPException ex) {
                System.out.println("Append failed: " + ex.getMessage());
            }

            // get bytes again - should be 2 x
            buf = ftp.get(filename);
            System.out.println("Got " + buf.length + " bytes");

            // rename
            ftp.rename(filename, filename + ".new");

            // get a remote file - the renamed one
            ftp.get(filename + ".tst", filename + ".new");
			*/
	
            // ASCII transfer
            ftp.setType(FTPTransferType.ASCII);

            // test that list() works
            String listing = ftp.list(".");
            System.out.println(listing);

            // test that dir() works in full mode
            String[] listings = ftp.dir(".", true);
            for (i = 0; i < listings.length; i++)
                System.out.println(listings[i]);

            // try system()
            System.out.println(ftp.system());

            // try pwd()
            System.out.println(ftp.pwd());

            ftp.quit();
        }
        catch (IOException ex) {
            System.out.println("Caught exception: " + ex.getMessage());
        }
        catch (FTPException ex) {
            System.out.println("Caught exception: " + ex.getMessage());
        }
    }


    /**
     *  Basic usage statement
     */
    public static void usage() {

        System.out.println("Usage: ");
        System.out.println("com.enterprisedt.net.ftp.FTPClientTest " +
                           "");
    }

}
