package de.cinek.rssview;

import javax.swing.*;
import java.net.*;
import java.util.*;
import de.cinek.rssview.images.IconContainer;

public class RssChannelModel extends DefaultListModel {

    public static final String GLOBAL_CHANNEL="All news";
    private Vector channels;
    private RssChannel exclusivechannel=null;

    public RssChannelModel() {
	channels=new Vector();

	initDefaultChannels();
    }

    private void initDefaultChannels() {
	addGlobalChannel();

    }

    public int getSize() {
	if (exclusivechannel==null) {
	    return channels.size();
	} else {
	    return 1;
	}
    }

    public Object getElementAt(int index) {
	if (exclusivechannel==null) {
	    if (index<0 || index>=channels.size()) return null;
	    return channels.elementAt(index);
	} else {
	    return index==0 ? exclusivechannel : null;
	}
    }

    public void addGlobalChannel() {
	addChannel(GLOBAL_CHANNEL,null,0,0,true,false);
	RssChannel channel=(RssChannel)getElementAt(0);
	channel.setTitle(" - Version "+RssView.APP_VER);
	channel.setDescription("A Java RSS/RDF ticker for You.");
	//channel.setIconFromImageIcon(IconContainer.APP_LOGO);
    }

    public void addChannel(String name,String url,int articlecount,
			   int pollinterval,boolean showtext,boolean beep) {
	channels.add(new RssChannel(name,url,articlecount,pollinterval,
				    showtext,beep));
	fireContentsChanged(this,getSize()-1,getSize()-1);
    }

    public void removeChannels(RssChannel channel[]) {

	if (channel==null) return;

	for (int i=0;i<channel.length;i++) {
	    int j=channels.indexOf(channel[i]);

	    if (j>0 && j<channels.size()) {
		channel[i].cleanUp();
		channels.remove(j);
	    }	    
	}

	fireContentsChanged(this,0,getSize()-1);
    }

    public RssChannel[] getChannels() {
	if (exclusivechannel==null) {
	    return (RssChannel [])channels.toArray(new RssChannel[0]);
	} else {
	    RssChannel[] chans=new RssChannel[1];
	    chans[0]=exclusivechannel;
	    return chans;
	}
    }

    public void moveUp(int index) {
	if (index>1) {
	    RssChannel upper=(RssChannel)getElementAt(index-1);
	    RssChannel tomove=(RssChannel)getElementAt(index);

	    channels.setElementAt(tomove,index-1);
	    channels.setElementAt(upper,index);
	}
    }

    public void moveDown(int index) {
	if ((index<getSize()-1) && (index>0)){
	    RssChannel tomove=(RssChannel)getElementAt(index);
	    RssChannel bottom=(RssChannel)getElementAt(index+1);

	    channels.setElementAt(tomove,index+1);
	    channels.setElementAt(bottom,index);
	}
    }
    
    public RssChannel getChannelForURL(String url) {
	    Enumeration e=channels.elements();
	    while (e.hasMoreElements()) {
		    RssChannel c=(RssChannel)e.nextElement();
		    if (c.isGlobal()) continue;
		    if (c.getURL().equals(url)) return c;
	    }
	    return null;
    }

    public void setExclusiveMode(String url) {
	exclusivechannel=new RssChannel(url,url,-1,
					RssSettings.DEFAULT_POLLINTERVAL,
					true,false);
    }

    public boolean isExclusiveMode() {
	return exclusivechannel!=null;
    }

    public void setInteractiveMode() {
	exclusivechannel=null;
    }
}
