package de.cinek.rssview;

import java.awt.Dimension;
import java.awt.Point;
import java.io.File;

public class RssSettings {

    private static final String settingsfilename=
	System.getProperty("user.home")+File.separator+".rssviewrc";
    private static IniFile settingsfile=null;

    public static final int DEFAULT_ARTICLESCOUNT=20;
    public static final int DEFAULT_POLLINTERVAL=15*60; /* 15 minutes */
    public static final boolean DEFAULT_SHOWTEXT=true;
    public static final boolean DEFAULT_BEEP=false;
    public static final String DEFAULT_BROWSER="mozilla";
    public static final String DEFAULT_BROWSERPARAMS="%u";

    /* will be moved to channel class !!! */
    private static int updateinterval=DEFAULT_POLLINTERVAL;
    private static int articlecount=DEFAULT_ARTICLESCOUNT;

    private static final String WINDOWSECTION="Window";
    private static final String CHANNELSSECTION="Channels";
    private static final String EXTERNALSECTION="External";
    
    private static int divider1=96;
    private static int divider2=128;
    private static int exclusivedivider1=96;
    private static int exclusivedivider2=128;

    private static int viewsize=RssView.VIEWSIZE_NORMAL;

    private static int smallwidth=256;
    private static int smallheight=384;
    private static int smallx=50;
    private static int smally=50;

    private static int normalwidth=256;
    private static int normalheight=384;
    private static int normalx=50;
    private static int normaly=50;

    private static int exclusivewidth=256;
    private static int exclusiveheight=384;
    private static int exclusivex=50;
    private static int exclusivey=50;

    private static String browserpath=DEFAULT_BROWSER;
    private static String browserparams=DEFAULT_BROWSERPARAMS;

    public static int getDivider1() {
	return divider1;
    }
    public static void setDivider1(int v) {
	divider1=v;
    }

    public static int getExclusiveDivider1() {
	return exclusivedivider1;
    }
    public static void setExclusiveDivider1(int v) {
	exclusivedivider1=v;
    }

    public static int getDivider2() {
	return divider2;
    }
    public static void setDivider2(int v) {
	divider2=v;
    }

    public static int getExclusiveDivider2() {
	return exclusivedivider2;
    }
    public static void setExclusiveDivider2(int v) {
	exclusivedivider2=v;
    }

    public static int getViewSize() {
	return viewsize;
    }
    public static void setViewSize(int v) {
	viewsize=v;
    }

    public static Dimension getSmallViewSize() {
	return new Dimension(smallwidth,smallheight);
    }
    public static void setSmallViewSize(Dimension d) {
	smallwidth=d.width;
	smallheight=d.height;
    }

    public static Dimension getNormalViewSize() {
	return new Dimension(normalwidth,normalheight);
    }
    public static void setNormalViewSize(Dimension d) {
	normalwidth=d.width;
	normalheight=d.height;
    }

    public static Dimension getExclusiveViewSize() {
	return new Dimension(exclusivewidth,exclusiveheight);
    }
    public static void setExclusiveViewSize(Dimension d) {
	exclusivewidth=d.width;
	exclusiveheight=d.height;
    }

    public static Point getSmallViewPosition() {
	return new Point(smallx,smally);
    }
    public static void setSmallViewPosition(Point p) {
	smallx=p.x;
	smally=p.y;
    }

    public static Point getNormalViewPosition() {
	return new Point(normalx,normaly);
    }
    public static void setNormalViewPosition(Point p) {
	normalx=p.x;
	normaly=p.y;
    }

    public static Point getExclusiveViewPosition() {
	return new Point(exclusivex,exclusivey);
    }
    public static void setExclusiveViewPosition(Point p) {
	exclusivex=p.x;
	exclusivey=p.y;
    }

    public static String getBrowserPath() {
	return (browserpath==null || browserpath.length()<1) ?
	    DEFAULT_BROWSER : browserpath;
    }
    public static void setBrowserPath(String path) {
	if (path==null || path.length()<1) browserpath=DEFAULT_BROWSER;
	else browserpath=path;
    }

    public static String getBrowserParams() {
	return browserparams==null ? "" : browserparams;
    }
    public static void setBrowserParams(String params) {
	browserparams=params==null ? "" : params;
    }

    public static void loadSettings() {
	settingsfile=new IniFile(settingsfilename);

	settingsfile.readSettings();
	divider1=settingsfile.getInt(WINDOWSECTION,"divider1",96);
	divider2=settingsfile.getInt(WINDOWSECTION,"divider2",128);
	exclusivedivider1=settingsfile.getInt
	    (WINDOWSECTION,"divider1.exclusive",96);
	exclusivedivider2=settingsfile.getInt
	    (WINDOWSECTION,"divider2.exclusive",128);

	viewsize=settingsfile.getInt(WINDOWSECTION,"viewsize",
				     RssView.VIEWSIZE_NORMAL);

	smallx=settingsfile.getInt(WINDOWSECTION,"x.small",50);
	smally=settingsfile.getInt(WINDOWSECTION,"y.small",50);
	smallwidth=settingsfile.getInt(WINDOWSECTION,"width.small",256);
	smallheight=settingsfile.getInt(WINDOWSECTION,"height.small",384);

	normalx=settingsfile.getInt(WINDOWSECTION,"x.normal",50);
	normaly=settingsfile.getInt(WINDOWSECTION,"y.normal",50);
	normalwidth=settingsfile.getInt(WINDOWSECTION,"width.normal",256);
	normalheight=settingsfile.getInt(WINDOWSECTION,"height.normal",384);

	exclusivex=settingsfile.getInt(WINDOWSECTION,"x.exclusive",50);
	exclusivey=settingsfile.getInt(WINDOWSECTION,"y.exclusive",50);
	exclusivewidth=settingsfile.getInt(WINDOWSECTION,"width.exclusive",
					   256);
	exclusiveheight=settingsfile.getInt(WINDOWSECTION,"height.exclusive",
					    384);

	browserpath=settingsfile.getString(EXTERNALSECTION,
					   "browser.path",DEFAULT_BROWSER);
	browserparams=settingsfile.getString(EXTERNALSECTION,"browser.params",
					     DEFAULT_BROWSERPARAMS);

	if (!RssArticleContainer.getChannelModel().
	    isExclusiveMode()) {

	    RssChannel globalchannel=
		(RssChannel)RssArticleContainer.getChannelModel().
		getElementAt(0);
	    if (globalchannel!=null) {
		globalchannel.setArticlesCount
		    (settingsfile.getInt(CHANNELSSECTION,
					 "globalchannel.articlescount",
					 DEFAULT_ARTICLESCOUNT));
		globalchannel.setShowText
		    (settingsfile.getBool(CHANNELSSECTION,
					  "globalchannel.showtext",
					  DEFAULT_SHOWTEXT));
	    }
	    
	    int i=1;
	    do {
		String name=settingsfile.getString
		    (CHANNELSSECTION,"channel"+i+".name",null);
		String url=settingsfile.getString
		    (CHANNELSSECTION,"channel"+i+".url",null);
		int articlescount=settingsfile.getInt
		    (CHANNELSSECTION,"channel"+i+".articlescount",
		     DEFAULT_ARTICLESCOUNT);
		int pollinterval=settingsfile.getInt
		    (CHANNELSSECTION,"channel"+i+".pollinterval",
		     DEFAULT_POLLINTERVAL);
		boolean showtext=settingsfile.getBool
		    (CHANNELSSECTION,"channel"+i+".showtext",
		     DEFAULT_SHOWTEXT);
		boolean beep=settingsfile.getBool
		    (CHANNELSSECTION,"channel"+i+".beep",
		     DEFAULT_BEEP);
		if (name==null) break;
		
		RssArticleContainer.getChannelModel().addChannel
		    (name,url,articlescount,pollinterval,showtext,beep);
		
		i++;
	    } while (true);
	}
    }

    public static void saveSettings() {
	/*if (RssArticleContainer.getChannelModel().
	  isExclusiveMode()) return;

	  IniFile settingsfile=new IniFile(settingsfilename);*/

	settingsfile.setInt(WINDOWSECTION,"divider1",divider1);
	settingsfile.setInt(WINDOWSECTION,"divider2",divider2);
	settingsfile.setInt(WINDOWSECTION,"divider1.exclusive",
			    exclusivedivider1);
	settingsfile.setInt(WINDOWSECTION,"divider2.exclusive",
			    exclusivedivider2);

	settingsfile.setInt(WINDOWSECTION,"viewsize",viewsize);

	settingsfile.setInt(WINDOWSECTION,"x.small",smallx);
	settingsfile.setInt(WINDOWSECTION,"y.small",smally);
	settingsfile.setInt(WINDOWSECTION,"width.small",smallwidth);
	settingsfile.setInt(WINDOWSECTION,"height.small",smallheight);

	settingsfile.setInt(WINDOWSECTION,"x.normal",normalx);
	settingsfile.setInt(WINDOWSECTION,"y.normal",normaly);
	settingsfile.setInt(WINDOWSECTION,"width.normal",normalwidth);
	settingsfile.setInt(WINDOWSECTION,"height.normal",normalheight);

	settingsfile.setInt(WINDOWSECTION,"x.exclusive",exclusivex);
	settingsfile.setInt(WINDOWSECTION,"y.exclusive",exclusivey);
	settingsfile.setInt(WINDOWSECTION,"width.exclusive",exclusivewidth);
	settingsfile.setInt(WINDOWSECTION,"height.exclusive",exclusiveheight);

	settingsfile.setString(EXTERNALSECTION,"browser.path",browserpath);
	settingsfile.setString(EXTERNALSECTION,"browser.params",browserparams);

	if (!RssArticleContainer.getChannelModel().
	    isExclusiveMode()) {

	    RssChannelModel rcm=RssArticleContainer.getChannelModel();
	    
	    RssChannel globalchannel=(RssChannel)rcm.getElementAt(0);
	    if (globalchannel!=null) {
		settingsfile.setInt(CHANNELSSECTION,
				    "globalchannel.articlescount",
				    globalchannel.getArticlesCount());
		settingsfile.setBool(CHANNELSSECTION,
				     "globalchannel.showtext",
				     globalchannel.getShowText());
	    }


	    for (int i=1;i<rcm.getSize();i++) {
		RssChannel channel=(RssChannel)rcm.getElementAt(i);
		settingsfile.setString(CHANNELSSECTION,
				       "channel"+i+".name",channel.getName());
		settingsfile.setString(CHANNELSSECTION,
				       "channel"+i+".url",channel.getURL());
		settingsfile.setInt(CHANNELSSECTION,
				    "channel"+i+".articlescount",
				    channel.getArticlesCount());
		settingsfile.setInt(CHANNELSSECTION,
				    "channel"+i+".pollinterval",
				    channel.getPollInterval());
		settingsfile.setBool(CHANNELSSECTION,
				     "channel"+i+".showtext",
				     channel.getShowText());
		settingsfile.setBool(CHANNELSSECTION,
				     "channel"+i+".beep",
				     channel.getBeep());
	    }
	}

	settingsfile.saveSettings();
    }
}
