//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.ont.fipa.fipaman;

//----------------------------------------------------------------------

import java.util.Set;
import java.util.List;
import java.io.Serializable;

import fipaos.ont.BASETOKENS;
import fipaos.agent.conversation.content.Content;
import fipaos.parser.ParserException;
import fipaos.parser.sl.SLParser;
import fipaos.util.DIAGNOSTICS;

//----------------------------------------------------------------------

/** This class represents the Agent Platform Description object and contains a APTDescription which represents the Agent Platform Transport Description object
 * TODO: there is at present to checking to see if the characters in the fields are valid as per the BNF definitions
 * <br>
 * <br>
 */
public class APDescription implements Serializable
{

	//----------------------------------------------------------------------
	// CONSTANTS
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	// STATIC VARIABLES
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	// INSTANCE VARIABLES
	//----------------------------------------------------------------------

	/** The name of the AP
	 */
	private String _name;

	/** The support for dynamic registration of the AP(optional)
	 */
	private Boolean _dynamic;

	/** The support for mobility of the AP (optional)
	 */

	private Boolean _mobility ;

	/** The description MTS capabilities of the AP (optional
	 */
	private APTDescription _transport_description;

	//----------------------------------------------------------------------
	// CONSTRUCTOR(S)
	//----------------------------------------------------------------------

	/** This constructor creates an empty Agent Platform Descriptioon.
	 */
	public APDescription() {
		_transport_description = new APTDescription();
	}

	/** This constructor creates an APDescriptoin from a string.
	 * @param description string representation of an APDescription
	 * @throws ParserException if it does not parse correctly
	 */

	public APDescription(String description) throws ParserException {

		try {
			parse(SLParser.parseTerm(description));
		}
		catch (fipaos.parser.sl.ParseException e) {
			throw new ParserException(e.getMessage());
		}
	}

	/** This constructor takes a Content object as a parameter.
	 * @param ap_content Content object created by the fipao.parser.sl.SLParser
	 * @throws ParserException if the String is invalid
	 */
	public APDescription(Content ap_content) throws ParserException {
		
		parse(ap_content);
	}


	private void parse(Content ap_content) throws IllegalArgumentException, ParserException {

		if (ap_content!=null) {

			boolean mandatorySatisfied=false;

			Content functional_content=(Content) ap_content.getAttribute("functionalTerm");

			if (functional_content==null || !functional_content.getAttribute("functionalOperator").equals(FIPAMANCONSTANTS.AP_DESCRIPTION)) {
				throw new IllegalArgumentException("Content is not a AP-DESCRIPTION");
			}

			Object parameters=functional_content.getAttribute("parameter");

			if (parameters instanceof List) {

				for (int i=0;i<((List) parameters).size();i++) {

					Content content=(Content) ((List) parameters).get(i);

					if (content.getName()!="parameter") {
						throw new IllegalArgumentException("Illegal AP-DESCRIPTION FORMAT");
					}

					String name=((String) content.getAttribute("parameterName"));

					String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");

					if (name.equals(FIPAMANCONSTANTS.NAME)) {

						mandatorySatisfied=true;

						_name=value;
					}
					else if (name.equals(FIPAMANCONSTANTS.DYNAMIC)) {

						_dynamic=new Boolean(value);
					}
					else if (name.equals(FIPAMANCONSTANTS.MOBILITY)) {

						_mobility=new Boolean(value);
					}
					else if (name.equals(FIPAMANCONSTANTS.TRANSPORT_PROFILE)) {

						_transport_description=new APTDescription(value);

					}
				}
			}
			else {

				String name=((String) ((Content) parameters).getAttribute("parameterName"));

				String value=(String) ((Content) ((Content) parameters).getAttribute("parameterValue")).getAttribute("formattedString");

				if (name.equals(FIPAMANCONSTANTS.NAME)) {

					mandatorySatisfied=true;

					_name=value;
				}
			}

			if (!mandatorySatisfied) {
				throw new IllegalArgumentException("AP Description must contain name attribute");
			}
		}
	}

	//----------------------------------------------------------------------
	// PRIVATE METHODS
	//----------------------------------------------------------------------

	private void addKeywordValuePair(StringBuffer buffer, String keyword, String value)
	{
		buffer.append(keyword);
		buffer.append(BASETOKENS.SPACE);
		buffer.append(value);
		buffer.append(BASETOKENS.NEWLINE);
	}

	//----------------------------------------------------------------------
	// PROTECTED METHODS
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	// PUBLIC METHODS
	//----------------------------------------------------------------------

	/** This method sets the name of the Agent Platform Description.
	 * @param name The name of the AP
	 */
	public void setName(String name) {
		_name = name;
	}

	/** Set whether the platform supports dynamic registration of Agents
	 * @param dynamic true if the platform supports dynamic registration
	 */
	public void setDynamic(Boolean dynamic) {
		_dynamic = dynamic;
	}

	/** Set whether the platform supports mobility of the AP
	 * @param mobility set to true if the agent platform supports mobility
	 */
	public void setMobility(Boolean mobility) {
		_mobility = mobility;
	}

	/** Set the description MTPDescription capabilities of the AP
	 * @param transport_description the agent transport description object
	 */

	public void setMTPDs(Set transport_description) {
		_transport_description.setMTPSSet( transport_description );
	}

	/** This method sets the transport profile of the Agent Platform
	 * @param transport_description Transport Profile
	 */
	public void setTransportDescription(APTDescription transport_description) {
		_transport_description=transport_description;
	}

	/** Get the name of the AP
	 * @return the name of the AP
	 */
	public String getName() {
		return _name;
	}

	/** Get the dynamic registration capability of the platform
	 * @return true if the platform supports dynamic registration capability - false if not
	 */
	public Boolean getDynamic() {
		return _dynamic;
	}

	/** Get the mobility capability of the platform
	 * @return true if the platform supports mobility
	 */
	public Boolean getMobility() {
		return _mobility;
	}

	/** Get the Agent Transport Description
	 * @return the Agent Transport Description
	 */
	public Set getMTPDs() {
		return _transport_description.getMTPSSet();
	}

	/** This method returns the transport profile of the Agent Platform.
	 * @return transport profile of the AP
	 */
	public APTDescription getTransportDescription() {
		return _transport_description;
	}

	/** Adds a MTPDescription to the list of MTPDs
	 * @param mtpd The MTPD to be added to the list
	 * @return as per MTPDescription.addMTPD
	 */
	public boolean addMTPD(MTPDescription mtpd) {
		return _transport_description.addMTPD( mtpd );
	}

	/** remove a Message Transport Protocol Description from the list
	 * @param mtpd the MTPD that needs to be removed
	 * @return as per the MTPDescription.removeMTPD()
	 */

	public boolean removeMTPD(MTPDescription mtpd) {
		return _transport_description.removeMTPD( mtpd );
	}

	/** A string representation of this APDescription
	 * @return A string representation of this APDescription
	 */
	public String toString() {

		StringBuffer buffer=new StringBuffer();
		buffer.append(BASETOKENS.OPENBRACKET);
		buffer.append(FIPAMANCONSTANTS.AP_DESCRIPTION);
		buffer.append(BASETOKENS.NEWLINE);

		if (_name !=null ) {
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.NAME,_name);
		}

		if (_dynamic !=null ) {
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.DYNAMIC,_dynamic.toString());
		}

		if (_mobility !=null ) {
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.MOBILITY,_mobility.toString());
		}

		if (_transport_description !=null ) {
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.TRANSPORT_PROFILE,_transport_description.toString());
		}

		buffer.append(BASETOKENS.CLOSEBRACKET);

		return buffer.toString();
	}

	//----------------------------------------------------------------------
	// MAIN/TESTING METHOD
	//----------------------------------------------------------------------

	/** This main method is used for testing.
	 * @param args Arguments
	 */
	public static void main(String[] args) {

		String ap_string=      "(ap-description " +
		":name platform_description " +
		":mobility false " +
		":dynamic true " +
		":transport-profile (ap-transport-description " +
		" :available-mtps " + "(set " +
		"(mtp-description " +
		":profile fipa-alpha " +
		":mtp-name fipa-iiop-std " +
		":addresses "  + "(set " +
		"iiop://monitor.co.uk:9000/acc " +
		"http://www.agentworld.co.uk:9000 " +
		") " +
		") " +
		") " +
		") " +
		") ";

		try {
			APDescription ap_description=new APDescription(ap_string);

			System.out.println(ap_description.getName());
			System.out.println(ap_description.getDynamic());
			System.out.println(ap_description.getMobility());
			System.out.println(ap_description.getTransportDescription());

			ap_description=new APDescription();

			ap_description.setName("Local");
			ap_description.setDynamic(new Boolean("true"));

			APTDescription apt=new APTDescription();

			MTPDescription[] mtpds=new MTPDescription[1];

			MTPDescription mtpd=new MTPDescription();
			mtpd.setProfile("Test Profile");

			List list=new java.util.LinkedList();
			list.add(new fipaos.util.URL("http://www.foo.com/acc"));
			list.add(new fipaos.util.URL("iiop://www.kso.com/ams"));
			mtpd.setAddresses(list);

			mtpds[0]=mtpd;

			apt.setMTPSSet(mtpds);

			ap_description.setTransportDescription(apt);

			System.out.println(ap_description);
		}
		catch (Throwable e) {
			e.printStackTrace();
		}

	}

} // End class