//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.ont.fipa.fipaman;

//----------------------------------------------------------------------

import fipaos.ont.BASETOKENS;
import fipaos.agent.conversation.content.Content;
import fipaos.parser.ParserException;
import fipaos.parser.sl.SLParser;
import fipaos.util.URL;

import java.util.List;
import java.util.LinkedList;
import java.util.Collections;
import java.util.StringTokenizer;
import java.io.Serializable;

//----------------------------------------------------------------------

/** Message Transport Protocol Description
 *  TODO: there is at present to checking to see if the characters in the fields are valid as per the BNF definitions
 * <br>
 * <br>
 */
public class MTPDescription implements Serializable
{

	//----------------------------------------------------------------------
	// CONSTANTS
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	// STATIC VARIABLES
	//----------------------------------------------------------------------




	//----------------------------------------------------------------------
	// INSTANCE VARIABLES
	//----------------------------------------------------------------------

	private String _profile;
	private String _mtp_name;
	private List _addresses;

	//----------------------------------------------------------------------
	// CONSTRUCTOR(S)
	//----------------------------------------------------------------------
	/** This constructor creates a MTPDescription with the specified profile, mtp-name and addresses.
	 * @param profile name of the FIPA transport profile
	 * @param mtp_name FIPA name of the MTP being supported
	 * @param addresses a list of the transport addresses of this MTP
	 */

	public MTPDescription(String profile,String mtp_name,List addresses) {
		_profile = profile;
		_mtp_name = mtp_name;
		_addresses = addresses;
	}

	/** Constructor
	 */
	public MTPDescription() {
		_addresses = Collections.synchronizedList( new LinkedList() );
	}

	/** Constructor - given a String representation of the Message Transport Description a mtpdescription will vce constructed from it.
	 * @param description String representation of the Message Transport Description
	 * @throws ParserException if the string is not valid
	 */

	public MTPDescription(String description) throws ParserException {

		_addresses = Collections.synchronizedList( new LinkedList() );

		try {
			parse(SLParser.parseTerm(description));
		}
		catch( fipaos.parser.sl.ParseException e )
		{
			throw new ParserException(e.getMessage());
		}
	}

	//----------------------------------------------------------------------
	// PRIVATE METHODS
	//----------------------------------------------------------------------

	private void addKeywordValuePair(StringBuffer buffer, String keyword, String value)
	{
		buffer.append(keyword);
		buffer.append(BASETOKENS.SPACE);
		buffer.append(value);
		buffer.append(BASETOKENS.SPACE);
	}

	private String convertListIntoString(List list)
	{
		StringBuffer buffer=new StringBuffer();

		buffer.append(BASETOKENS.OPENBRACKET);
		buffer.append(FIPAMANCONSTANTS.SEQUENCE);
		buffer.append(BASETOKENS.SPACE);

		URL url;

		for (int i=0;i<list.size();i++)
		{
			url=(URL) list.get(i);
			buffer.append(url);

			buffer.append(BASETOKENS.SPACE);
		}

		buffer.append(BASETOKENS.CLOSEBRACKET);

		return buffer.toString();
	}

	private List createList(Content content) {

		List sequence= Collections.synchronizedList( new LinkedList() );

		Content con=(Content) content.getAttribute("parameterValue");

		con=(Content) con.getAttribute("term");

		con=(Content) con.getAttribute("set");

		if (con==null) {
			throw new IllegalArgumentException("Illegal MTP-DESCRIPTION FORMAT: SEQUENCE");
		}

		if (con.getAttribute("term") instanceof List) {

			List list=(List) con.getAttribute("term");

			int size=list.size();

			String[] array=new String[size];

			for (int i=0;i<size;i++) {

				Content term=(Content) list.get(i);

				sequence.add(new URL((String) term.getAttribute("formattedString")));

			}
		}
		else {

			sequence.add(new URL((String) ((Content) con.getAttribute("term")).getAttribute("formattedString")));

		}

		return sequence;
	}

	/** parses a string into a MTPDescription object
	 * @param description srting representation of a MTPDescription Object
	 * @throws ParserException
	 */

	private void parse(Content mtp_content) throws IllegalArgumentException {

		if (mtp_content!=null) {

			boolean mandatorySatisfied=false;

			Content functional_content=(Content) mtp_content.getAttribute("functionalTerm");

			if (functional_content==null || !functional_content.getAttribute("functionalOperator").equals(FIPAMANCONSTANTS.MTP_DESCRIPTION)) {
				throw new IllegalArgumentException("Content is not a MTP-DESCRIPTION");
			}

			Object parameters=functional_content.getAttribute("parameter");

			if (parameters instanceof List) {

				for (int i=0;i<((List) parameters).size();i++) {

					Content content=(Content) ((List) parameters).get(i);

					if (content.getName()!="parameter") {
						throw new IllegalArgumentException("Illegal MTP-DESCRIPTION FORMAT");
					}

					String name=(String) content.getAttribute("parameterName");

					if (name.equals(FIPAMANCONSTANTS.PROFILE)) {

						String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");

						_profile=value;
					}
					else if (name.equals(FIPAMANCONSTANTS.MTP_NAME)) {

						String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");

						_mtp_name=value;
					}
					else if (name.equals(FIPAMANCONSTANTS.ADDRESSES)) {

						mandatorySatisfied=true;

						_addresses=createList(content);
					}
				}
			}
			else {

				String name=(String) ((Content) parameters).getAttribute("parameterName");

				if (name.equals(FIPAMANCONSTANTS.ADDRESSES)) {

					mandatorySatisfied=true;

					setAddresses(createList((Content) parameters));
				}

			}

			if (!mandatorySatisfied) {
				throw new IllegalArgumentException("Message Transport Protocol Description must contain addresses");
			}
		}
	}

	//----------------------------------------------------------------------
	// PROTECTED METHODS
	//----------------------------------------------------------------------




	//----------------------------------------------------------------------
	// PUBLIC METHODS
	//----------------------------------------------------------------------
	/** Sets the profile
	 * @param profile the profile
	 */
	public void setProfile(String profile) {
		_profile = profile;
	}

	/** Sets the message transport name
	 * @param mtp  the message transport name
	 */
	public void setMTPName(String mtp) {
		_mtp_name = mtp;
	}

	/** Set the list of addresses
	 * @param addresses a list of addresses
	 */
	public void setAddresses(List addresses) {
		_addresses = addresses;
	}

	/** add an address to the list of addresses
	 * @param address the address to be added
	 * @return as per the list interface
	 */
	public boolean addAddress(URL address) {
		synchronized( _addresses ) {

			if( !_addresses.contains( address ) ) {
				return _addresses.add( address );
			}
		}
		return false;
	}

	/** get the profile
	 * @return the profile
	 */
	public String getProfile() {
		return _profile;
	}

	/** get the message transport name
	 * @return  the message transport name
	 */
	public String getMTPName() {
		return _mtp_name;
	}

	/** Get the list of addresses
	 * @return the list of addresses
	 */
	public List getAddresses() {
		return _addresses;
	}

	/** remove an address from the list of addresses
	 * @param address the address to be removed
	 * @return as per the list interface
	 */

	public boolean removeAddress(URL address) {
		synchronized( _addresses ) {
			return _addresses.remove( address );
		}
	}

	/** this object as a string
	 * @return this object as a string
	 */
	public String toString() {

		StringBuffer buffer=new StringBuffer();
		buffer.append(BASETOKENS.OPENBRACKET);
		buffer.append(FIPAMANCONSTANTS.MTP_DESCRIPTION);
		buffer.append(BASETOKENS.NEWLINE);

		if (_profile !=null ) {
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.PROFILE,_profile);
		}

		if (_mtp_name  !=null ) {
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.MTP_NAME,_mtp_name);
		}

		if (_addresses  !=null ) {
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.ADDRESSES,convertListIntoString(_addresses));
		}

		buffer.append(BASETOKENS.CLOSEBRACKET);

		return buffer.toString();

	}

	//----------------------------------------------------------------------
	// MAIN/TESTING METHOD
	//----------------------------------------------------------------------
	/** This is for testing purposes.
	 * @param args arguments
	 */
	public static void main(String[] args) {

		String mtp_string="(mtp-description " +
		":profile fipa-alpha " +
		":mtp-name fipa-iiop-std " +
		":addresses "  + "(set " +
		"iiop://monitor.co.uk:9000/acc " +
		"http://www.agentworld.co.uk:9000 " +
		") " +
		") ";

		try {
			MTPDescription mtp_description=new MTPDescription(mtp_string);

			System.out.println(mtp_description.getProfile());
			System.out.println(mtp_description.getMTPName());
			System.out.println(mtp_description.getAddresses());
		}
		catch (ParserException e) {
			e.printStackTrace();
		}
	}
} // End class