//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.ont.fipa.fipaman;

//----------------------------------------------------------------------

import java.util.List;

import fipaos.ont.BASETOKENS;
import fipaos.parser.ParserException;
import fipaos.agent.conversation.content.Content;
import fipaos.parser.sl.SLParser;

//----------------------------------------------------------------------

/** The property template is to be used for specifying attribute/value pairs
 * <br>
 * <br>
 */

public class PropertyTemplate {

	/** The name of the object
	 */
	private String _name;

	/** the object
	 */
	private String _term;

	/** This constructor creates a PropertyTemplate object with the specified string
	 * @param property_template string representation of the property_template
	 * @throws ParserException if the string is invalid
	 */
	public PropertyTemplate(String property_template) throws ParserException {

		try {
			parse(SLParser.parseTerm(property_template));
		}
		catch (fipaos.parser.sl.ParseException e) {
			throw new ParserException(e.getMessage());
		}
	}

	/** This constructor creates a template with the specified name and term.
	 * @param name name of the property
	 * @param term the value of the property
	 */
	public PropertyTemplate(String name, String term) {
		_name = name;
		_term = term;
	}

	/** This is an empty constructor.
	 */
	public PropertyTemplate() {
		_name = null;
		_term = null;
	}

	/** set the name of the object
	 * @param name the name of the object
	 */
	public void setName(String name) {
		_name = name;
	}

	/** Set the object
	 * @param term the object
	 */
	public void setTerm(String term) {
		_term = term;
	}

	/** Get the name of the object
	 * @return The name of the object
	 */
	public String getName() {
		return _name;
	}

	/** Get the object
	 * @return the object
	 */

	public String getTerm() {
		return _term;
	}

	/** This method returns the String representation of the template.
	 * @return string representation of the template
	 */
	public String toString() {

		StringBuffer buffer=new StringBuffer();
		buffer.append(BASETOKENS.OPENBRACKET);
		buffer.append(FIPAMANCONSTANTS.PROPERTY_TEMPLATE);
		buffer.append(BASETOKENS.NEWLINE);

		if (_name !=null ) {
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.NAME,_name);
		}

		if (_term  !=null ) {
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.VALUE,_term);
		}

		buffer.append(BASETOKENS.CLOSEBRACKET);

		return buffer.toString();
	}

	private void addKeywordValuePair(StringBuffer buffer, String keyword, String value)
	{
		buffer.append(keyword);
		buffer.append(BASETOKENS.SPACE);
		buffer.append(value);
		buffer.append(BASETOKENS.NEWLINE);
	}

	private void parse(Content property_template_content) throws IllegalArgumentException {

		if (property_template_content!=null) {

			boolean m1=false;
			boolean m2=false;

			Content functional_content=(Content) property_template_content.getAttribute("functionalTerm");

			if (functional_content==null || !functional_content.getAttribute("functionalOperator").equals(FIPAMANCONSTANTS.PROPERTY_TEMPLATE)) {
				throw new IllegalArgumentException("Content is not a PROPERTY-TEMPLATE");
			}

			List parameters=(List) functional_content.getAttribute("parameter");

			for (int i=0;i<parameters.size();i++) {

				Content content=(Content) parameters.get(i);

				if (content.getName()!="parameter") {
					throw new IllegalArgumentException("Illegal PROPERTY-TEMPLATE FORMAT");
				}

				String name=(String) content.getAttribute("parameterName");

				if (name.equals(FIPAMANCONSTANTS.NAME)) {

					m1=true;

					String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");

					_name=value;
				}
				else if (name.equals(FIPAMANCONSTANTS.VALUE)) {

					m2=true;

					String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");

					_term=value;
				}
			}

			if (!m1 || !m2) {
				throw new IllegalArgumentException("Property Template must contain both name and value");
			}
		}
	}

	/** This is for testing purposes.
	 * @param args arguments
	 */
	public static void main(String[] args) {

		String property_template="(property :name king :value czech)";

		try {
			PropertyTemplate template=new PropertyTemplate(property_template);

			System.out.println(template.getName());
			System.out.println(template.getTerm());
		}
		catch (ParserException e) {
			e.printStackTrace();
		}
	}
} // End class