//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.ont.fipa.fipaman;

//----------------------------------------------------------------------

import fipaos.util.UTCTime;
import fipaos.util.URL;

import java.io.Serializable;

//----------------------------------------------------------------------

/**
 * Representation of ReceivedSlot entry in envelope
 * <br>
 * <br>
 */
public class Received implements Serializable, Cloneable
{
	 
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------

	private URL _from;

	private URL _by;
	
	private String _id;
	
	private String _via;
	
	private UTCTime _time;
	
//----------------------------------------------------------------------
// CONSTRUCTOR(S)
//----------------------------------------------------------------------

	public Received( URL from, URL by, String id, String via )
	{
		// Store variables & create datestamp
		_from = from;
		_by = by;
		_id = id;
		_via = via;
		_time = new UTCTime();
	}
	
	public Received( URL from, URL by, String id, String via, UTCTime time )
	{
		// Store variables & create datestamp
		this( from, by, id, via );
		_time = time;
	}
	
	public Object clone()
	{
		try
		{
			Received r = (Received) super.clone();
			
			if( _by != null )
			{
				r._by = (URL) _by.clone();
			}
			
			if( _from != null )
			{
				r._from = (URL) _from.clone();
			}

			if( _time != null )
			{
				r._time = (UTCTime) _time.clone();
			}
			
			return r;
		}
		catch( CloneNotSupportedException e )
		{
			return null;
		}
	}
	
//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------	
	
	public URL getFrom()
	{
		if( _from != null )
		{
			return (URL) _from.clone();
		}
		
		return null;
	}

	public URL getBy()
	{
		if( _by != null )
		{
			return (URL) _by.clone();
		}
		
		return null;
	}

	public String getId()
	{
		return _id;
	}

	public String getVia()
	{
		return _via;
	}

	public UTCTime getDateTime()
	{
		if( _time != null )
		{
			return _time.getZTime();
		}
		
		return null;
	}

} // End class