//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.ont.fipa.fipaman;

//----------------------------------------------------------------------

import java.util.List;

import fipaos.ont.BASETOKENS;
import fipaos.agent.conversation.content.Content;
import fipaos.parser.ParserException;
import fipaos.parser.sl.SLParser;

//----------------------------------------------------------------------

/** This type of object represents a set of constraints to limit the function of searching within a directory.
 * <br>
 * <br>
 */

public class SearchConstraints implements java.io.Serializable
{

	/** The max. depth of propagation
	 */
	private Integer _max_depth;

	/** The max. no. of results
	 */
	private Integer _max_results;

	/** This constructor creates a SearchConstraints with the specified string.
	 * @param constraints_string string to be parsed
	 * @throws ParserException if the string is invalid
	 */
	public SearchConstraints(String constraints_string) throws ParserException {

		try {
			parse(SLParser.parseTerm(constraints_string));
		}
		catch (fipaos.parser.sl.ParseException e) {
			throw new ParserException(e.getMessage());
		}
	}

	/** This constructor constructs a SearchConstraints object with a Content
	 * object generated by the SL parser.
	 * @param constraints_content Content object generated by the SL parser
	 */
	public SearchConstraints(Content constraints_content) {

		parse(constraints_content);
	}

	/** This constructor creates the object with the specified max.depth.
	 * @param max_depth The max. depth of propagation of the search to
	 * federated directories. This value should not be negative.
	 */
	public SearchConstraints(int max_depth) {
		_max_depth = new Integer(max_depth);
	}

	/** This constructor creates the object with the max-depth
	 * @param max_depth Integer wrapper object
	 */
	public SearchConstraints(Integer max_depth) {
		_max_depth = max_depth;
	}

	/** This constructor creates the object with the specified attribute.
	 * @param max_depth maximum depth
	 * @param max_results maximum number of results
	 */
	public SearchConstraints(int max_depth, int max_results) {
		_max_depth = new Integer(max_depth);
		_max_results = new Integer(max_results);
	}

	/** This method creates a SearchConstraints object with the specified attributes.
	 * @param max_depth maximum depth
	 * @param max_results maximum number of results
	 */
	public SearchConstraints(Integer max_depth, Integer max_results) {
		_max_depth = max_depth;
		_max_results = max_results;
	}

	/** This is a null constructor.
	 */
	public SearchConstraints() {
	}

	/** This method sets the max. depth.
	 * @param max_depth maximum depth
	 */
	public void setMaxDepth(Integer max_depth) {
		_max_depth = max_depth;
	}

	/** This method sets the maximum number of results
	 * @param max_results maximum number of results
	 */
	public void setMaxResults(Integer max_results) {
		_max_results = max_results;
	}

	/** Get the name of the object
	 * @return The name of the object
	 */
	public Integer getMaxDepth() {
		return _max_depth;
	}

	/** Get the object
	 * @return the object
	 */

	public Integer getMaxResults() {
		return _max_results;
	}

	private void addKeywordValuePair(StringBuffer buffer, String keyword, String value)
	{
		buffer.append(keyword);
		buffer.append(BASETOKENS.SPACE);
		buffer.append(value);
		buffer.append(BASETOKENS.SPACE);
	}

	/** This method returns the string representation of the object
	 * @return string representation of the object
	 */
	public String toString() {

		StringBuffer buffer=new StringBuffer();
		buffer.append(BASETOKENS.OPENBRACKET);
		buffer.append(FIPAMANCONSTANTS.SEARCH_CONSTRAINTS);
		buffer.append(BASETOKENS.SPACE);

		if (_max_depth !=null ) {
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.MAX_DEPTH,_max_depth.toString());
		}

		if (_max_results  !=null ) {
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.MAX_RESULTS,_max_results.toString());
		}

		buffer.append(BASETOKENS.CLOSEBRACKET);

		return buffer.toString();
	}

	private void parse(Content search_constraints_content) throws IllegalArgumentException {

		if (search_constraints_content!=null) {

			Content functional_content=(Content) search_constraints_content.getAttribute("functionalTerm");

			if (functional_content==null || !functional_content.getAttribute("functionalOperator").equals(FIPAMANCONSTANTS.SEARCH_CONSTRAINTS)) {
				throw new IllegalArgumentException("Content is not a SEARCH CONSTRAINTS");
			}

			if (functional_content.getAttribute("parameter") instanceof List) {

				List parameters=(List) functional_content.getAttribute("parameter");

				for (int i=0;i<parameters.size();i++) {

					Content content=(Content) parameters.get(i);

					if (content.getName()!="parameter") {
						throw new IllegalArgumentException("Illegal SEARCH CONSTRAINTS FORMAT");
					}

					String name=(String) content.getAttribute("parameterName");

					if (name.equals(FIPAMANCONSTANTS.MAX_DEPTH)) {

						String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");

						_max_depth=new Integer(value);
					}
					else if (name.equals(FIPAMANCONSTANTS.MAX_RESULTS)) {

						String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");

						_max_results=new Integer(value);
					}
				}
			}
			else if (functional_content.getAttribute("parameter") instanceof Content) {

				Content content=(Content) functional_content.getAttribute("parameter");

				String name=(String) content.getAttribute("parameterName");

				if (name.equals(FIPAMANCONSTANTS.MAX_DEPTH)) {

					String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");

					_max_depth=new Integer(value);
				}
				else if (name.equals(FIPAMANCONSTANTS.MAX_RESULTS)) {

					String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");

					_max_results=new Integer(value);
				}
			}
		}
	}

	/** This is used for testing purposes/.
	 * @param args arguments
	 */
	public static void main(String[] args) {

		String search_constraints="(search-constraints :max-depth 2 :max-results 20)";

		try {
			SearchConstraints constraints=new SearchConstraints(search_constraints);

			System.out.println(constraints.getMaxDepth());
			System.out.println(constraints.getMaxResults());

			constraints=new SearchConstraints(10,90);

			System.out.println(constraints);
		}
		catch (ParserException e) {
			e.printStackTrace();
		}
	}
} // End class