//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved.
//
//******************************************************************************

package fipaos.ont.fipa.fipaman;

// -------------------------------------------------------------------------

import java.util.*;
import fipaos.ont.BASETOKENS;
import fipaos.parser.ParserException;
import fipaos.parser.sl.SLParser;
import fipaos.agent.conversation.content.Content;
import fipaos.util.*;

// -------------------------------------------------------------------------

/** FIPA defined FIPA-Service-Desc class for the FIPA agent management ontology.
 * See FIPA 2000 XC00008D 6.1.3.
 * <br>
 * <br>
 */

public class ServiceDescription implements java.io.Serializable
{
	
	//----------------------------------------------------------------------
	// PRIVATE CONSTANTS
	//----------------------------------------------------------------------
	
	//----------------------------------------------------------------------
	// INSTANCE VARIABLES
	//----------------------------------------------------------------------
	
	private String _service_name = null;
	private String _service_type = null;
	private String _ownership = null;
	private Set _ontologies = null;
	private Set _properties = null;
	private Set _protocols = null;
	
	//----------------------------------------------------------------------
	// CONSTRUCTOR
	//----------------------------------------------------------------------
	
	/** This constructor parses the string passed to it to populate the
	 * object.
	 * @param desc_string the DF-servicedescription
	 * @throws ParserException if the string is invalid
	 */
	
	public ServiceDescription( String desc_string ) throws ParserException
	{
		
		try
		{
			//converts the String into a Content object
			
			parse(SLParser.parseTerm(desc_string));
		}
		catch (fipaos.parser.sl.ParseException e)
		{
			
			throw new ParserException(e.getMessage());
		}
	}
	
	/** constructor parses the Content object passed to it to populate the
	 * object.
	 * @param desc_content the DF-servicedescription
	 * @throws ParserException if the string is invalid
	 */
	
	public ServiceDescription( Content desc_content) throws ParserException
	{
		parse(desc_content);
	}
	
	/** This is an empty constructor where fields may be set specifically
	 * rather than using implicit parsing of a string.
	 */
	public ServiceDescription()
	{
	}
	
	
	//----------------------------------------------------------------------
	// GET/SET METHODS
	//----------------------------------------------------------------------
	
	
	/** This method returns the service name of the description
	 * @return service name of the description
	 */
	public String getServiceName()
	{
		return _service_name;
	}
	
	/** This method returns the service type of the description.
	 * @return service type of the description
	 */
	public String getServiceType()
	{
		return _service_type;
	}
	
	/** This method returns the ownership of the description.
	 * @return ownership of the description
	 */
	public String getOwnership()
	{
		return _ownership;
	}
	
	/** This method returns the ontologies.
	 * @return Set of ontologies
	 */
	public Set getOntologies()
	{
		return _ontologies;
	}
	
	/** This method return the properties.
	 * @return set of properties
	 */
	public Set getProperties()
	{
		return _properties;
	}
	
	/** This method returns the protocols.
	 * @return set of protocols
	 */
	public Set getProtocols()
	{
		return _protocols;
	}
	
	//----------------------------------------------------------------------
	
	/** This method sets the service name of the description.
	 * @param desc_item service name
	 */
	public void setServiceName(String desc_item)
	{
		_service_name = desc_item;
	}
	
	/** This method sets the service type.
	 * @param desc_item service type
	 */
	public void setServiceType(String desc_item)
	{
		_service_type = desc_item;
	}
	
	/** This method sets the ownership.
	 * @param desc_item ownership of the service
	 */
	public void setOwnership(String desc_item)
	{
		_ownership = desc_item;
	}
	
	/** This method sets the ontologies.
	 * @param desc_item set of ontologies
	 */
	public void setOntologies(Set desc_item)
	{
		_ontologies = desc_item;
	}
	
	/** This method sets the ontologies
	 * @param desc_item string array of ontologies
	 */
	public void setOntologies(String[] desc_item)
	{
		Set set=new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE );
		
		for (int i=0;i< desc_item.length;i++)
		{
			set.add(desc_item[i]);
		}
		
		_ontologies=set;
	}
	
	/** This method sets the properties
	 * @param desc_item set of properties
	 */
	public void setProperties(Set desc_item)
	{
		_properties = desc_item;
	}
	
	/** This method sets the properties
	 * @param desc_item array of PropertyTemplate
	 * @throws ParserException if the PropertyTemplate array is not valid
	 */
	public void setProperties(PropertyTemplate[] desc_item) throws ParserException
	{
		Set set=new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE );
		
		for (int i=0;i< desc_item.length;i++)
		{
			set.add(desc_item[i]);
		}
		
		_properties = set;
	}
	
	/** This method sets the properties
	 * @param desc_item string array of property templates
	 * @throws ParserException if the string array is not valid
	 */
	public void setProperties(String[] desc_item) throws ParserException
	{
		Set set=new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE );
		
		for (int i=0;i< desc_item.length;i++)
		{
			set.add(new PropertyTemplate(desc_item[i]));
		}
		
		_properties = set;
	}
	
	/** This method sets the protocols.
	 * @param desc_item set of protocols
	 */
	public void setProtocols(Set desc_item)
	{
		_protocols = desc_item;
	}
	
	/** This method sets the protocols
	 * @param desc_item string array of protocols
	 */
	public void setProtocols(String[] desc_item)
	{
		Set set=new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE );
		
		for (int i=0;i< desc_item.length;i++)
		{
			set.add(desc_item[i]);
		}
		
		_protocols = set;
	}
	
	/** This method returns an Service-description as a string
	 * @return string representation of the service description
	 */
	
	public String toString()
	{
		
		StringBuffer buffer=new StringBuffer();
		buffer.append(BASETOKENS.OPENBRACKET);
		buffer.append(FIPAMANCONSTANTS.SERVICE_DESCRIPTION);
		buffer.append(BASETOKENS.NEWLINE);
		
		if (_service_name !=null )
		{
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.NAME,_service_name);
		}
		
		if (_service_type  !=null )
		{
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.SERVICE_TYPE_NEW,_service_type);
		}
		
		if (_ontologies  !=null )
		{
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.ONTOLOGY,convertSetIntoString(_ontologies));
		}
		
		if (_protocols  !=null )
		{
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.PROTOCOL,convertSetIntoString(_protocols));
		}
		
		if (_ownership  !=null )
		{
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.OWNERSHIP,_ownership);
		}
		
		if (_properties  !=null )
		{
			addKeywordValuePair(buffer,FIPAMANCONSTANTS.PROPERTIES,convertPropertySetIntoString(_properties));
		}
		
		buffer.append(BASETOKENS.CLOSEBRACKET);
		
		return buffer.toString();
		
	}
	
	/** This method compares if a Service description is the same.
	 * @param new_desc service description to be compared
	 * @return true if equal, false otherwise
	 */
	public boolean equals(ServiceDescription new_desc)
	{
		
		if(new_desc == null)
		{
			return false;
		}
		
		if(_service_name != null
		&& !_service_name.equals(new_desc.getServiceName()))
		{
			return false;
		}
		
		if(_service_type != null
		&& !_service_type.equals(new_desc.getServiceType()))
		{
			return false;
		}
		
		if(_ownership != null
		&& !_ownership.equals(new_desc.getOwnership()))
		{
			return false;
		}
		
		if(_ontologies != null
		&& !_ontologies.equals(new_desc.getOntologies()))
		{
			return false;
		}
		
		if(_properties != null
		&& !_properties.equals(new_desc.getProperties()))
		{
			return false;
		}
		
		if(_protocols != null
		&& !_protocols.equals(new_desc.getProtocols()))
		{
			return false;
		}
		
		return true;
	}
	
	
	//----------------------------------------------------------------------
	// PRIVATE METHODS
	//----------------------------------------------------------------------
	
	private void addKeywordValuePair(StringBuffer buffer, String keyword, String value)
	{
		buffer.append(keyword);
		buffer.append(BASETOKENS.SPACE);
		buffer.append(value);
		buffer.append(BASETOKENS.NEWLINE);
	}
	
	private String convertSetIntoString(Set set)
	{
		StringBuffer buffer=new StringBuffer();
		
		buffer.append(BASETOKENS.OPENBRACKET);
		buffer.append(FIPAMANCONSTANTS.SET);
		buffer.append(BASETOKENS.SPACE);
		
		String value;
		
		Iterator iterator=set.iterator();
		
		while (iterator.hasNext())
		{
			value=(String) iterator.next();
			buffer.append(value);
			
			buffer.append(BASETOKENS.SPACE);
		}
		
		buffer.append(BASETOKENS.CLOSEBRACKET);
		
		return buffer.toString();
	}
	
	private String convertPropertySetIntoString(Set set)
	{
		StringBuffer buffer=new StringBuffer();
		
		buffer.append(BASETOKENS.OPENBRACKET);
		buffer.append(FIPAMANCONSTANTS.SET);
		buffer.append(BASETOKENS.SPACE);
		
		PropertyTemplate pt;
		
		Iterator iterator=set.iterator();
		
		while (iterator.hasNext())
		{
			pt=(PropertyTemplate) iterator.next();
			buffer.append(pt.toString());
			
			buffer.append(BASETOKENS.SPACE);
		}
		
		buffer.append(BASETOKENS.CLOSEBRACKET);
		
		return buffer.toString();
	}
	
	
	/**
	 * Creates a Java set from a SL Set content
	 */
	
	private Set createSet(Content content)
	{
		
		Set set= Collections.synchronizedSet( new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE ) );
		
		Content con=(Content) content.getAttribute("parameterValue");
		
		con=(Content) con.getAttribute("term");
		
		con=(Content) con.getAttribute("set");
		
		if (con==null)
		{
			throw new IllegalArgumentException("Illegal DF-AGENT-DESCRIPTION FORMAT: SET");
		}
		
		if (con.getAttribute("term") instanceof List)
		{
			
			List list=(List) con.getAttribute("term");
			
			int size=list.size();
			
			for (int i=0;i<size;i++)
			{
				
				Content term=(Content) list.get(i);
				
				set.add((String) term.getAttribute("formattedString"));
				
			}
		}
		else
		{
			set.add((String) ((Content) con.getAttribute("term")).getAttribute("formattedString"));
		}
		
		return set;
	}
	
	/**
	 * Creates a Java set from a SL Set content
	 */
	
	private Set createPropertySet(Content content) throws ParserException
	{
		
		Set set= Collections.synchronizedSet( new TreeSet( fipaos.util.Comparators.OBJECT_COMPARE ) );
		
		// Handle empty set case
		if( content != null )
		{
			Content con=(Content) content.getAttribute("parameterValue");
			
			if( con != null )
			{
				con=(Content) con.getAttribute("term");
				
				if( con != null )
				{
					con=(Content) con.getAttribute("set");
					
					if (con==null)
					{
						throw new IllegalArgumentException("Illegal DF-AGENT-DESCRIPTION FORMAT: SET");
					}
					
					if (con.getAttribute("term") instanceof List)
					{
						List list=(List) con.getAttribute("term");
						
						int size=list.size();
						
						for (int i=0;i<size;i++)
						{
							
							Content term=(Content) list.get(i);
							
							set.add(new PropertyTemplate((String) term.getAttribute("formattedString")));
							
						}
					}
					else
					{
						Content con2 = (Content) con.getAttribute("term");
						
						if( con2 != null )
						{
							String value = (String) con2.getAttribute("formattedString");
							
							if( value != null )
							{
								PropertyTemplate pt = new PropertyTemplate(value);
								set.add( pt );
							}
						}
					}
				}
			}
		}
		
		return set;
	}
	
	/**
	 * parse an ACL FIPA-Service-Desc into its component objects
	 * e.g.
	 * <b>
	 * "(:service-description (:name fipa-df))"
	 * </b>
	 * <br>
	 * @param desc_string the FIPA-Service-Desc component of an ACL string
	 *
	 * The desc_content is traversed to construct a ServiceDescription object
	 **/
	
	private void parse(Content desc_content) throws IllegalArgumentException, ParserException
	{
		
		if (desc_content!=null)
		{
			
			boolean m1=false;
			boolean m2=false;
			
			Content functional_content=(Content) desc_content.getAttribute("functionalTerm");
			
			if (functional_content==null || !functional_content.getAttribute("functionalOperator").equals(FIPAMANCONSTANTS.SERVICE_DESCRIPTION))
			{
				throw new IllegalArgumentException("Content is not a SERVICE-DESCRIPTION");
			}
			
			List parameters = null;
			Object attr = functional_content.getAttribute("parameter");
			
			// Ensure we are dealing with a List
			if ( attr instanceof List )
			{
				parameters = (List) attr;
			}
			else
			{
				parameters = Arrays.asList( new Object[]
				{ attr } );
			}
			
			for (int i=0;i<parameters.size();i++)
			{
				Content content=(Content) parameters.get(i);
				
				if (content.getName()!="parameter")
				{
					throw new IllegalArgumentException("Illegal SERVICE-DESCRIPTION FORMAT");
				}
				
				String name=(String) content.getAttribute("parameterName");
				
				if (name.equals(FIPAMANCONSTANTS.NAME))
				{
					String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");
					
					_service_name=value;
				}
				else if (name.equals(FIPAMANCONSTANTS.SERVICE_TYPE_NEW))
				{
					String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");
					
					_service_type=value;
				}
				else if (name.equals(FIPAMANCONSTANTS.OWNERSHIP))
				{
					String value=(String) ((Content) content.getAttribute("parameterValue")).getAttribute("formattedString");
					
					_ownership=value;
				}
				else if (name.equals(FIPAMANCONSTANTS.PROTOCOL))
				{
					_protocols=createSet(content);
				}
				else if (name.equals(FIPAMANCONSTANTS.ONTOLOGY))
				{
					_ontologies=createSet(content);
				}
				else if (name.equals(FIPAMANCONSTANTS.PROPERTIES))
				{
					_properties=createPropertySet(content);
				}
			}
		}
	}
	
	
	//----------------------------------------------------------------------
	// MAIN() METHOD
	//----------------------------------------------------------------------
	
	// used for testing purposes
	
	/** This method is used for testing.
	 * @param args arguments
	 */
	public static void main(String [] args)
	{
		
		fipaos.util.DIAGNOSTICS.println("Testing ServiceDescription");
		
		try
		{
			ServiceDescription service_desc = new ServiceDescription("(service-description "+
			":name profiling " +
			":type user-profiling " +
			":ontology (set meeting-scheduler) " +
			":properties (set (property :name learning algorithm :value BBN) (property :name max-nodes :value 1000000)))");
			
			fipaos.util.DIAGNOSTICS.println("Service Description object: " + service_desc);
			
			service_desc=new ServiceDescription();
			service_desc.setServiceName("profiling");
			service_desc.setServiceType("user-profiling");
			
			String[] ont=new String[1];
			ont[0]="meeting scheduler";
			
			service_desc.setOntologies(ont);
			
			PropertyTemplate[] pts=new PropertyTemplate[2];
			pts[0]=new PropertyTemplate("learning algorithm","BBN");
			pts[1]=new PropertyTemplate("max-nodes","1000000");
			
			service_desc.setProperties(pts);
			
			System.out.println(service_desc);
			
		}
		catch (ParserException e)
		{
			e.printStackTrace();
		}
	}
}