//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

/* Generated By:JavaCC: Do not edit this line. SLParser.java */
package fipaos.parser.sl;

import java.io.IOException;
import java.io.StringReader;
import java.io.FileReader;
import java.io.BufferedReader;
import java.util.Vector;
import fipaos.parser.*;
import fipaos.util.*;
import fipaos.agent.conversation.content.Content;

				/**	 
	 * The SLParser is used to parse a string that has the
	 * format of an Content as defined by XC00008 Fipa2000 (Experimental).
	 * For parsing to occur, the string should be converted
	 * to a StringReader and then passed to the constructor
	 * of the SLParser. Alternatively the static method parse()
	 * may be used to do this for the user.
	 */

	public class SLParser implements Parser, SLParserConstants {

	 /**
	  *	Empty constructor for manipulating Parser instance
	*/

	public SLParser() {
	}

	/**
	 * parse method is the preferred method for parsing an
	 * SL content message string.
	 * @param sl_content_expression the expression to parse
	 * @return fipaos.agent.conversation.content.Content object
	 */

	public static Content parseContent(String sl_content) throws ParserException
	{
		  Content con=null;

		   try {

			StringReader stringreader = new StringReader(sl_content);
			SLParser slparser       = new SLParser(stringreader);
			//DIAGNOSTICS.println("created the sl parser with str reader", this, DIAGNOSTICS_LEVEL_4);  	
//            DIAGNOSTICS.println("Start parsing...");

		  con=slparser.Content();

//          DIAGNOSTICS.println("Finish parsing...");

//          DIAGNOSTICS.println(con);

		   }
		   catch (ParseException e) {
				throw new ParserException(e.getMessage());
		   }

		   return con;
	}

	/**
	 * parseTerm method is the preferred method for parsing an
	 * SL term message string.
	 * @param sl_term the term to parse
	 * @return fipaos.agent.conversation.content.Content object
	 */

	public static Content parseTerm(String sl_term) throws ParseException
	{
				StringReader stringreader = new StringReader(sl_term);
				SLParser slparser       = new SLParser(stringreader);
				//DIAGNOSTICS.println("created the sl parser with str reader", this, DIAGNOSTICS_LEVEL_4);  	
//             DIAGNOSTICS.println("Start parsing...");

		  Content con=slparser.Term();

//          DIAGNOSTICS.println("Finish parsing...");

//          DIAGNOSTICS.println(con);

		  return con;
		}

	/**
	 * parseAtomicFormula method is the preferred method for parsing an
	 * SL atomicFormula message string.
	 * @param sl_atomic_formula the formula to parse
	 * @return fipaos.agent.conversation.content.Content object
	 */

	public static Content parseAtomicFormula(String sl_atomic_formula) throws ParseException
	{
				StringReader stringreader = new StringReader(sl_atomic_formula);
				SLParser slparser       = new SLParser(stringreader);
				//DIAGNOSTICS.println("created the sl parser with str reader", this, DIAGNOSTICS_LEVEL_4);  	
//              DIAGNOSTICS.println("Start parsing...");

		   Content con=slparser.AtomicFormula();

//           DIAGNOSTICS.println("Finish parsing...");

//           DIAGNOSTICS.println(con);

		   return con;
		}


	/**
	 * parseActionExpression method is the preferred method for parsing an
	 * SL actionExpression message string.
	 * @param sl_term the term to parse
	 * @return fipaos.agent.conversation.content.Content object
	 */

	public static Content parseActionExpression(String sl_action_expression) throws ParseException
	{
				StringReader stringreader = new StringReader(sl_action_expression);
				SLParser slparser       = new SLParser(stringreader);
				//DIAGNOSTICS.println("created the sl parser with str reader", this, DIAGNOSTICS_LEVEL_4);  	
//             DIAGNOSTICS.println("Start parsing...");

		  Content con=slparser.ActionExpression();

//          DIAGNOSTICS.println("Finish parsing...");

//          DIAGNOSTICS.println(con);

		  return con;
		}

   /**
	* This method converts a content String into a set of Content objects
	*
	* @param content The String to parse into Content objects
	*
	* @return A Content object representing the content String given.
	*/

		public Content parse(String sl_content) throws ParserException {

				return SLParser.parseContent(sl_content);

		}

   /**
	* This method converts a Content object into a document
	* which is of a format determined by the Parser implementation.
	*
	* @param content The Content object to deparse
	*
	* @return The Stringified representation of the Content object
	*/

		public String deparse(Content sl_content) {

				return (String) sl_content.getAttribute("formattedString");

		}


   /**
	* Determines if the Parser implementation can parse the given content
	*
	* @param A content String.
	*
	* @return True if the Parser implementation can parse the given String
	*/

		public boolean canParse(String content) {

				try {
						SLParser.parseContent(content);
				}
				catch (ParserException e) {
						return false;
				}

				return true;
		}

	/**
	 * Returns the type of content language that the Parser implementation
	 * handles.
	 *
	 * @return A String representation of the content format (e.g. RDF, SL)
	 */

		public String getParserType() {
				return "SL";
		}


	/**
	 * this static method creates a new Content object
	 *
	 * @param name name of the content
	 * @return fipaos.agent.conversation.content.Content object
	 */

		private static Content buildContent(String name) {

		return new Content(name);

	 }

	/**
	 * this static method creates a new Content object with
	 * the specified Content as its attribute
	 *
	 * @param name name of the content
	 * @param con Content attribute
	 * @return fipaos.agent.conversation.content.Content object
	 */


		private static Content buildContent(String name, Content con) {

		Content element= new Content(name);

		element.addAttribute(con.getName(),con);

		return element;
	 }

	/**
	 * this static method creates a new Content object with
	 * the specified Content as its attribute
	 *
	 * @param name name of the content
	 * @param vector vector of Content
	 * @return fipaos.agent.conversation.content.Content object
	 */

	 private static Content buildContent(String name, Vector vector) {

		Content element= new Content(name);

		for (int i=0;i<vector.size();i++) {

		  Content con=(Content) vector.elementAt(i);
		  element.addAttribute(con.getName(),con);
		}

		return element;
	 }


		/**
  	 * Test Harness method where a file of strings is parsed.
  	 * Each line is regarded as a separate SL Content
  	 */

		public static void main(String[] args) throws Exception
	 {
		String df_agent_desc = "(df-agent-description "+
									 ":name (agent-identifier :name dummy@foo.com :addresses (sequence iiop://foo.com/acc )) "+
									 ":services "+ "(set (service-description :name profiling :type user-profiling)) "+
									 ":ontology (set fixed-properties fxprops) "+
									 ":protocol (set FIPA-Request Application-Protocol)))";

		 SLParser.parseTerm(df_agent_desc);

			//print out usage instructions if needed
			if(args.length == 0)
			{
			  DIAGNOSTICS.println("usage: java fipaos.parser.sl.SLParser filename");
			  System.exit(0);
			}

		 FileReader file_in = new FileReader(args[0]);
			BufferedReader in = new BufferedReader(file_in);
			String fileLine  = null;

		 //Loop through the file parsing each line.
			// (null indicated eof)

		 while ((fileLine = in.readLine()) != null)
		   {
				DIAGNOSTICS.println("Line to parse is: " + fileLine);

		   try
				{
						//SLParser.parseContent(fileLine);
						SLParser parser=new SLParser();
						Content con=parser.parse(fileLine);
						DIAGNOSTICS.println(con);
				}
				catch (Exception e)
				{
						e.printStackTrace();
				}
			}
		}

//----------------------------------------------------------------------
// BNF PRODUCTIONS - used to define the grammar (the non-terminals
// to parse for
// Note that lookahead is typically two as most non-terminals have
// an open parenthesis as the first character - need to test how
// lookahead works with non-terminals though!
//----------------------------------------------------------------------	
  final public Content Content() throws ParseException {
	 Vector vector=new Vector();
	 String formattedString=new String();
	 String vectorString=new String();
	 Content con,con1;
	jj_consume_token(Lparen);
	label_1:
	while (true) {
	  con1 = ContentExpression();
			  vectorString +=con1.getAttribute("formattedString")+" ";
			  vector.addElement(con1);
	  switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	  case Lparen:
	  case True:
	  case False:
	  case String:
		;
		break;
	  default:
		jj_la1[0] = jj_gen;
		break label_1;
	  }
	}
	jj_consume_token(Rparen);
	 con=buildContent("content",vector);
	 formattedString="( "+vectorString+" )";
	 con.setAttribute("formattedString",formattedString);
	 {if (true) return con;}
	throw new Error("Missing return statement in function");
  }  

  final public Content ContentExpression() throws ParseException {
  Content con,con1;
	if (jj_2_1(2)) {
	  con1 = IdentifyingExpression();
	   con=buildContent("contentExpression",con1);
	   con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	   {if (true) return con;}
	} else if (jj_2_2(2)) {
	  con1 = ActionExpression();
	   con=buildContent("contentExpression",con1);
	   con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	   {if (true) return con;}
	} else {
	  switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	  case Lparen:
	  case True:
	  case False:
	  case String:
		con1 = Proposition();
	   con=buildContent("contentExpression",con1);
	   con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	   {if (true) return con;}
		break;
	  default:
		jj_la1[1] = jj_gen;
		jj_consume_token(-1);
		throw new ParseException();
	  }
	}
	throw new Error("Missing return statement in function");
  }  

  final public Content Proposition() throws ParseException {
  Content con,con1;
	con1 = Wff();
	 con=buildContent("proposition",con1);
	 con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	 {if (true) return con;}
	throw new Error("Missing return statement in function");
  }  

  final public Content Wff() throws ParseException {
  Token t;
  String formattedString=new String();
  Content con,con1;
	if (jj_2_3(2)) {
	  jj_consume_token(Lparen);
	  con1 = WffFactored();
	  jj_consume_token(Rparen);
	formattedString="( "+con1.getAttribute("formattedString")+" )";
	con1.setAttribute("formattedString",formattedString);
	{if (true) return con1;}
	} else {
	  switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	  case Lparen:
	  case True:
	  case False:
	  case String:
		con1 = AtomicFormula();
	con=buildContent("wff",con1);
	con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	{if (true) return con;}
		break;
	  default:
		jj_la1[2] = jj_gen;
		jj_consume_token(-1);
		throw new ParseException();
	  }
	}
	throw new Error("Missing return statement in function");
  }  

  final public String UnaryLogicalOp() throws ParseException {
   Token t;
	t = jj_consume_token(Not);
	  {if (true) return t.image;}
	throw new Error("Missing return statement in function");
  }  

  final public String BinaryLogicalOp() throws ParseException {
   Token t;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case And:
	  t = jj_consume_token(And);
	  {if (true) return t.image;}
	  break;
	case Or:
	  t = jj_consume_token(Or);
	  {if (true) return t.image;}
	  break;
	case Implies:
	  t = jj_consume_token(Implies);
	  {if (true) return t.image;}
	  break;
	case Equiv:
	  t = jj_consume_token(Equiv);
	  {if (true) return t.image;}
	  break;
	default:
	  jj_la1[3] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public String BinaryTermOp() throws ParseException {
   Token t;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Equal:
	  t = jj_consume_token(Equal);
	  {if (true) return t.image;}
	  break;
	case NotEqual:
	  t = jj_consume_token(NotEqual);
	  {if (true) return t.image;}
	  break;
	case GreaterThan:
	  t = jj_consume_token(GreaterThan);
	  {if (true) return t.image;}
	  break;
	case GreaterThanOrEqual:
	  t = jj_consume_token(GreaterThanOrEqual);
	  {if (true) return t.image;}
	  break;
	case LessThan:
	  t = jj_consume_token(LessThan);
	  {if (true) return t.image;}
	  break;
	case LessThanOrEqual:
	  t = jj_consume_token(LessThanOrEqual);
	  {if (true) return t.image;}
	  break;
	case Member:
	  t = jj_consume_token(Member);
	  {if (true) return t.image;}
	  break;
	case Contains:
	  t = jj_consume_token(Contains);
	  {if (true) return t.image;}
	  break;
	case Result:
	  t = jj_consume_token(Result);
	  {if (true) return t.image;}
	  break;
	default:
	  jj_la1[4] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public Content WffFactored() throws ParseException {
 String op,var;
 String formattedString=new String();
 Content con,con1,con2;
 Vector vector;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Not:
	  op = UnaryLogicalOp();
	  con1 = Wff();
	con=buildContent("wwf",con1);
	con.setAttribute("unaryLogicalOp",op);
	formattedString=op+" "+con1.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case And:
	case Or:
	case Implies:
	case Equiv:
	  op = BinaryLogicalOp();
	  con1 = Wff();
	  con2 = Wff();
	vector=new Vector();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("wwf",vector);
	con.setAttribute("binaryLogicalOp",op);
	formattedString=op+" "+con1.getAttribute("formattedString")+" "+con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case Forall:
	case Exists:
	  op = Quantifier();
	  var = Variable();
	  con1 = Wff();
	con=buildContent("wwf",con1);
	con.setAttribute("quantifier",op);
	con.setAttribute("variable",var);
	formattedString=op+" "+var+" "+con1.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case Belief:
	case Uncertainty:
	case PersistentGoal:
	case Intention:
	  op = ModalOp();
	  con1 = Agent();
	  con2 = Wff();
	vector=new Vector();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("wwf",vector);
	con.setAttribute("modalOp",op);
	formattedString=op+" "+con1.getAttribute("formattedString")+con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case Feasible:
	case Done:
	  op = ActionOp();
	  con1 = ActionFactored();
	con=buildContent("wwf",con1);
	con.setAttribute("actionOp",op);
	formattedString=op+" "+con1.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	default:
	  jj_la1[5] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public Content ActionFactored() throws ParseException {
   Content con,con1;
   Content con2=null;
   String formattedString;
   Vector vector=new Vector();
	con1 = ActionExpression();
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Lparen:
	case True:
	case False:
	case String:
	  con2 = Wff();
	  break;
	default:
	  jj_la1[6] = jj_gen;
	  ;
	}
	  if (con2==null) {
		 con=buildContent("wwf",con1);
		 con.setAttribute("formattedString",con1.getAttribute("formattedString"));
		 {if (true) return con;}
	  }
	  else {
		 vector.addElement(con1);
		 vector.addElement(con2);
		 con=buildContent("wwf",vector);
		 formattedString=(String) con1.getAttribute("formattedString")+" "+ (String) con2.getAttribute("formattedString");
		 con.setAttribute("formattedString",formattedString);
		 {if (true) return con;}
	  }
	throw new Error("Missing return statement in function");
  }  

  final public Content AtomicFormula() throws ParseException {
  Token t;
  String symbol,formattedString;
  Content con;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case True:
	  t = jj_consume_token(True);
	   con=buildContent("atomicFormula");
	   con.setAttribute("boolean",t.image);
	   con.setAttribute("formattedString",t.image);
	   {if (true) return con;}
	  break;
	case False:
	  t = jj_consume_token(False);
	   con=buildContent("atomicFormula");
	   con.setAttribute("boolean",t.image);
	   con.setAttribute("formattedString",t.image);
	   {if (true) return con;}
	  break;
	case String:
	  symbol = PropositionSymbol();
	   con=buildContent("atomicFormula");
	   con.setAttribute("propositionSymbol",symbol);
	   con.setAttribute("formattedString",symbol);
	   {if (true) return con;}
	  break;
	case Lparen:
	  jj_consume_token(Lparen);
	  con = AtomicFormulaFactored();
	  jj_consume_token(Rparen);
	formattedString="( "+(String) con.getAttribute("formattedString")+" )";
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	default:
	  jj_la1[7] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

/**
 * The production obtained by left-factorising SLAtomicFormula
 * for the beginning parenthesis.
 */
  final public Content AtomicFormulaFactored() throws ParseException {
  Content con,con1,con2;
  Vector vector=new Vector();
  String op,formattedString;
  String vectorString=new String();
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Equal:
	case NotEqual:
	case GreaterThan:
	case GreaterThanOrEqual:
	case LessThan:
	case LessThanOrEqual:
	case Member:
	case Contains:
	case Result:
	  op = BinaryTermOp();
	  con1 = Term();
	  con2 = Term();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("atomicFormula",vector);
	con.setAttribute("binaryTermOp",op);
	formattedString=op+" "+(String) con1.getAttribute("formattedString")+" "+(String) con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case String:
	  op = PredicateSymbol();
	  label_2:
	  while (true) {
		switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
		case Lparen:
		case VariableIdentifier:
		case String:
		case Integer:
		case Float:
		case DateTime:
		  ;
		  break;
		default:
		  jj_la1[8] = jj_gen;
		  break label_2;
		}
		con1 = Term();
						  vectorString +=(String) con1.getAttribute("formattedString")+" ";
						  vector.addElement(con1);
	  }
	 con=buildContent("atomicFormula",vector);
	 con.setAttribute("predicateSymbol",op);
	 formattedString=op+" "+vectorString;
	 con.setAttribute("formattedString",formattedString);
	 {if (true) return con;}
	  break;
	default:
	  jj_la1[9] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public String Quantifier() throws ParseException {
 Token t;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Forall:
	  t = jj_consume_token(Forall);
	 {if (true) return t.image;}
	  break;
	case Exists:
	  t = jj_consume_token(Exists);
	 {if (true) return t.image;}
	  break;
	default:
	  jj_la1[10] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public String ModalOp() throws ParseException {
 Token t;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Belief:
	  t = jj_consume_token(Belief);
	 {if (true) return t.image;}
	  break;
	case Uncertainty:
	  t = jj_consume_token(Uncertainty);
	 {if (true) return t.image;}
	  break;
	case Intention:
	  t = jj_consume_token(Intention);
	 {if (true) return t.image;}
	  break;
	case PersistentGoal:
	  t = jj_consume_token(PersistentGoal);
	 {if (true) return t.image;}
	  break;
	default:
	  jj_la1[11] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public String ActionOp() throws ParseException {
  Token t;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Done:
	  t = jj_consume_token(Done);
	 {if (true) return t.image;}
	  break;
	case Feasible:
	  t = jj_consume_token(Feasible);
	 {if (true) return t.image;}
	  break;
	default:
	  jj_la1[12] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public Content Term() throws ParseException {
  Content con,con1;
  String var;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case VariableIdentifier:
	  var = Variable();
	con=buildContent("term");
	con.setAttribute("variable",var);
	con.setAttribute("formattedString",var);
	{if (true) return con;}
	  break;
	default:
	  jj_la1[13] = jj_gen;
	  if (jj_2_4(2)) {
		con1 = FunctionalTerm();
	con=buildContent("term",con1);
	con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	{if (true) return con;}
	  } else if (jj_2_5(2)) {
		con1 = ActionExpression();
	con=buildContent("term",con1);
	con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	{if (true) return con;}
	  } else if (jj_2_6(2)) {
		con1 = IdentifyingExpression();
	con=buildContent("term",con1);
	con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	{if (true) return con;}
	  } else {
		switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
		case String:
		case Integer:
		case Float:
		case DateTime:
		  con1 = Constant();
	con=buildContent("term",con1);
	con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	{if (true) return con;}
		  break;
		default:
		  jj_la1[14] = jj_gen;
		  if (jj_2_7(2)) {
			con1 = Sequence();
	con=buildContent("term",con1);
	con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	{if (true) return con;}
		  } else if (jj_2_8(2)) {
			con1 = Set();
	con=buildContent("term",con1);
	con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	{if (true) return con;}
		  } else {
			jj_consume_token(-1);
			throw new ParseException();
		  }
		}
	  }
	}
	throw new Error("Missing return statement in function");
  }  

  final public Content IdentifyingExpression() throws ParseException {
  String op,formattedString;
  Vector vector=new Vector();
  Content con,con1,con2;
	jj_consume_token(Lparen);
	op = ReferentialOperator();
	con1 = Term();
	con2 = Wff();
	jj_consume_token(Rparen);
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("identifyingExpression",vector);
	con.setAttribute("referentialOperator",op);
	formattedString="( "+op+" "+(String) con1.getAttribute("formattedString")+" "+(String) con2.getAttribute("formattedString")+" )";
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	throw new Error("Missing return statement in function");
  }  

  final public String ReferentialOperator() throws ParseException {
  Token t;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Iota:
	  t = jj_consume_token(Iota);
   {if (true) return t.image;}
	  break;
	case Any:
	  t = jj_consume_token(Any);
   {if (true) return t.image;}
	  break;
	case All:
	  t = jj_consume_token(All);
   {if (true) return t.image;}
	  break;
	default:
	  jj_la1[15] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public Content FunctionalTerm() throws ParseException {
  String formattedString;
  Content con;
	jj_consume_token(Lparen);
	con = FunctionalTermFactored();
	jj_consume_token(Rparen);
	 formattedString="( "+(String) con.getAttribute("formattedString")+" )";
	 con.setAttribute("formattedString",formattedString);
	 {if (true) return con;}
	throw new Error("Missing return statement in function");
  }  

  final public Content FunctionalTermFactored() throws ParseException {
  Token t;
  Content con,con1,con2;
  Vector vector=new Vector();
  String op,formattedString;
  String vectorString=new String();
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Cons:
	  t = jj_consume_token(Cons);
	  con1 = Term();
	  con2 = Term();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("functionalTerm",vector);
	con.setAttribute("functionalOperator",t.image);
	formattedString=t.image+" "+(String) con1.getAttribute("formattedString")+" "+(String) con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case First:
	  t = jj_consume_token(First);
	  con1 = Term();
	con=buildContent("functionalTerm",con1);
	con.setAttribute("functionalOperator",t.image);
	formattedString=t.image+" "+(String) con1.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case Rest:
	  t = jj_consume_token(Rest);
	  con1 = Term();
	con=buildContent("functionalTerm",con1);
	con.setAttribute("functionalOperator",t.image);
	formattedString=t.image+" "+(String) con1.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case Nth:
	  t = jj_consume_token(Nth);
	  con1 = Term();
	  con2 = Term();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("functionalTerm",vector);
	con.setAttribute("functionalOperator",t.image);
	formattedString=t.image+" "+(String) con1.getAttribute("formattedString")+" "+(String) con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case Append:
	  t = jj_consume_token(Append);
	  con1 = Term();
	  con2 = Term();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("functionalTerm",vector);
	con.setAttribute("functionalOperator",t.image);
	formattedString=t.image+" "+(String) con1.getAttribute("formattedString")+" "+(String) con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case Union:
	  t = jj_consume_token(Union);
	  con1 = Term();
	  con2 = Term();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("functionalTerm",vector);
	con.setAttribute("functionalOperator",t.image);
	formattedString=t.image+" "+(String) con1.getAttribute("formattedString")+" "+(String) con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case Intersection:
	  t = jj_consume_token(Intersection);
	  con1 = Term();
	  con2 = Term();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("functionalTerm",vector);
	con.setAttribute("functionalOperator",t.image);
	formattedString=t.image+" "+(String) con1.getAttribute("formattedString")+" "+(String) con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case Difference:
	  t = jj_consume_token(Difference);
	  con1 = Term();
	  con2 = Term();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("functionalTerm",vector);
	con.setAttribute("functionalOperator",t.image);
	formattedString=t.image+" "+(String) con1.getAttribute("formattedString")+" "+(String) con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case Plus:
	case Minus:
	case Multiply:
	case Divide:
	case Remainder:
	  op = ArithmeticOp();
	  con1 = Term();
	  con2 = Term();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("functionalTerm",vector);
	con.setAttribute("functionalOperator",op);
	formattedString=op+" "+(String) con1.getAttribute("formattedString")+" "+(String) con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case String:
	  op = FunctionSymbol();
	  label_3:
	  while (true) {
		switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
		case Lparen:
		case ParameterName:
		case VariableIdentifier:
		case String:
		case Integer:
		case Float:
		case DateTime:
		  ;
		  break;
		default:
		  jj_la1[16] = jj_gen;
		  break label_3;
		}
		con1 = symbolTermFactored();
						vectorString+=(String) con1.getAttribute("formattedString")+" ";
						vector.addElement(con1);
	  }
	con=buildContent("functionalTerm",vector);
	con.setAttribute("functionalOperator",op);
	formattedString=op+" "+vectorString;
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	default:
	  jj_la1[17] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public Content symbolTermFactored() throws ParseException {
  Content con;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Lparen:
	case VariableIdentifier:
	case String:
	case Integer:
	case Float:
	case DateTime:
	  con = Term();
	{if (true) return con;}
	  break;
	case ParameterName:
	  con = Parameter();
	{if (true) return con;}
	  break;
	default:
	  jj_la1[18] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public String ArithmeticOp() throws ParseException {
  Token t;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Plus:
	  t = jj_consume_token(Plus);
   {if (true) return t.image;}
	  break;
	case Minus:
	  t = jj_consume_token(Minus);
   {if (true) return t.image;}
	  break;
	case Multiply:
	  t = jj_consume_token(Multiply);
   {if (true) return t.image;}
	  break;
	case Divide:
	  t = jj_consume_token(Divide);
   {if (true) return t.image;}
	  break;
	case Remainder:
	  t = jj_consume_token(Remainder);
   {if (true) return t.image;}
	  break;
	default:
	  jj_la1[19] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public Content Constant() throws ParseException {
  Content con;
  Token t;
  String num;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Integer:
	case Float:
	  num = NumericalConstant();
	 con=buildContent("constant");
	 con.setAttribute("numericalConstant",num);
	 con.setAttribute("formattedString",num);
	 {if (true) return con;}
	  break;
	case String:
	  t = jj_consume_token(String);
	 con=buildContent("constant");
	 con.setAttribute("string",t.image);
	 con.setAttribute("formattedString",t.image);
	 {if (true) return con;}
	  break;
	case DateTime:
	  t = jj_consume_token(DateTime);
	 con=buildContent("constant");
	 con.setAttribute("dateTime",t.image);
	 con.setAttribute("formattedString",t.image);
	 {if (true) return con;}
	  break;
	default:
	  jj_la1[20] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public String NumericalConstant() throws ParseException {
  Token t;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Integer:
	  t = jj_consume_token(Integer);
	{if (true) return t.image;}
	  break;
	case Float:
	  t = jj_consume_token(Float);
	{if (true) return t.image;}
	  break;
	default:
	  jj_la1[21] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public String Variable() throws ParseException {
  Token t;
	t = jj_consume_token(VariableIdentifier);
	{if (true) return t.image;}
	throw new Error("Missing return statement in function");
  }  

  final public Content ActionExpression() throws ParseException {
  Content con;
  String formattedString;
	jj_consume_token(Lparen);
	con = ActionExpressionFactored();
	jj_consume_token(Rparen);
	formattedString="( "+(String) con.getAttribute("formattedString")+" )";
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	throw new Error("Missing return statement in function");
  }  

/**
 * The production obtained by left-factorising SLActionExpression
 */
  final public Content ActionExpressionFactored() throws ParseException {
  Token t;
  Content con,con1,con2;
  Vector vector=new Vector();
  String op,formattedString;
	switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	case Action:
	  t = jj_consume_token(Action);
	  con1 = Agent();
	  con2 = Term();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("actionExpression",vector);
	con.setAttribute("actionExpressionOperator",t.image);
	formattedString=t.image+" "+(String) con1.getAttribute("formattedString")+" "+(String) con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case Pipe:
	  t = jj_consume_token(Pipe);
	  con1 = ActionExpression();
	  con2 = ActionExpression();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("actionExpression",vector);
	con.setAttribute("actionExpressionOperator",t.image);
	formattedString=t.image+" "+(String) con1.getAttribute("formattedString")+" "+(String) con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	case SemiColon:
	  t = jj_consume_token(SemiColon);
	  con1 = ActionExpression();
	  con2 = ActionExpression();
	vector.addElement(con1);
	vector.addElement(con2);
	con=buildContent("actionExpression",vector);
	con.setAttribute("actionExpressionOperator",t.image);
	formattedString=t.image+" "+(String) con1.getAttribute("formattedString")+" "+(String) con2.getAttribute("formattedString");
	con.setAttribute("formattedString",formattedString);
	{if (true) return con;}
	  break;
	default:
	  jj_la1[22] = jj_gen;
	  jj_consume_token(-1);
	  throw new ParseException();
	}
	throw new Error("Missing return statement in function");
  }  

  final public String PropositionSymbol() throws ParseException {
  Token t;
	t = jj_consume_token(String);
		{if (true) return t.image;}
	throw new Error("Missing return statement in function");
  }  

  final public String PredicateSymbol() throws ParseException {
  Token t;
	t = jj_consume_token(String);
		{if (true) return t.image;}
	throw new Error("Missing return statement in function");
  }  

  final public String FunctionSymbol() throws ParseException {
   Token t;
	t = jj_consume_token(String);
		{if (true) return t.image;}
	throw new Error("Missing return statement in function");
  }  

  final public Content Agent() throws ParseException {
  Content con,con1;
	con1 = Term();
	  con=buildContent("agent",con1);
	  con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	  {if (true) return con;}
	throw new Error("Missing return statement in function");
  }  

  final public Content Sequence() throws ParseException {
  Content con,con1;
  String formattedString;
  String vectorString=new String();
  Vector vector=new Vector();
  Token t;
	jj_consume_token(Lparen);
	t = jj_consume_token(Sequence);
	label_4:
	while (true) {
	  switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	  case Lparen:
	  case VariableIdentifier:
	  case String:
	  case Integer:
	  case Float:
	  case DateTime:
		;
		break;
	  default:
		jj_la1[23] = jj_gen;
		break label_4;
	  }
	  con1 = Term();
						  vectorString+=(String) con1.getAttribute("formattedString")+" ";
						  vector.addElement(con1);
	}
	jj_consume_token(Rparen);
	  con=buildContent("sequence",vector);
	  formattedString="( "+t.image+" "+vectorString+" )";
	  con.setAttribute("formattedString",formattedString);
	  {if (true) return con;}
	throw new Error("Missing return statement in function");
  }  

  final public Content Set() throws ParseException {
  Content con,con1;
  String formattedString;
  String vectorString=new String();
  Vector vector=new Vector();
  Token t;
	jj_consume_token(Lparen);
	t = jj_consume_token(Set);
	label_5:
	while (true) {
	  switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
	  case Lparen:
	  case VariableIdentifier:
	  case String:
	  case Integer:
	  case Float:
	  case DateTime:
		;
		break;
	  default:
		jj_la1[24] = jj_gen;
		break label_5;
	  }
	  con1 = Term();
								  vectorString+=(String) con1.getAttribute("formattedString")+" ";
						  vector.addElement(con1);
	}
	jj_consume_token(Rparen);
	  con=buildContent("set",vector);
	  formattedString="( "+t.image+" "+vectorString+" )";
	  con.setAttribute("formattedString",formattedString);
	  {if (true) return con;}
	throw new Error("Missing return statement in function");
  }  

  final public Content Parameter() throws ParseException {
   Token t;
   String formattedString;
   Content con,con1;
	t = jj_consume_token(ParameterName);
	con1 = ParameterValue();
	  con=buildContent("parameter",con1);
	  con.setAttribute("parameterName",t.image);
	  formattedString=t.image+" "+(String) con1.getAttribute("formattedString");
	  con.setAttribute("formattedString",formattedString);
	  {if (true) return con;}
	throw new Error("Missing return statement in function");
  }  

  final public Content ParameterValue() throws ParseException {
  Content con,con1;
	con1 = Term();
	  con=buildContent("parameterValue",con1);
	  con.setAttribute("formattedString",con1.getAttribute("formattedString"));
	  {if (true) return con;}
	throw new Error("Missing return statement in function");
  }  

  final private boolean jj_2_1(int xla) {
	jj_la = xla; jj_lastpos = jj_scanpos = token;
	boolean retval = !jj_3_1();
	jj_save(0, xla);
	return retval;
  }  

  final private boolean jj_2_2(int xla) {
	jj_la = xla; jj_lastpos = jj_scanpos = token;
	boolean retval = !jj_3_2();
	jj_save(1, xla);
	return retval;
  }  

  final private boolean jj_2_3(int xla) {
	jj_la = xla; jj_lastpos = jj_scanpos = token;
	boolean retval = !jj_3_3();
	jj_save(2, xla);
	return retval;
  }  

  final private boolean jj_2_4(int xla) {
	jj_la = xla; jj_lastpos = jj_scanpos = token;
	boolean retval = !jj_3_4();
	jj_save(3, xla);
	return retval;
  }  

  final private boolean jj_2_5(int xla) {
	jj_la = xla; jj_lastpos = jj_scanpos = token;
	boolean retval = !jj_3_5();
	jj_save(4, xla);
	return retval;
  }  

  final private boolean jj_2_6(int xla) {
	jj_la = xla; jj_lastpos = jj_scanpos = token;
	boolean retval = !jj_3_6();
	jj_save(5, xla);
	return retval;
  }  

  final private boolean jj_2_7(int xla) {
	jj_la = xla; jj_lastpos = jj_scanpos = token;
	boolean retval = !jj_3_7();
	jj_save(6, xla);
	return retval;
  }  

  final private boolean jj_2_8(int xla) {
	jj_la = xla; jj_lastpos = jj_scanpos = token;
	boolean retval = !jj_3_8();
	jj_save(7, xla);
	return retval;
  }  

  final private boolean jj_3R_54() {
	if (jj_scan_token(String)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3_6() {
	if (jj_3R_6()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_44() {
	if (jj_scan_token(Equiv)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_18() {
	if (jj_3R_30()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_43() {
	if (jj_scan_token(Implies)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_19() {
	Token xsp;
	xsp = jj_scanpos;
	if (jj_3R_31()) {
	jj_scanpos = xsp;
	if (jj_3R_32()) {
	jj_scanpos = xsp;
	if (jj_3R_33()) {
	jj_scanpos = xsp;
	if (jj_3R_34()) {
	jj_scanpos = xsp;
	if (jj_3R_35()) {
	jj_scanpos = xsp;
	if (jj_3R_36()) {
	jj_scanpos = xsp;
	if (jj_3R_37()) {
	jj_scanpos = xsp;
	if (jj_3R_38()) {
	jj_scanpos = xsp;
	if (jj_3R_39()) {
	jj_scanpos = xsp;
	if (jj_3R_40()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_31() {
	if (jj_scan_token(Cons)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3_5() {
	if (jj_3R_7()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_42() {
	if (jj_scan_token(Or)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_17() {
	if (jj_3R_29()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_41() {
	if (jj_scan_token(And)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_27() {
	Token xsp;
	xsp = jj_scanpos;
	if (jj_3R_41()) {
	jj_scanpos = xsp;
	if (jj_3R_42()) {
	jj_scanpos = xsp;
	if (jj_3R_43()) {
	jj_scanpos = xsp;
	if (jj_3R_44()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_40() {
	if (jj_3R_54()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3_4() {
	if (jj_3R_9()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_16() {
	if (jj_3R_28()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_9() {
	if (jj_scan_token(Lparen)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	if (jj_3R_19()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_39() {
	if (jj_3R_53()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_26() {
	if (jj_scan_token(Not)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_15() {
	if (jj_3R_27()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_52() {
	if (jj_scan_token(Feasible)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_38() {
	if (jj_scan_token(Difference)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_22() {
	if (jj_scan_token(All)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_51() {
	if (jj_scan_token(Done)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_30() {
	Token xsp;
	xsp = jj_scanpos;
	if (jj_3R_51()) {
	jj_scanpos = xsp;
	if (jj_3R_52()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_21() {
	if (jj_scan_token(Any)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_14() {
	if (jj_3R_26()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_8() {
	Token xsp;
	xsp = jj_scanpos;
	if (jj_3R_14()) {
	jj_scanpos = xsp;
	if (jj_3R_15()) {
	jj_scanpos = xsp;
	if (jj_3R_16()) {
	jj_scanpos = xsp;
	if (jj_3R_17()) {
	jj_scanpos = xsp;
	if (jj_3R_18()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_12() {
	Token xsp;
	xsp = jj_scanpos;
	if (jj_3R_20()) {
	jj_scanpos = xsp;
	if (jj_3R_21()) {
	jj_scanpos = xsp;
	if (jj_3R_22()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_20() {
	if (jj_scan_token(Iota)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_37() {
	if (jj_scan_token(Intersection)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3_3() {
	if (jj_scan_token(Lparen)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	if (jj_3R_8()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_25() {
	if (jj_scan_token(SemiColon)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_50() {
	if (jj_scan_token(PersistentGoal)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_11() {
	if (jj_scan_token(Lparen)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	if (jj_scan_token(Set)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_49() {
	if (jj_scan_token(Intention)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_36() {
	if (jj_scan_token(Union)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_24() {
	if (jj_scan_token(Pipe)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_48() {
	if (jj_scan_token(Uncertainty)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_59() {
	if (jj_scan_token(Remainder)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_6() {
	if (jj_scan_token(Lparen)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	if (jj_3R_12()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_47() {
	if (jj_scan_token(Belief)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_29() {
	Token xsp;
	xsp = jj_scanpos;
	if (jj_3R_47()) {
	jj_scanpos = xsp;
	if (jj_3R_48()) {
	jj_scanpos = xsp;
	if (jj_3R_49()) {
	jj_scanpos = xsp;
	if (jj_3R_50()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_58() {
	if (jj_scan_token(Divide)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_35() {
	if (jj_scan_token(Append)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_57() {
	if (jj_scan_token(Multiply)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_13() {
	Token xsp;
	xsp = jj_scanpos;
	if (jj_3R_23()) {
	jj_scanpos = xsp;
	if (jj_3R_24()) {
	jj_scanpos = xsp;
	if (jj_3R_25()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_23() {
	if (jj_scan_token(Action)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_56() {
	if (jj_scan_token(Minus)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_46() {
	if (jj_scan_token(Exists)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_55() {
	if (jj_scan_token(Plus)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_53() {
	Token xsp;
	xsp = jj_scanpos;
	if (jj_3R_55()) {
	jj_scanpos = xsp;
	if (jj_3R_56()) {
	jj_scanpos = xsp;
	if (jj_3R_57()) {
	jj_scanpos = xsp;
	if (jj_3R_58()) {
	jj_scanpos = xsp;
	if (jj_3R_59()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_10() {
	if (jj_scan_token(Lparen)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	if (jj_scan_token(Sequence)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_34() {
	if (jj_scan_token(Nth)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_45() {
	if (jj_scan_token(Forall)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_28() {
	Token xsp;
	xsp = jj_scanpos;
	if (jj_3R_45()) {
	jj_scanpos = xsp;
	if (jj_3R_46()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	} else if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3_8() {
	if (jj_3R_11()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3_2() {
	if (jj_3R_7()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_33() {
	if (jj_scan_token(Rest)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3_7() {
	if (jj_3R_10()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_7() {
	if (jj_scan_token(Lparen)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	if (jj_3R_13()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3_1() {
	if (jj_3R_6()) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  final private boolean jj_3R_32() {
	if (jj_scan_token(First)) return true;
	if (jj_la == 0 && jj_scanpos == jj_lastpos) return false;
	return false;
  }  

  public SLParserTokenManager token_source;
  ASCII_CharStream jj_input_stream;
  public Token token, jj_nt;
  private int jj_ntk;
  private Token jj_scanpos, jj_lastpos;
  private int jj_la;
  public boolean lookingAhead = false;
  private boolean jj_semLA;
  private int jj_gen;
  final private int[] jj_la1 = new int[25];
  final private int[] jj_la1_0 = {0x1a0,0x1a0,0x1a0,0x3c00,0x7fc00000,0x3ffe00,0x1a0,0x1a0,0x20,0x7fc00000,0xc000,0xf0000,0x300000,0x0,0x0,0x0,0x20,0x80000000,0x20,0x0,0x0,0x0,0x0,0x20,0x20,};
  final private int[] jj_la1_1 = {0x400000,0x400000,0x400000,0x0,0x0,0x0,0x400000,0x400000,0x14600000,0x400000,0x0,0x0,0x0,0x200000,0x14400000,0x7000,0x14700000,0x400fff,0x14700000,0xf80,0x14400000,0x14000000,0xe0000,0x14600000,0x14600000,};
  final private int[] jj_la1_2 = {0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x4,0x0,0x0,0x0,0x0,0x0,0x4,0x0,0x4,0x0,0x4,0x0,0x4,0x0,0x0,0x4,0x4,};
  final private JJCalls[] jj_2_rtns = new JJCalls[8];
  private boolean jj_rescan = false;
  private int jj_gc = 0;

  public SLParser(java.io.InputStream stream) {
	jj_input_stream = new ASCII_CharStream(stream, 1, 1);
	token_source = new SLParserTokenManager(jj_input_stream);
	token = new Token();
	jj_ntk = -1;
	jj_gen = 0;
	for (int i = 0; i < 25; i++) jj_la1[i] = -1;
	for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }  

  public void ReInit(java.io.InputStream stream) {
	jj_input_stream.ReInit(stream, 1, 1);
	token_source.ReInit(jj_input_stream);
	token = new Token();
	jj_ntk = -1;
	jj_gen = 0;
	for (int i = 0; i < 25; i++) jj_la1[i] = -1;
	for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }  

  public SLParser(java.io.Reader stream) {
	jj_input_stream = new ASCII_CharStream(stream, 1, 1);
	token_source = new SLParserTokenManager(jj_input_stream);
	token = new Token();
	jj_ntk = -1;
	jj_gen = 0;
	for (int i = 0; i < 25; i++) jj_la1[i] = -1;
	for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }  

  public void ReInit(java.io.Reader stream) {
	jj_input_stream.ReInit(stream, 1, 1);
	token_source.ReInit(jj_input_stream);
	token = new Token();
	jj_ntk = -1;
	jj_gen = 0;
	for (int i = 0; i < 25; i++) jj_la1[i] = -1;
	for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }  

  public SLParser(SLParserTokenManager tm) {
	token_source = tm;
	token = new Token();
	jj_ntk = -1;
	jj_gen = 0;
	for (int i = 0; i < 25; i++) jj_la1[i] = -1;
	for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }  

  public void ReInit(SLParserTokenManager tm) {
	token_source = tm;
	token = new Token();
	jj_ntk = -1;
	jj_gen = 0;
	for (int i = 0; i < 25; i++) jj_la1[i] = -1;
	for (int i = 0; i < jj_2_rtns.length; i++) jj_2_rtns[i] = new JJCalls();
  }  

  final private Token jj_consume_token(int kind) throws ParseException {
	Token oldToken;
	if ((oldToken = token).next != null) token = token.next;
	else token = token.next = token_source.getNextToken();
	jj_ntk = -1;
	if (token.kind == kind) {
	  jj_gen++;
	  if (++jj_gc > 100) {
		jj_gc = 0;
		for (int i = 0; i < jj_2_rtns.length; i++) {
		  JJCalls c = jj_2_rtns[i];
		  while (c != null) {
			if (c.gen < jj_gen) c.first = null;
			c = c.next;
		  }
		}
	  }
	  return token;
	}
	token = oldToken;
	jj_kind = kind;
	throw generateParseException();
  }  

  final private boolean jj_scan_token(int kind) {
	if (jj_scanpos == jj_lastpos) {
	  jj_la--;
	  if (jj_scanpos.next == null) {
		jj_lastpos = jj_scanpos = jj_scanpos.next = token_source.getNextToken();
	  } else {
		jj_lastpos = jj_scanpos = jj_scanpos.next;
	  }
	} else {
	  jj_scanpos = jj_scanpos.next;
	}
	if (jj_rescan) {
	  int i = 0; Token tok = token;
	  while (tok != null && tok != jj_scanpos) { i++; tok = tok.next; }
	  if (tok != null) jj_add_error_token(kind, i);
	}
	return (jj_scanpos.kind != kind);
  }  

  final public Token getNextToken() {
	if (token.next != null) token = token.next;
	else token = token.next = token_source.getNextToken();
	jj_ntk = -1;
	jj_gen++;
	return token;
  }  

  final public Token getToken(int index) {
	Token t = lookingAhead ? jj_scanpos : token;
	for (int i = 0; i < index; i++) {
	  if (t.next != null) t = t.next;
	  else t = t.next = token_source.getNextToken();
	}
	return t;
  }  

  final private int jj_ntk() {
	if ((jj_nt=token.next) == null)
	  return (jj_ntk = (token.next=token_source.getNextToken()).kind);
	else
	  return (jj_ntk = jj_nt.kind);
  }  

  private java.util.Vector jj_expentries = new java.util.Vector();
  private int[] jj_expentry;
  private int jj_kind = -1;
  private int[] jj_lasttokens = new int[100];
  private int jj_endpos;

  private void jj_add_error_token(int kind, int pos) {
	if (pos >= 100) return;
	if (pos == jj_endpos + 1) {
	  jj_lasttokens[jj_endpos++] = kind;
	} else if (jj_endpos != 0) {
	  jj_expentry = new int[jj_endpos];
	  for (int i = 0; i < jj_endpos; i++) {
		jj_expentry[i] = jj_lasttokens[i];
	  }
	  boolean exists = false;
	  for (java.util.Enumeration enum = jj_expentries.elements(); enum.hasMoreElements();) {
		int[] oldentry = (int[])(enum.nextElement());
		if (oldentry.length == jj_expentry.length) {
		  exists = true;
		  for (int i = 0; i < jj_expentry.length; i++) {
			if (oldentry[i] != jj_expentry[i]) {
			  exists = false;
			  break;
			}
		  }
		  if (exists) break;
		}
	  }
	  if (!exists) jj_expentries.addElement(jj_expentry);
	  if (pos != 0) jj_lasttokens[(jj_endpos = pos) - 1] = kind;
	}
  }  

  final public ParseException generateParseException() {
	jj_expentries.removeAllElements();
	boolean[] la1tokens = new boolean[76];
	for (int i = 0; i < 76; i++) {
	  la1tokens[i] = false;
	}
	if (jj_kind >= 0) {
	  la1tokens[jj_kind] = true;
	  jj_kind = -1;
	}
	for (int i = 0; i < 25; i++) {
	  if (jj_la1[i] == jj_gen) {
		for (int j = 0; j < 32; j++) {
		  if ((jj_la1_0[i] & (1<<j)) != 0) {
			la1tokens[j] = true;
		  }
		  if ((jj_la1_1[i] & (1<<j)) != 0) {
			la1tokens[32+j] = true;
		  }
		  if ((jj_la1_2[i] & (1<<j)) != 0) {
			la1tokens[64+j] = true;
		  }
		}
	  }
	}
	for (int i = 0; i < 76; i++) {
	  if (la1tokens[i]) {
		jj_expentry = new int[1];
		jj_expentry[0] = i;
		jj_expentries.addElement(jj_expentry);
	  }
	}
	jj_endpos = 0;
	jj_rescan_token();
	jj_add_error_token(0, 0);
	int[][] exptokseq = new int[jj_expentries.size()][];
	for (int i = 0; i < jj_expentries.size(); i++) {
	  exptokseq[i] = (int[])jj_expentries.elementAt(i);
	}
	return new ParseException(token, exptokseq, tokenImage);
  }  

  final public void enable_tracing() {
  }  

  final public void disable_tracing() {
  }  

  final private void jj_rescan_token() {
	jj_rescan = true;
	for (int i = 0; i < 8; i++) {
	  JJCalls p = jj_2_rtns[i];
	  do {
		if (p.gen > jj_gen) {
		  jj_la = p.arg; jj_lastpos = jj_scanpos = p.first;
		  switch (i) {
			case 0: jj_3_1(); break;
			case 1: jj_3_2(); break;
			case 2: jj_3_3(); break;
			case 3: jj_3_4(); break;
			case 4: jj_3_5(); break;
			case 5: jj_3_6(); break;
			case 6: jj_3_7(); break;
			case 7: jj_3_8(); break;
		  }
		}
		p = p.next;
	  } while (p != null);
	}
	jj_rescan = false;
  }  

  final private void jj_save(int index, int xla) {
	JJCalls p = jj_2_rtns[index];
	while (p.gen > jj_gen) {
	  if (p.next == null) { p = p.next = new JJCalls(); break; }
	  p = p.next;
	}
	p.gen = jj_gen + xla - jj_la; p.first = token; p.arg = xla;
  }  

  static final class JJCalls {
	int gen;
	Token first;
	int arg;
	JJCalls next;
  }

  }