//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.util;

import java.io.Serializable;
import java.util.Comparator;

/**
 * Contains constant definitons of Comparators (primarily used for JDK1.1
 * compatibility).
 */
public class Comparators
{
	public static final Comparator STRING_COMPARE = new StringCompare();
	public static final Comparator OBJECT_COMPARE = new ObjectCompare();	
	
	private static class StringCompare implements Comparator, Serializable
	{
		public int compare(Object o1, Object o2)
		{
			if( o1 == null )
			{
				return -1;
			}
			if( o2 == null )
			{
				return 1;
			}
			String s1 = o1.toString();
			String s2 = o2.toString();
			int len1 = s1.length();
			int len2 = s2.length();
			for (int i=0, n=Math.min(len1, len2); i<n; i++)
			{
				char c1 = s1.charAt(i);
				char c2 = s2.charAt(i);
				if (c1 != c2)
				{
					return c1 - c2;
				}
			}
			return len1 - len2;
		}
	}
	
	private static class ObjectCompare implements Comparator, Serializable
	{
		public int compare(Object o1, Object o2)
		{
			if( o1 == null )
			{
				return -1;
			}
			if( o2 == null )
			{
				return 1;
			}
			String s1 = o1.toString();
			String s2 = o2.toString();
			int len1 = s1.length();
			int len2 = s2.length();
			for (int i=0, n=Math.min(len1, len2); i<n; i++)
			{
				char c1 = s1.charAt(i);
				char c2 = s2.charAt(i);
				if (c1 != c2)
				{
					return c1 - c2;
				}
			}
			return len1 - len2;
		}
	}	
	
}