//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files ("the
//     Programs") are Copyright (c) 1999 - 2000 Nortel Networks Corporation, 
//     8200 Dixie Road, Suite 100, Brampton, Ontario, Canada L6R 5P6.  
//     All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out in
//     the Nortel Networks FIPA-OS Public License, a copy of which can be found
//     in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version can
//     also be found at http://fipa-os.sourceforge.net/. By downloading the
//     files containing the Programs you accept the terms and  conditions of the
//     Public License. You do not have to accept these terms and conditions, but
//     unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.util;

//----------------------------------------------------------------------

import java.util.Vector;

/**
 * This is a simple queue which provides blocking and non-blocking
 * methods for getting the next element in the queue
 * <br>
 * <br> 
 **/

public class LIFOQueue extends Queue
{	
	/**
	 * Creates a LIFOQueue with the given maximum size
	 *
	 * @param max_size The most elements that can be contained within the
	 *                 queue at once, or -1 for no limit
	 */
	public LIFOQueue( int max_size )
	{
		super( max_size );	
	}


	/**
	 * Create a LIFOQueue with no maximum size
	 */
	public LIFOQueue()
	{
		super( -1 );	
	}


	/**
	 * Get the next message from the queue.  If
	 * the queue is empty, the thread will block
	 * until a message is added.
	 *
	 * @return The next message in the queue, or null
	 *					if the thread is interrupted
	 */
	protected synchronized Object getNextItem()
	{
		if( _queue.isEmpty() )
		{
			return null;
		}
		else
		{
			Object next = _queue.lastElement();
			_queue.removeElement( next );
			return next;
		}
	}
}