//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.util;

//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

//----------------------------------------------------------------------

  /**
   * Extends UTCContainer - contains a duration specified in relative UTC
   * format (e.g. 1 hour = "+00000000T010000000")
   * <br>
   * <br>
   */

public class UTCDuration extends UTCContainer {
	 
//----------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// STATIC VARIABLES
//----------------------------------------------------------------------
		
//----------------------------------------------------------------------
// INSTANCE VARIABLES
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// CONSTRUCTOR
//----------------------------------------------------------------------

	/**
	 * Constructs a UTCDuration based upon a string based upon the format
	 * +YYYYMMDDTHHmmSSuuu, where Y = year, M = month, D = date, H = hour
	 * m = minute, S = seconds and u = milliseconds
	 *
	 * @param duration Duration specified in relative UTC format
	 */
	public UTCDuration( String duration ) 
	{
		// Initialise date storage array
		_use_seconds = false;
		_use_millis = false;

		
		// Parse the string for the elements it contains
		_date_data[YEAR] = Integer.parseInt( duration.substring( 1, 5 ) );
		_date_data[MONTH] = Integer.parseInt( duration.substring( 5, 7 ) );
		_date_data[DATE] = Integer.parseInt( duration.substring( 7, 9) );
		_date_data[HOUR] = Integer.parseInt( duration.substring( 10, 12) );
		_date_data[MINUTE] = Integer.parseInt( duration.substring( 12, 14) );
		
		try
		{
			// Attempt to get seconds and millis
			_date_data[SECOND] = Integer.parseInt( duration.substring( 14, 16) );

			// Got seconds OK
			_use_seconds = true;
			
			_date_data[MILLISECOND] = Integer.parseInt( duration.substring( 16, 19) );

			// Got milliseconds OK
			_use_millis = true;
		}
		catch( Throwable e )
		{
			// Catch any parsing exceptions (i.e. seconds and milliseconds not specified)
		}
	}
	
	
	/**
	 * Constructs a UTCDuration based upon the values given
	 */
	public UTCDuration( int year, int month, int date, int hour,
											int minute, int second, int millisecond )
	{
		super( year, month, date, hour, minute, second, millisecond );		
		validate();			
	}
	

//----------------------------------------------------------------------
// PUBLIC METHODS
//----------------------------------------------------------------------

	/**
	 * Overrides UTCContainer.toString()
	 *
	 * @return String representation of this UTCDuration
	 */
	public String toString()	{
		
		String out_string = "+" + super.toString();
		
		return out_string;
	}
	
	/**
	 * Returns the number of milliseconds this duration represents - doesn't
	 * use the month or year field since months and years can be a variable
	 * length
	 *
	 * @result The duration in millis
	 */
	public long getDurationInMillis()	{
		
		// Accumulate the time in millis
		long millis = _date_data[MILLISECOND];
		millis += _date_data[SECOND] * 1000;
		millis += _date_data[MINUTE] * 60000;
		millis += _date_data[HOUR] * 3600000;
		millis += _date_data[DATE] * 3600000 * 24;
		
		return millis;
	}
	
	public void addDuration( String duration )
	{
		
		// Initialise date storage array
		_use_seconds = false;
		_use_millis = false;

		
		// Parse the string for the elements it contains
		_date_data[YEAR] += Integer.parseInt( duration.substring( 1, 5 ) );
		_date_data[MONTH] += Integer.parseInt( duration.substring( 5, 7 ) );
		_date_data[DATE] += Integer.parseInt( duration.substring( 7, 9) );
		_date_data[HOUR] += Integer.parseInt( duration.substring( 10, 12) );
		_date_data[MINUTE] += Integer.parseInt( duration.substring( 12, 14) );
		
		try
		{
			// Attempt to get seconds and millis
			_date_data[SECOND] += Integer.parseInt( duration.substring( 14, 16) );

			// Got seconds OK
			_use_seconds = true;
			
			_date_data[MILLISECOND] += Integer.parseInt( duration.substring( 16, 19) );

			// Got milliseconds OK
			_use_millis = true;
		}
		catch( Throwable e )
		{
			// Catch any parsing exceptions (i.e. seconds and milliseconds not specified)
		}		
		
		validate();
		
	}
	
	public void addDuration( int year, int month, int date, int hour,
											int minute, int second, int millis )
	{
	
		// Store the values given
		_date_data[YEAR] += year;
		_date_data[MONTH] += month;
		_date_data[DATE] += date;
		_date_data[HOUR] += hour;
		_date_data[MINUTE] += minute;
		
		if( second == -1 )
		{
			_use_seconds = false;
			_use_millis = false;
		}
		else
		{
			_date_data[SECOND] += second;
		}

		if( millis == -1 )
		{
			_use_millis = false;
		}
		else if( _use_millis )
		{
			_date_data[MILLISECOND] += millis;
		}	
		
		validate();
												
	}
	
	
//----------------------------------------------------------------------
// PRIVATE METHODS
//----------------------------------------------------------------------

	/**
	 * Method to normalise the contents of the UTCDuration object	 
	 */
	public void validate() {
		
		
		int days_per_month[] = {
														31, 29, 31, //start on leap year
														30, 31, 30,
														31, 31, 30,
														31, 30, 31
														};
														
		// Validate milliseconds
		int value = _date_data[MILLISECOND];
		_date_data[MILLISECOND] = _date_data[MILLISECOND] % 1000;
		_date_data[SECOND] += (value - _date_data[MILLISECOND]) / 1000;
		
		// Validate seconds
		value = _date_data[SECOND];
		_date_data[SECOND] = _date_data[SECOND] % 60;
		_date_data[MINUTE] += (value - _date_data[SECOND]) / 60;

		// Validate minutes
		value = _date_data[MINUTE];
		_date_data[MINUTE] = _date_data[MINUTE] % 60;
		_date_data[HOUR] += (value - _date_data[MINUTE]) / 60;

		// Validate hours
		value = _date_data[HOUR];
		_date_data[HOUR] = _date_data[HOUR] % 24;
		_date_data[DATE] += (value - _date_data[HOUR]) / 24;

		// Validate date
		int month = 0;
		int year = 0;
		
		while (_date_data[DATE] > days_per_month[month]) {
			
			if (month == 12) {
				
				month = 0;
				year++;
				_date_data[YEAR]++;
				
				days_per_month[1] = 28; // assume not leap year, 28 days in feb

				if( (year % 4 == 0) && 	// if year divisible by   4
						(year % 100 != 0) 	// and not divisible by 100
					) { 
						
						days_per_month[1] = 29; // leap year, 29 days in feb
							
				}
				
			} // end if new year			
			
			_date_data[DATE] -= days_per_month[month++];

			_date_data[MONTH]++;
		}				

		// Validate month
		value = _date_data[MONTH];
		_date_data[MONTH] = _date_data[MONTH] % 12;
		_date_data[YEAR] += (value - _date_data[MONTH]) / 12;
		
		// Validate year
		_date_data[YEAR] = _date_data[YEAR] % 10000;
		
	} // end validate()

	private int getIssue(String name) {
		
		String issue_string = name.substring(name.indexOf('_' ) + 1);
		int issue = java.lang.Integer.getInteger(issue_string).intValue();
		
		return issue;
	}
	
	public static void main(String args[]) {
		
		UTCDuration test = new UTCDuration(0, 0, 0, 25, 61, 34, 2000);
		
		fipaos.util.DIAGNOSTICS.println("Duration: " + test);
		
		long dur = test.getDurationInMillis();
		
		fipaos.util.DIAGNOSTICS.println("test millis: " + dur + " int value: " + (int)dur);
		
		UTCDuration test2 = new UTCDuration(0, 0, 0, 0, 0, 0, (int)dur);
		
		fipaos.util.DIAGNOSTICS.println("Duration2: " + test2);
		
		test2.addDuration( 0, 0, 0, 0, 30, 0, 0 );
		
		fipaos.util.DIAGNOSTICS.println("Duration2: " + test2);
		
		test2.addDuration( "+00000000T031300000" );
		
		fipaos.util.DIAGNOSTICS.println("Duration2: " + test2);

		
	} // end main
	
} // end class