package ranab.io;

import java.io.File;

/**
 * Do something on files recursively.<br/>
 * Assumption : root exists.
 * 
 * @author <a href="mailto:rana_b@yahoo.com">Rana Bhattacharyya</a>
 */

public class FileNavigator implements Runnable {
    
    private File mTopFile;
    private FileAction mAction; 
    private boolean mbStopRequest;
    
    /**
     * Constructors.
     * @param top root directory
     * @action action to be done on each file.
     */
    public FileNavigator(File top, FileAction action) {
        if( (top == null) || (action == null) ) {
            throw new NullPointerException();
        }
        mTopFile = top;
        mAction = action;
    }
    
    
    /**
     * Start navigating the directory tree.
     */
    public void navigate() {
        run();
    } 
    
    
    /**
     * Thread starting point.
     */
    public void run() {
       
       // check top file object 
       if(!mTopFile.exists()) {
           return;
       }
       mAction.startAction();
       navigateFile(mTopFile);
       mAction.endAction();
    }
    
    /**
     * Recursively touch files.
     */
    private void navigateFile(File fl) {
        
        // stop request
        if(mbStopRequest) {
            return;
        }
        
        // touch children recursively
        if(fl.isDirectory()) {
            File fileList[] = fl.listFiles();
            for(int i=0; i<fileList.length; i++) {
                if(mbStopRequest) {
                    return;
                }
                navigateFile(fileList[i]);
            }
        }
        
        // now touct the current file
        mAction.actionPerformed(fl);
    }
    
    /**
     * Stop navigation.
     */
    public void stopNavigation() {
        mbStopRequest = true;
    }
}