package ranab.unit;

/**
 * Pressure unit converter class.
 *
 * @author <a href="mailto:rana_b@yahoo.com">Rana Bhattacharyya</a>
 */
public 
class PressureUnit extends BaseUnit {
    
    private final static String[] UNITS = {
        "atm (kgf/cm)", 
        "atm (std)", 
        "bar", 
        "cm Hg (0C)",
        "cm water (4C)",
        "dyne/cm",
        "ft water (39.2F)",
        "gf/cm",
        "in Hg (32F)",
        "in Hg (60F)",
        "in water (39.2F)",
        "in water (60F)",
        "kgf/cm",
        "kgf/m",
        "kgf/mm",
        "kPa",
        "kip/in",
        "ksi",
        "micron ()",
        "mbar",
        "mm Hg (0C)",
        "Pascal (Pa)",
        "poundal/ft",
        "lbf/ft",
        "lbf/in",
        "psi",
        "torr (mm Hg 0C)"    
    }; 
    
    // base weight unit is Pascal
    private final static double[] PASCAL_TABLE = {
        1.0197162e-05,
        9.8692327e-06,
        1e-05,
        0.00075006376,
        0.010197443,
        10,
        0.00033456229,
        0.010197162,
        0.00029530059,
        0.00029613397,
        0.0040147421,
        0.0040186465,
        1.0197162e-05,
        0.10197162,
        1.0197162e-07,
        0.001,
        1.4503774e-07,
        1.4503774e-07,
        7.5006376,
        0.01,
        0.0075006376,
        1,
        0.67196895,
        0.0208854,
        0.0001450377,
        0.0001450377,
        0.0075006    
    };
    
    /**
     * Get all the possible units.
     */
    public String[] getAllUnits() {
        return UNITS;
    }
    
    /**
     * Convert one unit from another
     */
    public String convertUnit(String value, int from, int to) throws UnitException {
        double inVal = getDoubleValue(value);
        double outVal = inVal*PASCAL_TABLE[to]/PASCAL_TABLE[from];
        return String.valueOf(outVal);
    }
    
    /**
     * Return unit type.
     */
    public String toString() {
        return "Pressure";
    }
    
}