package ranab.unit;

/**
 * Temperature unit converter class.
 * 
 * @author <a href="mailto:rana_b@yahoo.com">Rana Bhattacharyya</a>
 */
public 
class TemperatureUnit extends BaseUnit {
    
    // all temperature units (base unit is celsius)
    private final static String[] UNITS = {
        " Fahrenheit", 
        " Celsius", 
        "Kelvin", 
    }; 
    
    private final static int FAHRENHEIT = 0;
    private final static int CELSIUS    = 1;
    private final static int KELVIN     = 2;
        
    
    /**
     * Get all the possible units.
     */
    public String[] getAllUnits() {
        return UNITS;
    }
    
    /**
     * Convert one unit from another
     */
    public String convertUnit(String value, int from, int to) throws UnitException {
        double val = getDoubleValue(value);
        
        // first convert it to celsius
        switch(from) {
          case FAHRENHEIT:
              val = (val - 32.0)/1.8;
              break;
              
          case CELSIUS:
              break;
              
          case KELVIN:
              val = val - 273.15;
              break;
        }
        
        // now convert it to the desired unit
        switch(to) {
            case FAHRENHEIT:
                val = val*1.8 + 32;
                break;
                
            case CELSIUS:
                break;
                
            case KELVIN:
                val = val + 273.15;
                
        }
        return String.valueOf(val);
    }
    
    /**
     * Return unit type.
     */
    public String toString() {
        return "Temperature";
    }
    
}